#include "cppdefs.h"
      MODULE ad_step3d_t_mod
#if !defined TS_FIXED && (defined ADJOINT && defined SOLVE3D)
!
!=======================================================================
!  Copyright (c) 2005 ROMS/TOMS Adjoint Group                          !
!================================================== Hernan G. Arango ===
!                                                                      !
!  This routine time-steps adjoint tracer equations.  Notice that      !
!  advective and diffusive terms are time-stepped differently.         !
!                                                                      !
!=======================================================================
!
      implicit none

      PRIVATE
      PUBLIC  :: ad_step3d_t

      CONTAINS
!
!***********************************************************************
      SUBROUTINE ad_step3d_t (ng, tile)
!***********************************************************************
!
      USE mod_param
# ifdef CLIMATOLOGY
      USE mod_clima
# endif
# ifdef DIAGNOSTICS
!!    USE mod_diags
# endif
      USE mod_grid
      USE mod_mixing
# if defined ASSIMILATION || defined NUDGING
      USE mod_obs
# endif
      USE mod_ocean
# ifdef TS_PSOURCE
      USE mod_sources
# endif
      USE mod_stepping
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, tile
!
!  Local variable declarations.
!
# include "tile.h"
!
# ifdef PROFILE
      CALL wclock_on (ng, iADM, 35)
# endif
      CALL ad_step3d_t_tile (ng, Istr, Iend, Jstr, Jend,                &
     &                       LBi, UBi, LBj, UBj,                        &
     &                       nrhs(ng), nstp(ng), nnew(ng),              &
# ifdef TS_PSOURCE
     &                       Nsrc(ng),                                  &
     &                       SOURCES(ng) % Lsrc,                        &
     &                       SOURCES(ng) % Isrc,                        &
     &                       SOURCES(ng) % Jsrc,                        &
     &                       SOURCES(ng) % Dsrc,                        &
     &                       SOURCES(ng) % Tsrc,                        &
# endif
# ifdef MASKING
     &                       GRID(ng) % rmask,                          &
     &                       GRID(ng) % umask,                          &
     &                       GRID(ng) % vmask,                          &
# endif
# ifdef TS_MPDATA
     &                       GRID(ng) % omn,                            &
     &                       GRID(ng) % om_u,                           &
     &                       GRID(ng) % om_v,                           &
     &                       GRID(ng) % on_u,                           &
     &                       GRID(ng) % on_v,                           &
# endif
     &                       GRID(ng) % pm,                             &
     &                       GRID(ng) % pn,                             &
     &                       GRID(ng) % Hz,                             &
     &                       GRID(ng) % ad_Hz,                          &
     &                       GRID(ng) % Huon,                           &
     &                       GRID(ng) % ad_Huon,                        &
     &                       GRID(ng) % Hvom,                           &
     &                       GRID(ng) % ad_Hvom,                        &
     &                       GRID(ng) % z_r,                            &
     &                       GRID(ng) % ad_z_r,                         &
# if defined TCLM_NUDGING && defined TCLIMATOLOGY
     &                       CLIMA(ng) % Tnudgcof,                      &
     &                       CLIMA(ng) % tclm,                          &
# endif
# if defined NUDGING_SST || defined NUDGING_T
!!   &                       OBS(ng) % EobsT,                           &
!!   &                       OBS(ng) % Tobs,                            &
# endif
     &                       MIXING(ng) % Akt,                          &
     &                       MIXING(ng) % ad_Akt,                       &
# ifdef TS_MPDATA
     &                       OCEAN(ng) % u,                             &
     &                       OCEAN(ng) % ad_u,                          &
     &                       OCEAN(ng) % v,                             &
     &                       OCEAN(ng) % ad_v,                          &
# endif
     &                       OCEAN(ng) % W,                             &
     &                       OCEAN(ng) % ad_W,                          &
# if defined FLOATS_NOT_YET && defined FLOAT_VWALK
     &                       MIXING(ng) % dAktdz,                       &
# endif
# ifdef DIAGNOSTICS_TS
!!   &                       DIAGS(ng) % DiaTwrk,                       &
# endif
     &                       OCEAN(ng) % t,                             &
     &                       OCEAN(ng) % ad_t_sol,                      &
     &                       OCEAN(ng) % ad_t)
# ifdef PROFILE
      CALL wclock_off (ng, iADM, 35)
# endif
      RETURN
      END SUBROUTINE ad_step3d_t
!
!***********************************************************************
      SUBROUTINE ad_step3d_t_tile (ng, Istr, Iend, Jstr, Jend,          &
     &                             LBi, UBi, LBj, UBj,                  &
     &                             nrhs, nstp, nnew,                    &
# ifdef TS_PSOURCE
     &                             Nsrc, Lsrc, Isrc, Jsrc, Dsrc,        &
     &                             Tsrc,                                &
# endif
# ifdef MASKING
     &                             rmask, umask, vmask,                 &
# endif
# ifdef TS_MPDATA
     &                             omn, om_u, om_v, on_u, on_v,         &
# endif
     &                             pm, pn,                              &
     &                             Hz, ad_Hz,                           &
     &                             Huon, ad_Huon,                       &
     &                             Hvom, ad_Hvom,                       &
     &                             z_r, ad_z_r,                         &
# if defined TCLM_NUDGING && defined TCLIMATOLOGY
     &                             Tnudgcof, tclm,                      &
# endif
# if defined NUDGING_SST || defined NUDGING_T
!!   &                             EobsT, Tobs,                         &
# endif
     &                             Akt, ad_Akt,                         &
# ifdef TS_MPDATA
     &                             u, ad_u,                             &
     &                             v, ad_v,                             &
# endif
     &                             W, ad_W,                             &
# if defined FLOATS_NOT_YET && defined FLOAT_VWALK
     &                             dAktdz,                              &
# endif
# ifdef DIAGNOSTICS_TS
!!   &                             DiaTwrk,                             &
# endif
     &                             t,                                   &
     &                             ad_t_sol, ad_t)
!***********************************************************************
!
      USE mod_param
      USE mod_ncparam
      USE mod_scalars
!
# if defined EW_PERIODIC || defined NS_PERIODIC
      USE ad_exchange_3d_mod, ONLY : ad_exchange_r3d_tile
# endif
# ifdef DISTRIBUTE
      USE mp_exchange_mod, ONLY : ad_mp_exchange4d
# endif
# ifdef TS_MPDATA
      USE ad_mpdata_adiff_mod
# endif
      USE ad_t3dbc_mod, ONLY : ad_t3dbc_tile
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, Iend, Istr, Jend, Jstr
      integer, intent(in) :: LBi, UBi, LBj, UBj
      integer, intent(in) :: nrhs, nstp, nnew
!
# ifdef ASSUMED_SHAPE
#  ifdef TS_PSOURCE
      integer, intent(in) :: Nsrc

      logical, intent(in) :: Lsrc(:,:)

      integer, intent(in) :: Isrc(:)
      integer, intent(in) :: Jsrc(:)

      real(r8), intent(in) :: Dsrc(:)
      real(r8), intent(in) :: Tsrc(:,:,:)
#  endif
#  ifdef MASKING
      real(r8), intent(in) :: rmask(LBi:,LBj:)
      real(r8), intent(in) :: umask(LBi:,LBj:)
      real(r8), intent(in) :: vmask(LBi:,LBj:)
#  endif
#  ifdef TS_MPDATA
      real(r8), intent(in) :: omn(LBi:,LBj:)
      real(r8), intent(in) :: om_u(LBi:,LBj:)
      real(r8), intent(in) :: om_v(LBi:,LBj:)
      real(r8), intent(in) :: on_u(LBi:,LBj:)
      real(r8), intent(in) :: on_v(LBi:,LBj:)
#  endif
      real(r8), intent(in) :: pm(LBi:,LBj:)
      real(r8), intent(in) :: pn(LBi:,LBj:)
#  if defined TCLM_NUDGING && defined TCLIMATOLOGY
      real(r8), intent(in) :: Tnudgcof(LBi:,LBj:,:)
      real(r8), intent(in) :: tclm(LBi:,LBj:,:,:)
#  endif
#  if defined NUDGING_SST || defined NUDGING_T
!!    real(r8), intent(in) :: EobsT(LBi:,LBj:,:,:)
!!    real(r8), intent(in) :: Tobs(LBi:,LBj:,:,:)
#  endif
      real(r8), intent(in) :: Hz(LBi:,LBj:,:)
      real(r8), intent(in) :: Huon(LBi:,LBj:,:)
      real(r8), intent(in) :: Hvom(LBi:,LBj:,:)
      real(r8), intent(in) :: z_r(LBi:,LBj:,:)
#  ifdef SUN
      real(r8), intent(in) :: Akt(LBi:UBi,LBj:UBj,0:N(ng),NAT)
      real(r8), intent(in) :: t(LBi:UBi,LBj:UBj,N(ng),3,NT(ng))
#  else
      real(r8), intent(in) :: Akt(LBi:,LBj:,0:,:)
      real(r8), intent(in) :: t(LBi:,LBj:,:,:,:)
#  endif
#  ifdef TS_MPDATA
      real(r8), intent(in) :: u(LBi:,LBj:,:,:)
      real(r8), intent(in) :: v(LBi:,LBj:,:,:)
#  endif
      real(r8), intent(in) :: W(LBi:,LBj:,0:)

#  ifdef DIAGNOSTICS_TS
!!    real(r8), intent(inout) :: DiaTwrk(LBi:,LBj:,:,:,:)
#  endif
      real(r8), intent(inout) :: ad_Hz(LBi:,LBj:,:)
      real(r8), intent(inout) :: ad_Huon(LBi:,LBj:,:)
      real(r8), intent(inout) :: ad_Hvom(LBi:,LBj:,:)
      real(r8), intent(inout) :: ad_z_r(LBi:,LBj:,:)
#  ifdef SUN
      real(r8), intent(inout) :: ad_Akt(LBi:UBi,LBj:UBj,0:N(ng),NAT)
#  else
      real(r8), intent(inout) :: ad_Akt(LBi:,LBj:,0:,:)
#  endif
#  ifdef TS_MPDATA
      real(r8), intent(inout) :: ad_u(LBi:,LBj:,:,:)
      real(r8), intent(inout) :: ad_v(LBi:,LBj:,:,:)
#  endif
      real(r8), intent(inout) :: ad_W(LBi:,LBj:,0:)
#  ifdef SUN
      real(r8), intent(inout) :: ad_t(LBi:UBi,LBj:UBj,N(ng),3,NT(ng))
#  else
      real(r8), intent(inout) :: ad_t(LBi:,LBj:,:,:,:)
#  endif
#  if defined FLOATS_NOT_YET && defined FLOAT_VWALK
      real(r8), intent(out) :: dAktdz(LBi:,LBj:,:)
#  endif
      real(r8), intent(out) :: ad_t_sol(LBi:,LBj:,:,:)

# else

#  ifdef TS_PSOURCE
      integer, intent(in) :: Nsrc

      logical, intent(in) :: Lsrc(Msrc,NT(ng))

      integer, intent(in) :: Isrc(Msrc)
      integer, intent(in) :: Jsrc(Msrc)

      real(r8), intent(in) :: Dsrc(Msrc)
      real(r8), intent(in) :: Tsrc(Msrc,N(ng),NT(ng))
#  endif

#  ifdef MASKING
      real(r8), intent(in) :: rmask(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: umask(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: vmask(LBi:UBi,LBj:UBj)
#  endif
#  ifdef TS_MPDATA
      real(r8), intent(in) :: omn(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: om_u(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: om_v(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: on_u(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: on_v(LBi:UBi,LBj:UBj)
#  endif
      real(r8), intent(in) :: pm(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: pn(LBi:UBi,LBj:UBj)
#  if defined TCLM_NUDGING && defined TCLIMATOLOGY
      real(r8), intent(in) :: Tnudgcof(LBi:UBi,LBj:UBj,NT(ng))
      real(r8), intent(in) :: tclm(LBi:UBi,LBj:UBj,N(ng),NT(ng))
#  endif
#  if defined NUDGING_SST || defined NUDGING_T
!!    real(r8), intent(in) :: EobsT(LBi:UBi,LBj:UBj,N(ng),NT(ng))
!!    real(r8), intent(in) :: Tobs(LBi:UBi,LBj:UBj,N(ng),NT(ng))
#  endif
      real(r8), intent(in) :: Hz(LBi:UBi,LBj:UBj,N(ng))
      real(r8), intent(in) :: Huon(LBi:UBi,LBj:UBj,N(ng))
      real(r8), intent(in) :: Hvom(LBi:UBi,LBj:UBj,N(ng))
      real(r8), intent(in) :: z_r(LBi:UBi,LBj:UBj,N(ng))
      real(r8), intent(in) :: Akt(LBi:UBi,LBj:UBj,0:N(ng),NAT)
      real(r8), intent(in) :: t(LBi:UBi,LBj:UBj,N(ng),3,NT(ng))
#  ifdef TS_MPDATA
      real(r8), intent(in) :: u(LBi:UBi,LBj:UBj,N(ng),2)
      real(r8), intent(in) :: v(LBi:UBi,LBj:UBj,N(ng),2)
#  endif
      real(r8), intent(in) :: W(LBi:UBi,LBj:UBj,0:N(ng))

#  ifdef DIAGNOSTICS_TS
!!    real(r8), intent(inout) :: DiaTwrk(LBi:UBi,LBj:UBj,N(ng),NT(ng),  &
!!   &                                   NDT)
#  endif
      real(r8), intent(inout) :: ad_Hz(LBi:UBi,LBj:UBj,N(ng))
      real(r8), intent(inout) :: ad_Huon(LBi:UBi,LBj:UBj,N(ng))
      real(r8), intent(inout) :: ad_Hvom(LBi:UBi,LBj:UBj,N(ng))
      real(r8), intent(inout) :: ad_z_r(LBi:UBi,LBj:UBj,N(ng))
      real(r8), intent(inout) :: ad_Akt(LBi:UBi,LBj:UBj,0:N(ng),NAT)
#  ifdef TS_MPDATA
      real(r8), intent(inout) :: ad_u(LBi:UBi,LBj:UBj,N(ng),2)
      real(r8), intent(inout) :: ad_v(LBi:UBi,LBj:UBj,N(ng),2)
#  endif
      real(r8), intent(inout) :: ad_W(LBi:UBi,LBj:UBj,0:N(ng))
      real(r8), intent(inout) :: ad_t(LBi:UBi,LBj:UBj,N(ng),3,NT(ng))
#  if defined FLOATS_NOT_YET && defined FLOAT_VWALK
      real(r8), intent(out) :: dAktdz(LBi:UBi,LBj:UBj,N(ng))
#  endif
      real(r8), intent(inout) :: ad_t_sol(LBi:UBi,LBj:UBj,N(ng),NT(ng))
# endif
!
!  Local variable declarations.
!
# ifdef DISTRIBUTE
#  ifdef EW_PERIODIC
      logical :: EWperiodic=.TRUE.
#  else
      logical :: EWperiodic=.FALSE.
#  endif
#  ifdef NS_PERIODIC
      logical :: NSperiodic=.TRUE.
#  else
      logical :: NSperiodic=.FALSE.
#  endif
# endif
      integer :: IstrR, IendR, JstrR, JendR, IstrU, JstrV
      integer :: ILB, IUB, JLB, JUB
      integer :: i, is, itrc, j, k, ltrc
      integer :: idiag

      real(r8), parameter :: eps = 1.0E-16_r8

      real(r8) :: cff, cff1, cff2, cff3
      real(r8) :: ad_cff, ad_cff1, ad_cff2
      real(r8) :: adfac, adfac1, adfac2

      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: CF
      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: BC
      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: DC
# ifdef SPLINES
      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: DC1
# endif
      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: FC

      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: ad_CF
      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: ad_BC
      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: ad_DC
      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N(ng)) :: ad_FC

      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY) :: FE
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY) :: FX
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY) :: curv
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY) :: grad

      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY) :: ad_FE
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY) :: ad_FX
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY) :: ad_curv
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY) :: ad_grad

      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,N(ng)) :: oHz
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,N(ng)) :: ad_oHz
# ifdef TS_MPDATA
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,                     &
     &                    N(ng),NT(ng)) :: Ta
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,                     &
     &                    N(ng),NT(ng)) :: ad_Ta

      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,N(ng)) :: Ua
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,N(ng)) :: Va
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,0:N(ng)) :: Wa

      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,N(ng)) :: ad_Ua
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,N(ng)) :: ad_Va
      real(r8), dimension(PRIVATE_2D_SCRATCH_ARRAY,0:N(ng)) :: ad_Wa
# endif

# include "set_bounds.h"
!
      ILB=LBOUND(ad_FE,DIM=1)
      IUB=UBOUND(ad_FE,DIM=1)
      JLB=LBOUND(ad_FE,DIM=2)
      JUB=UBOUND(ad_FE,DIM=2)
!
!-----------------------------------------------------------------------
!  Initialize adjoint private variables.
!-----------------------------------------------------------------------
!
      ad_cff=0.0_r8
      ad_cff1=0.0_r8
      ad_cff2=0.0_r8
      DO j=JLB,JUB
        DO i=ILB,IUB
          ad_FE(i,j)=0.0_r8
          ad_FX(i,j)=0.0_r8
          ad_curv(i,j)=0.0_r8
          ad_grad(i,j)=0.0_r8
        END DO
        DO k=1,N(ng)
          DO i=ILB,IUB
            ad_oHz(i,j,k)=0.0_r8
          END DO
        END DO 
# ifdef TS_MPDATA
        DO k=1,N(ng)
          DO i=ILB,IUB
            ad_Ua(i,j,k)=0.0_r8
            ad_Va(i,j,k)=0.0_r8
          END DO
        END DO
        DO k=0,N(ng)
          DO i=ILB,IUB
            ad_Wa(i,j,k)=0.0_r8
          END DO
        END DO
        DO itrc=1,NT(ng)
          DO k=1,N(ng)
            DO i=ILB,IUB
              ad_Ta(i,j,k,itrc)=0.0_r8
            END DO
          END DO
        END DO
# endif
      END DO
      DO k=0,N(ng)
        DO i=ILB,IUB
          ad_CF(i,k)=0.0_r8
          ad_BC(i,k)=0.0_r8
          ad_DC(i,k)=0.0_r8
          ad_FC(i,k)=0.0_r8
        END DO
      END DO
!
!-----------------------------------------------------------------------
!  Apply adjoint lateral boundary conditions and, if appropriate, nudge
!  to tracer data and apply Land/Sea mask.
!-----------------------------------------------------------------------
!
# ifdef DISTRIBUTE
!  Exchange boundary data.
!
!>    CALL mp_exchange4d (ng, iTLM, 1, Istr, Iend, Jstr, Jend,          &
!>   &                    LBi, UBi, LBj, UBj, 1, N(ng), 1, NT(ng),      &
!>   &                    NghostPoints, EWperiodic, NSperiodic,         &
!>   &                    tl_t(:,:,:,nnew,:))
!>
      CALL ad_mp_exchange4d (ng, iADM, 1, Istr, Iend, Jstr, Jend,       &
     &                       LBi, UBi, LBj, UBj, 1, N(ng), 1, NT(ng),   &
     &                       NghostPoints, EWperiodic, NSperiodic,      &
     &                       ad_t(:,:,:,nnew,:))
!
# endif
      DO itrc=1,NT(ng)

# if defined EW_PERIODIC || defined NS_PERIODIC
!
!  Apply adjoint periodic boundary conditions.
!
!>      CALL exchange_r3d_tile (ng, Istr, Iend, Jstr, Jend,             &
!>   &                          LBi, UBi, LBj, UBj, 1, N(ng),           &
!>   &                          tl_t(:,:,:,nnew,itrc))
!>
        CALL ad_exchange_r3d_tile (ng, Istr, Iend, Jstr, Jend,          &
     &                             LBi, UBi, LBj, UBj, 1, N(ng),        &
     &                             ad_t(:,:,:,nnew,itrc))
# endif
!
!  Save adjoint solution for IO purpuses.
!
        DO k=1,N(ng)
          DO j=JstrR,JendR
            DO i=IstrR,IendR
              ad_t_sol(i,j,k,itrc)=ad_t(i,j,k,nnew,itrc)
            END DO
          END DO
        END DO

# ifdef DIAGNOSTICS_TS
!!
!!  Compute time-rate-of-change diagnostic term.
!!
!!      DO k=1,N(ng)
!!        DO j=JstrR,JendR
!!          DO i=IstrR,IendR
!!            DiaTwrk(i,j,k,itrc,iTrate)=t(i,j,k,nnew,itrc)-            &
!!   &                                   DiaTwrk(i,j,k,itrc,iTrate)
!!            DiaTwrk(i,j,k,itrc,iTrate)=t(i,j,k,nnew,itrc)-            &
!!   &                                   t(i,j,k,nstp,itrc)
!!          END DO
!!        END DO
!!      END DO
# endif
# ifdef MASKING
!
!  Apply Land/Sea mask.
!
        DO k=1,N(ng)
          DO j=JstrR,JendR
            DO i=IstrR,IendR
!>            tl_t(i,j,k,nnew,itrc)=tl_t(i,j,k,nnew,itrc)*rmask(i,j)
!>
              ad_t(i,j,k,nnew,itrc)=ad_t(i,j,k,nnew,itrc)*rmask(i,j)
            END DO
          END DO
        END DO
# endif
# if defined NUDGING_SST || defined NUDGING_T
!!
!! Assimilate tracer observations via nudging.
!!
!!      IF (update_T(itrc,ng)) THEN
!!        DO k=1,N(ng)
!!          DO j=JstrR,JendR
!!            DO i=IstrR,IendR
!!              cff=MIN(1.0_r8,MAX(0.0_r8,EobsT(i,j,k,itrc)))
!!              cff=dt(ng)*Tnudass(itrc,ng)*(1.0_r8-cff)
!!              t(i,j,k,nnew,itrc)=t(i,j,k,nnew,itrc)+                  &
!!   &                             cff*(Tobs(i,j,k,itrc)-               &
!!   &                                  t(i,j,k,nnew,itrc))
!!            END DO
!!          END DO
!!        END DO
!!      END IF
# endif
# if defined TCLM_NUDGING && defined TCLIMATOLOGY
!
!  Nudge towards tracer climatology.
!
        DO k=1,N(ng)
          DO j=JstrR,JendR
            DO i=IstrR,IendR
!>            tl_t(i,j,k,nnew,itrc)=tl_t(i,j,k,nnew,itrc)-              &
!>   &                              dt(ng)*Tnudgcof(i,j,itrc)*          &
!>   &                              tl_t(i,j,k,nnew,itrc)
!>
              ad_t(i,j,k,nnew,itrc)=ad_t(i,j,k,nnew,itrc)-              &
     &                              dt(ng)*Tnudgcof(i,j,itrc)*          &
     &                              ad_t(i,j,k,nnew,itrc)
            END DO
          END DO
        END DO
# endif
!
!  Set adjoint lateral boundary conditions.
!
!>      CALL tl_t3dbc_tile (ng, Istr, Iend, Jstr, Jend, itrc,           &
!>   &                      LBi, UBi, LBj, UBj, N(ng), NT(ng),          &
!>   &                      nstp, nnew,                                 &
!>   &                      tl_t)
!>
        CALL ad_t3dbc_tile (ng, Istr, Iend, Jstr, Jend, itrc,           &
     &                      LBi, UBi, LBj, UBj, N(ng), NT(ng),          &
     &                      nstp, nnew,                                 &
     &                      ad_t)
      END DO
!
!-----------------------------------------------------------------------
!  Time-step adjoint vertical diffusion term.
!-----------------------------------------------------------------------
!
!  Compute BASIC STATE inverse thickness.
!
# ifdef TS_MPDATA
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-3,Iend+2
#  else
#   define I_RANGE MAX(Istr-3,1),MIN(Iend+2,Lm(ng))
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-3,Jend+2
#  else
#   define J_RANGE MAX(Jstr-3,1),MIN(Jend+2,Mm(ng))
#  endif
# else
#  define I_RANGE Istr,Iend
#  define J_RANGE Jstr,Jend
# endif

      DO k=1,N(ng)
        DO j=J_RANGE
          DO i=I_RANGE
            oHz(i,j,k)=1.0_r8/Hz(i,j,k)
          END DO
        END DO
      END DO
# undef I_RANGE
# undef J_RANGE
!
!  Compute adjoint vertical diffusion term.
!
      DO j=Jstr,Jend
        DO itrc=1,NT(ng)
          ltrc=MIN(NAT,itrc)

# if defined SPLINES
!
!  Use conservative, parabolic spline reconstruction of BASIC STATE
!  vertical diffusion derivatives.  Solve BASIC STATE tridiagonal
!  system.
!
          cff1=1.0_r8/6.0_r8
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              FC(i,k)=cff1*Hz(i,j,k  )-                                 &
     &                dt(ng)*Akt(i,j,k-1,ltrc)*oHz(i,j,k  )
              CF(i,k)=cff1*Hz(i,j,k+1)-                                 &
     &                dt(ng)*Akt(i,j,k+1,ltrc)*oHz(i,j,k+1)
            END DO
          END DO
          DO i=Istr,Iend
            CF(i,0)=0.0_r8
            DC(i,0)=0.0_r8
          END DO
!
!  LU decomposition and forward substitution.
!
          cff1=1.0_r8/3.0_r8
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              BC(i,k)=cff1*(Hz(i,j,k)+Hz(i,j,k+1))+                     &
     &                dt(ng)*Akt(i,j,k,ltrc)*(oHz(i,j,k)+oHz(i,j,k+1))
              cff=1.0_r8/(BC(i,k)-FC(i,k)*CF(i,k-1))
              CF(i,k)=cff*CF(i,k)
              DC(i,k)=cff*(t(i,j,k+1,nnew,itrc)-t(i,j,k,nnew,itrc)-     &
     &                     FC(i,k)*DC(i,k-1))
            END DO
          END DO
!
!  Backward substitution. Save DC for the adjoint code below.
!
          DO i=Istr,Iend
            DC(i,N(ng))=0.0_r8
          END DO
          DO k=N(ng)-1,1,-1
            DO i=Istr,Iend
              DC(i,k)=DC(i,k)-CF(i,k)*DC(i,k+1)
            END DO
          END DO
!
!  Multiply DC by Akt and save it on DC1.
!
          DO k=0,N(ng)
            DO i=Istr,Iend
              DC1(i,k)=DC(i,k)*Akt(i,j,k,ltrc)
            END DO
          END DO
!
!  Time-step adjoint diffusion term.
!
!>        DO k=1,N(ng)
!>
          DO k=N(ng),1,-1
            DO i=Istr,Iend
#  ifdef DIAGNOSTICS_TS
!!            DiaTwrk(i,j,k,itrc,iTvdif)=DiaTwrk(i,j,k,itrc,iTvdif)+    &
!!   &                                   cff1
#  endif
!>            tl_t(i,j,k,nnew,itrc)=tl_t(i,j,k,nnew,itrc)+tl_cff1
!>
              ad_cff1=ad_cff1+ad_t(i,j,k,nnew,itrc)
!>            tl_cff1=dt(ng)*(tl_oHz(i,j,k)*(DC(i,k)-DC(i,k-1))+        &
!>   &                        oHz(i,j,k)*(tl_DC(i,k)-tl_DC(i,k-1)))
!>                                                       use DC1 instead
              adfac=dt(ng)*ad_cff1
              adfac1=adfac*oHz(i,j,k)
              ad_DC(i,k-1)=ad_DC(i,k-1)-adfac1
              ad_DC(i,k  )=ad_DC(i,k  )+adfac1
              ad_oHz(i,j,k)=ad_oHz(i,j,k)+(DC1(i,k)-DC1(i,k-1))*adfac
              ad_cff1=0.0_r8
!>            tl_DC(i,k)=tl_DC(i,k)*Akt(i,j,k,ltrc)+                    &
!>   &                   DC(i,k)*tl_Akt(i,j,k,ltrc)
!>                                                       use DC here
              ad_DC(i,k)=ad_DC(i,k)*Akt(i,j,k,ltrc)
              ad_Akt(i,j,k,ltrc)=ad_Akt(i,j,k,ltrc)+DC(i,k)*ad_DC(i,k)
            END DO
          END DO
!
!  Adjoint back substitution
!
          DO k=1,N(ng)-1
            DO i=Istr,Iend
!>            tl_DC(i,k)=tl_DC(i,k)-CF(i,k)*tl_DC(i,k+1)
!>
              ad_DC(i,k+1)=ad_DC(i,k+1)-CF(i,k)*ad_DC(i,k)
            END DO
          END DO
          DO i=Istr,Iend
!>          tl_DC(i,N(ng))=0.0_r8
!>
            ad_DC(i,N(ng))=0.0_r8
          END DO
!
!  Adjoint LU decomposition and forward substitution.
!
          cff1=1.0_r8/3.0_r8
          DO k=N(ng)-1,1,-1
            DO i=Istr,Iend
              BC(i,k)=cff1*(Hz(i,j,k)+Hz(i,j,k+1))+                     &
     &                dt(ng)*Akt(i,j,k,ltrc)*(oHz(i,j,k)+oHz(i,j,k+1))
              cff=1.0_r8/(BC(i,k)-FC(i,k)*CF(i,k-1))
!>            tl_DC(i,k)=cff*(tl_t(i,j,k+1,nnew,itrc)-                  &
!>   &                        tl_t(i,j,k  ,nnew,itrc)-                  &
!>   &                        (tl_FC(i,k)*DC(i,k-1)+                    &
!>   &                         tl_BC(i,k)*DC(i,k  )+                    &
!>   &                         tl_CF(i,k)*DC(i,k+1))-                   &
!>   &                        FC(i,k)*tl_DC(i,k-1))
!>
              adfac=cff*ad_DC(i,k)
              ad_DC(i,k-1)=ad_DC(i,k-1)-FC(i,k)*adfac
              ad_CF(i,k)=ad_CF(i,k)-DC(i,k+1)*adfac
              ad_BC(i,k)=ad_BC(i,k)-DC(i,k  )*adfac
              ad_FC(i,k)=ad_FC(i,k)-DC(i,k-1)*adfac
              ad_t(i,j,k  ,nnew,itrc)=ad_t(i,j,k  ,nnew,itrc)-adfac
              ad_t(i,j,k+1,nnew,itrc)=ad_t(i,j,k+1,nnew,itrc)+adfac
              ad_DC(i,k)=0.0_r8
!>            tl_BC(i,k)=cff1*(tl_Hz(i,j,k)+tl_Hz(i,j,k+1))+            &
!>   &                   dt(ng)*(tl_Akt(i,j,k,ltrc)*                    &
!>   &                           (oHz(i,j,k)+oHz(i,j,k+1))+             &
!>   &                           Akt(i,j,k,ltrc)*                       &
!>   &                           (tl_oHz(i,j,k)+tl_oHz(i,j,k+1)))
!>
              adfac=cff1*ad_BC(i,k)
              adfac1=dt(ng)*ad_BC(i,k)
              adfac2=adfac1*Akt(i,j,k,ltrc)
              ad_oHz(i,j,k  )=ad_oHz(i,j,k  )+adfac2
              ad_oHz(i,j,k+1)=ad_oHz(i,j,k+1)+adfac2
              ad_Akt(i,j,k,ltrc)=ad_Akt(i,j,k,ltrc)+                    &
     &                           (oHz(i,j,k)+oHz(i,j,k+1))*adfac1
              ad_Hz(i,j,k  )=ad_Hz(i,j,k  )+adfac
              ad_Hz(i,j,k+1)=ad_Hz(i,j,k+1)+adfac
              ad_BC(i,k)=0.0_r8
            END DO
          END DO
!
!  Use conservative, parabolic spline reconstruction of tangent linear
!  vertical diffusion derivatives.  Then, time step vertical diffusion
!  term implicitly.
!
!  Note that the BASIC STATE "t" must in Tunits when used in the
!  tangent spline routine below, which it does in the present code.
!
          DO i=Istr,Iend
!>          tl_CF(i,0)=0.0_r8
!>
            ad_CF(i,0)=0.0_r8
!>          tl_DC(i,0)=0.0_r8
!>
            ad_DC(i,0)=0.0_r8
          END DO
          cff1=1.0_r8/6.0_r8
          DO k=1,N(ng)-1
            DO i=Istr,Iend
!>            tl_CF(i,k)=cff1*tl_Hz(i,j,k+1)-                           &
!>   &                   dt(ng)*(tl_Akt(i,j,k+1,ltrc)*oHz(i,j,k+1)+     &
!>   &                           Akt(i,j,k+1,ltrc)*tl_oHz(i,j,k+1))
!>
              adfac=dt(ng)*ad_CF(i,k)
              ad_oHz(i,j,k+1)=ad_oHz(i,j,k+1)-                          &
     &                        Akt(i,j,k+1,ltrc)*adfac
              ad_Akt(i,j,k+1,ltrc)=ad_Akt(i,j,k+1,ltrc)-                &
     &                             oHz(i,j,k+1)*adfac
              ad_Hz(i,j,k+1)=ad_Hz(i,j,k+1)+cff1*ad_CF(i,k)
              ad_CF(i,k)=0.0_r8
!>            tl_FC(i,k)=cff1*tl_Hz(i,j,k  )-                           &
!>   &                   dt(ng)*(tl_Akt(i,j,k-1,ltrc)*oHz(i,j,k  )+     &
!>   &                           Akt(i,j,k-1,ltrc)*tl_oHz(i,j,k  ))
!>
              adfac=dt(ng)*ad_FC(i,k)
              ad_oHz(i,j,k  )=ad_oHz(i,j,k  )-                          &
     &                        Akt(i,j,k-1,ltrc)*adfac
              ad_Akt(i,j,k-1,ltrc)=ad_Akt(i,j,k-1,ltrc)-                &
     &                             oHz(i,j,k  )*adfac
              ad_Hz(i,j,k  )=ad_Hz(i,j,k  )+cff1*ad_FC(i,k)
              ad_FC(i,k)=0.0_r8
            END DO
          END DO
# else
!
!  Compute off-diagonal BASIC STATE coefficients FC [lambda*dt*Akt/Hz]
!  for the implicit vertical diffusion terms at future time step,
!  located at horizontal RHO-points and vertical W-points.
!  Also set FC at the top and bottom levels.
!
          cff=-dt(ng)*lambda
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              cff1=1.0_r8/(z_r(i,j,k+1)-z_r(i,j,k))
              FC(i,k)=cff*cff1*Akt(i,j,k,ltrc)
            END DO
          END DO
          DO i=Istr,Iend
            FC(i,0)=0.0_r8
            FC(i,N(ng))=0.0_r8
          END DO
!
!  Compute diagonal matrix coefficients BC.
!
          DO k=1,N(ng)
            DO i=Istr,Iend
              BC(i,k)=Hz(i,j,k)-FC(i,k)-FC(i,k-1)
            END DO
          END DO
!
!  Compute new solution by back substitution.
!  (DC is a tangent linear variable here).
!
          DO i=Istr,Iend
            cff=1.0_r8/BC(i,1)
            CF(i,1)=cff*FC(i,1)
          END DO
          DO k=2,N(ng)-1
            DO i=Istr,Iend
              cff=1.0_r8/(BC(i,k)-FC(i,k-1)*CF(i,k-1))
              CF(i,k)=cff*FC(i,k)
            END DO
          END DO
!>        DO k=N(ng)-1,1,-1
!>
          DO k=1,N(ng)-1
            DO i=Istr,Iend
#  ifdef DIAGNOSTICS_TS
!!            DiaTwrk(i,j,k,itrc,iTvdif)=DiaTwrk(i,j,k,itrc,iTvdif)+    &
!!   &                                   t(i,j,k,nnew,itrc)-cff1
#  endif
!>            tl_t(i,j,k,nnew,itrc)=DC(i,k)
!>
              ad_DC(i,k)=ad_DC(i,k)+ad_t(i,j,k,nnew,itrc)
              ad_t(i,j,k,nnew,itrc)=0.0_r8
!>            DC(i,k)=DC(i,k)-CF(i,k)*DC(i,k+1)
!>
              ad_DC(i,k+1)=-CF(i,k)*ad_DC(i,k)
#  ifdef DIAGNOSTICS_TS
!!            cff1=t(i,j,k,nnew,itrc)*oHz(i,j,k)
#  endif
            END DO
          END DO
          DO i=Istr,Iend
#  ifdef DIAGNOSTICS_TS
!!          DiaTwrk(i,j,N(ng),itrc,iTvdif)=                             &
!!   &                             DiaTwrk(i,j,N(ng),itrc,iTvdif)+      &
!!   &                             t(i,j,N(ng),nnew,itrc)-cff1
#  endif
!>          tl_t(i,j,N(ng),nnew,itrc)=DC(i,N(ng))
!>
            ad_DC(i,N(ng))=ad_DC(i,N(ng))+ad_t(i,j,N(ng),nnew,itrc)
            ad_t(i,j,N(ng),nnew,itrc)=0.0_r8
!>          DC(i,N(ng))=(DC(i,N(ng))-FC(i,N(ng)-1)*DC(i,N(ng)-1))/      &
!>   &                  (BC(i,N(ng))-FC(i,N(ng)-1)*CF(i,N(ng)-1))
!>
            adfac=ad_DC(i,N(ng))/                                       &
     &            (BC(i,N(ng))-FC(i,N(ng)-1)*CF(i,N(ng)-1))
            ad_DC(i,N(ng)-1)=ad_DC(i,N(ng)-1)-FC(i,N(ng)-1)*adfac
            ad_DC(i,N(ng)  )=adfac
          END DO
!
!  Solve the adjoint tridiagonal system.
!  (DC is a tangent linear variable here).
!
          DO k=N(ng)-1,2,-1
            DO i=Istr,Iend
              cff=1.0_r8/(BC(i,k)-FC(i,k-1)*CF(i,k-1))
!>            DC(i,k)=cff*(DC(i,k)-FC(i,k-1)*DC(i,k-1))
!>
              adfac=cff*ad_DC(i,k)
              ad_DC(i,k-1)=ad_DC(i,k-1)-FC(i,k-1)*adfac
              ad_DC(i,k  )=adfac
            END DO
          END DO
          DO i=Istr,Iend
            cff=1.0_r8/BC(i,1)
!>          DC(i,1)=cff*DC(i,1)
!>
            ad_DC(i,1)=cff*ad_DC(i,1)
          END DO
!
          DO i=Istr,Iend
!>          DC(i,N(ng))=tl_t(i,j,N(ng),nnew,itrc)-                      &
!>   &                  (tl_FC(i,N(ng)-1)*t(i,j,N(ng)-1,nnew,itrc)+     &
!>   &                   tl_BC(i,N(ng)  )*t(i,j,N(ng)  ,nnew,itrc))
!>
            ad_BC(i,N(ng)  )=-t(i,j,N(ng)  ,nnew,itrc)*ad_DC(i,N(ng))
            ad_FC(i,N(ng)-1)=-t(i,j,N(ng)-1,nnew,itrc)*ad_DC(i,N(ng))
            ad_t(i,j,N(ng),nnew,itrc)=ad_DC(i,N(ng))
            ad_DC(i,N(ng))=0.0_r8
!>          DC(i,1)=tl_t(i,j,1,nnew,itrc)-                              &
!>   &              (tl_BC(i,1)*t(i,j,1,nnew,itrc)+                     &
!>   &               tl_FC(i,1)*t(i,j,2,nnew,itrc))
!>
            ad_FC(i,1)=-t(i,j,2,nnew,itrc)*ad_DC(i,1)
            ad_BC(i,1)=-t(i,j,1,nnew,itrc)*ad_DC(i,1)
            ad_t(i,j,1,nnew,itrc)=ad_DC(i,1)
            ad_DC(i,1)=0.0_r8
          END DO
          DO k=2,N(ng)-1
            DO i=Istr,Iend
!>            DC(i,k)=tl_t(i,j,k,nnew,itrc)-                            &
!>   &                (tl_FC(i,k-1)*t(i,j,k-1,nnew,itrc)+               &
!>   &                 tl_BC(i,k  )*t(i,j,k  ,nnew,itrc)+               &
!>   &                 tl_FC(i,k  )*t(i,j,k+1,nnew,itrc))
!>
              ad_FC(i,k-1)=ad_FC(i,k-1)-                                &
     &                     t(i,j,k-1,nnew,itrc)*ad_DC(i,k)
              ad_FC(i,k  )=ad_FC(i,k  )-                                &
     &                     t(i,j,k+1,nnew,itrc)*ad_DC(i,k)
              ad_BC(i,k)=ad_BC(i,k)-                                    &
     &                   t(i,j,k  ,nnew,itrc)*ad_DC(i,k)
              ad_t(i,j,k,nnew,itrc)=ad_t(i,j,k,nnew,itrc)+ad_DC(i,k)
              ad_DC(i,k)=0.0_r8
            END DO
          END DO
!
!  Compute diagonal matrix coefficients BC.
!
          DO k=1,N(ng)
            DO i=Istr,Iend
!>            tl_BC(i,k)=tl_Hz(i,j,k)-tl_FC(i,k)-tl_FC(i,k-1)
!>
              ad_FC(i,k-1)=ad_FC(i,k-1)-ad_BC(i,k)
              ad_FC(i,k  )=ad_FC(i,k  )-ad_BC(i,k)
              ad_Hz(i,j,k)=ad_Hz(i,j,k)+ad_BC(i,k)
              ad_BC(i,k)=0.0_r8
            END DO
          END DO
!
!  Compute off-diagonal coefficients FC [lambda*dt*Akt/Hz] for the
!  implicit vertical diffusion terms at future time step, located
!  at horizontal RHO-points and vertical W-points.
!  Also set FC at the top and bottom levels.
!
!  NOTE: The original code solves the tridiagonal system A*t=r where
!        A is a matrix and t and r are vectors. We need to solve the
!        tangent linear C of this system which is A*tl_t+tl_A*t=tl_r.
!        Here, tl_A*t and tl_r are known, so we must solve for tl_t
!        by inverting A*tl_t=tl_r-tl_A*t.
!
          DO i=Istr,Iend
!>          tl_FC(i,N(ng))=0.0_r8
!>
            ad_FC(i,N(ng))=0.0_r8
!>          tl_FC(i,0)=0.0_r8
!>
            ad_FC(i,0)=0.0_r8
          END DO
          cff=-dt(ng)*lambda
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              cff1=1.0_r8/(z_r(i,j,k+1)-z_r(i,j,k))
!>            tl_FC(i,k)=cff*(tl_cff1*Akt(i,j,k,ltrc)+                  &
!>   &                        cff1*tl_Akt(i,j,k,ltrc))
!>
              adfac=cff*ad_FC(i,k)
              ad_Akt(i,j,k,ltrc)=ad_Akt(i,j,k,ltrc)+cff1*adfac
              ad_cff1=ad_cff1+Akt(i,j,k,ltrc)*adfac
              ad_FC(i,k)=0.0_r8
!>            tl_cff1=-cff1*cff1*(tl_z_r(i,j,k+1)-tl_z_r(i,j,k))
!>
              adfac=-cff1*cff1*ad_cff1
              ad_z_r(i,j,k  )=ad_z_r(i,j,k  )-adfac
              ad_z_r(i,j,k+1)=ad_z_r(i,j,k+1)+adfac
              ad_cff1=0.0_r8
            END DO
          END DO
# endif
        END DO
      END DO
# ifdef TS_MPDATA
!
!-----------------------------------------------------------------------
!  Compute adjoint anti-diffusive velocities and correct advected
!  tracers using MPDATA recursive method.  Notice that pipelined
!  J-loop ended.
!-----------------------------------------------------------------------
!
!  Compute BASIC STATE private tracer diffusion and anti-diffusive
!  velocities.
!
!  NOTE: the BASIC STATE retrived is already in Tunits so it is not
!        necessary to divide by Hz.
!
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-2,Iend+3
#  else
#   define I_RANGE MAX(Istr-2,1),MIN(Iend+3,Lm(ng)+1)
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-2,Jend+2
#  else
#   define J_RANGE MAX(Jstr-2,1),MIN(Jend+2,Mm(ng))
#  endif

      DO itrc=1,NT(ng)
        DO k=1,N(ng)
          DO j=J_RANGE
             DO i=I_RANGE
              cff1=MAX(Huon(i,j,k),0.0_r8)
              cff2=MIN(Huon(i,j,k),0.0_r8)
              FX(i,j)=cff1*t(i-1,j,k,3,itrc)+                           &
     &                cff2*t(i  ,j,k,3,itrc)
            END DO
          END DO
#  undef I_RANGE
#  undef J_RANGE
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-2,Iend+2
#  else
#   define I_RANGE MAX(Istr-2,1),MIN(Iend+2,Lm(ng))
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-2,Jend+3
#  else
#   define J_RANGE MAX(Jstr-2,1),MIN(Jend+3,Mm(ng)+1)
#  endif
          DO j=J_RANGE
            DO i=I_RANGE
              cff1=MAX(Hvom(i,j,k),0.0_r8)
              cff2=MIN(Hvom(i,j,k),0.0_r8)
              FE(i,j)=cff1*t(i,j-1,k,3,itrc)+                           &
     &                cff2*t(i,j  ,k,3,itrc)
            END DO
          END DO
#  undef I_RANGE
#  undef J_RANGE
#  ifdef TS_PSOURCE
!
!  Apply tracers point sources to the BASIC STATE horizontal advection
!  terms.
!
#   ifdef TS_MPDATA
          DO is=1,Nsrc
            i=Isrc(is)
            j=Jsrc(is)
            IF (INT(Dsrc(is)).eq.0) THEN
              IF (((Istr-2.le.i).and.(i.le.Iend+2)).and.                &
     &            ((Jstr-2.le.j).and.(j.le.Jend+2))) THEN
                IF (Lsrc(is,itrc)) THEN
                  FX(i,j)=Huon(i,j,k)*Tsrc(is,k,itrc)
#    ifdef MASKING
                ELSE
                  IF ((rmask(i  ,j).eq.0.0_r8).and.                     &
     &                (rmask(i-1,j).eq.1.0_r8)) THEN
                    FX(i,j)=Huon(i,j,k)*t(i-1,j,k,3,itrc)*oHz(i-1,j,k)
                  ELSE IF ((rmask(i  ,j).eq.1.0_r8).and.                &
     &                     (rmask(i-1,j).eq.0.0_r8)) THEN
                    FX(i,j)=Huon(i,j,k)*t(i  ,j,k,3,itrc)*oHz(i  ,j,k)
                  END IF
#    endif
                END IF
              END IF
            ELSE IF (INT(Dsrc(is)).eq.1) THEN
              IF (((Istr-2.le.i).and.(i.le.Iend+2)).and.                &
     &            ((Jstr-2.le.j).and.(j.le.Jend+2))) THEN
                IF (Lsrc(is,itrc)) THEN
                  FE(i,j)=Hvom(i,j,k)*Tsrc(is,k,itrc)
#    ifdef MASKING
                ELSE
                  IF ((rmask(i,j  ).eq.0.0_r8).and.                     &
     &                (rmask(i,j-1).eq.1.0_r8)) THEN
                    FE(i,j)=Hvom(i,j,k)*t(i,j-1,k,3,itrc)*oHz(i,j-1,k)
                  ELSE IF ((rmask(i,j  ).eq.1.0_r8).and.                &
     &                     (rmask(i,j-1).eq.0.0_r8)) THEN
                    FE(i,j)=Hvom(i,j,k)*t(i,j  ,k,3,itrc)*oHz(i,j  ,k)
                  END IF
#    endif
                END IF
              END IF
            END IF
          END DO
#   else
          DO is=1,Nsrc
            i=Isrc(is)
            j=Jsrc(is)
            IF (INT(Dsrc(is)).eq.0) THEN
              IF (((Istr.le.i).and.(i.le.Iend+1)).and.                  &
     &            ((Jstr.le.j).and.(j.le.Jend))) THEN
                IF (Lsrc(is,itrc)) THEN
                  FX(i,j)=Huon(i,j,k)*Tsrc(is,k,itrc)
#    ifdef MASKING
                ELSE
                  IF ((rmask(i  ,j).eq.0.0_r8).and.                     &
     &                (rmask(i-1,j).eq.1.0_r8)) THEN
                    FX(i,j)=Huon(i,j,k)*t(i-1,j,k,3,itrc)
                  ELSE IF ((rmask(i  ,j).eq.1.0_r8).and.                &
     &                     (rmask(i-1,j).eq.0.0_r8)) THEN
                    FX(i,j)=Huon(i,j,k)*t(i  ,j,k,3,itrc)
                  END IF
#    endif
                END IF
              END IF
            ELSE IF (INT(Dsrc(is)).eq.1) THEN
              IF (((Istr.le.i).and.(i.le.Iend)).and.                    &
     &            ((Jstr.le.j).and.(j.le.Jend+1))) THEN
                IF (Lsrc(is,itrc)) THEN
                  FE(i,j)=Hvom(i,j,k)*Tsrc(is,k,itrc)
#    ifdef MASKING
                ELSE
                  IF ((rmask(i,j  ).eq.0.0_r8).and.                     &
     &                (rmask(i,j-1).eq.1.0_r8)) THEN
                    FE(i,j)=Hvom(i,j,k)*t(i,j-1,k,3,itrc)

                  ELSE IF ((rmask(i,j  ).eq.1.0_r8).and.                &
     &                     (rmask(i,j-1).eq.0.0_r8)) THEN
                    FE(i,j)=Hvom(i,j,k)*t(i,j  ,k,3,itrc)
                  END IF
#    endif
                END IF
              END IF
            END IF
          END DO
#   endif
#  endif
!
!  Time-step BASIC STATE horizontal advection term (Tunits).
!
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-2,Iend+2
#  else
#   define I_RANGE MAX(Istr-2,1),MIN(Iend+2,Lm(ng))
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-2,Jend+2
#  else
#   define J_RANGE MAX(Jstr-2,1),MIN(Jend+2,Mm(ng))
#  endif
          DO j=J_RANGE
            DO i=I_RANGE
              cff=dt(ng)*pm(i,j)*pn(i,j)*oHz(i,j,k)
              cff1=cff*(FX(i+1,j)-FX(i,j)+                              &
     &                  FE(i,j+1)-FE(i,j))
              Ta(i,j,k,itrc)=t(i,j,k,3,itrc)-cff1
            END DO
          END DO
        END DO
      END DO
!
!  First_order, upstream differences vertical BASIC STATE advective
!  flux.
!
      DO j=J_RANGE
        DO itrc=1,NT(ng)
          DO i=I_RANGE
            DO k=1,N(ng)-1
              cff1=MAX(W(i,j,k),0.0_r8)
              cff2=MIN(W(i,j,k),0.0_r8)
              FC(i,k)=cff1*t(i,j,k  ,3,itrc)+                           &
     &                cff2*t(i,j,k+1,3,itrc)
            END DO
            FC(i,0)=0.0_r8
            FC(i,N(ng))=0.0_r8
          END DO
!
!  Time-step BASIC STATE vertical advection term.
!
          DO i=I_RANGE
            CF(i,0)=dt(ng)*pm(i,j)*pn(i,j)
          END DO
          DO k=1,N(ng)
            DO i=I_RANGE
              cff1=CF(i,0)*(FC(i,k)-FC(i,k-1))*oHz(i,j,k)
              Ta(i,j,k,itrc)=Ta(i,j,k,itrc)-cff1
            END DO
          END DO
        END DO
      END DO
#  undef I_RANGE
#  undef J_RANGE
!
!  Compute BASIC STATE anti-diffusive velocities to corrected advected
!  tracers using MPDATA recursive method.
!
      DO itrc=1,NT(ng)
        CALL rp_mpdata_adiff_tile (ng, Istr, Iend, Jstr, Jend, itrc,    &
     &                             LBi, UBi, LBj, UBj,                  &
     &                             ILB, IUB, JLB, JUB,                  &
     &                             nrhs,                                &
#  ifdef MASKING
     &                             rmask, umask, vmask,                 &
#  endif
     &                             pm, pn, omn, om_u, on_v,             &
     &                             Hz, ad_Hz,                           &
     &                             t, ad_t,                             &
     &                             u, ad_u,                             &
     &                             v, ad_v,                             &
     &                             W, ad_W,                             &
     &                             Ta(:,:,:,itrc),                      &
     &                             ad_Ta(:,:,:,itrc),                   &
     &                             Ua, ad_Ua,                           &
     &                             Va, ad_Va,                           &
     &                             Wa, ad_Wa)
!
!  Compute BASIC STATE anti-difussive corrected advection fluxes.
!
        DO k=1,N(ng)
          DO j=Jstr,Jend
            DO i=Istr,Iend+1
              cff1=MAX(Ua(i,j,k),0.0_r8)
              cff2=MIN(Ua(i,j,k),0.0_r8)
              FX(i,j)=(cff1*Ta(i-1,j,k,itrc)+                           &
     &                 cff2*Ta(i  ,j,k,itrc))*                          &
     &                0.5_r8*(Hz(i,j,k)+Hz(i-1,j,k))*on_u(i,j)
            END DO
          END DO
          DO j=Jstr,Jend+1
            DO i=Istr,Iend
              cff1=MAX(Va(i,j,k),0.0_r8)
              cff2=MIN(Va(i,j,k),0.0_r8)
              FE(i,j)=(cff1*Ta(i,j-1,k,itrc)+                           &
     &                 cff2*Ta(i,j  ,k,itrc))*                          &
     &                0.5_r8*(Hz(i,j,k)+Hz(i,j-1,k))*om_v(i,j)
            END DO
          END DO
#  ifdef TS_PSOURCE
!
!  Apply tracers point sources to the BASIC STATE corrected horizontal
!  advection terms.
!
          DO is=1,Nsrc
            i=Isrc(is)
            j=Jsrc(is)
            IF (INT(Dsrc(is)).eq.0) THEN
              IF (((Istr.le.i).and.(i.le.Iend+1)).and.                  &
     &            ((Jstr.le.j).and.(j.le.Jend))) THEN
                IF (Lsrc(is,itrc)) THEN
                  FX(i,j)=Huon(i,j,k)*Tsrc(is,k,itrc)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i  ,j).eq.0.0_r8).and.                     &
     &                (rmask(i-1,j).eq.1.0_r8)) THEN
                    FX(i,j)=Huon(i,j,k)*t(i-1,j,k,3,itrc)
                  ELSE IF ((rmask(i  ,j).eq.1.0_r8).and.                &
     &                     (rmask(i-1,j).eq.0.0_r8)) THEN
                    FX(i,j)=Huon(i,j,k)*t(i  ,j,k,3,itrc)
                  END IF
#   endif
                END IF
              END IF
            ELSE IF (INT(Dsrc(is)).eq.1) THEN
              IF (((Istr.le.i).and.(i.le.Iend)).and.                    &
     &            ((Jstr.le.j).and.(j.le.Jend+1))) THEN
                IF (Lsrc(is,itrc)) THEN
                  FE(i,j)=Hvom(i,j,k)*Tsrc(is,k,itrc)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i,j  ).eq.0.0_r8).and.                     &
     &                (rmask(i,j-1).eq.1.0_r8)) THEN
                    FE(i,j)=Hvom(i,j,k)*t(i,j-1,k,3,itrc)
                  ELSE IF ((rmask(i,j  ).eq.1.0_r8).and.                &
     &                     (rmask(i,j-1).eq.0.0_r8)) THEN
                    FE(i,j)=Hvom(i,j,k)*t(i,j  ,k,3,itrc)
                  END IF
#   endif
                END IF
              END IF
            END IF
          END DO
#  endif
        END DO
!
!  Compute BASIC anti-difussive corrected vertical advection flux.
!
        DO j=Jstr,Jend
          DO k=1,N(ng)-1
            DO i=Istr,Iend
               cff1=MAX(Wa(i,j,k),0.0_r8)
               cff2=MIN(Wa(i,j,k),0.0_r8)
               FC(i,k)=cff1*Ta(i,j,k  ,itrc)+                           &
     &                 cff2*Ta(i,j,k+1,itrc)
            END DO
            DO i=Istr,Iend
              FC(i,0)=0.0_r8
              FC(i,N(ng))=0.0_r8
            END DO
          END DO
!
!  Time-step adjoint corrected vertical advection (Tunits).
#  ifdef DIAGNOSTICS_TS
!  Convert units of tracer diagnostic terms to Tunits.
#  endif
!
          DO i=Istr,Iend
            CF(i,0)=dt(ng)*pm(i,j)*pn(i,j)
          END DO
          DO k=1,N(ng)
            DO i=Istr,Iend
#  ifdef DIAGNOSTICS_TS
!!            DiaTwrk(i,j,k,itrc,iTvadv)=DiaTwrk(i,j,k,itrc,iTvadv)-    &
!!   &                                   cff1
!!            DiaTwrk(i,j,k,itrc,iThadv)=DiaTwrk(i,j,k,itrc,iThadv)*    &
!!   &                                   oHz(i,j,k)
!!            DiaTwrk(i,j,k,itrc,iTvadv)=DiaTwrk(i,j,k,itrc,iTvadv)*    &
!!   &                                   oHz(i,j,k)
#   if defined TS_DIF2 || defined TS_DIF4
!!            DiaTwrk(i,j,k,itrc,iThdif)=DiaTwrk(i,j,k,itrc,iThdif)*    &
!!   &                                   oHz(i,j,k)
#   endif
!!            DiaTwrk(i,j,k,itrc,iTvdif)=DiaTwrk(i,j,k,itrc,iTvdif)*    &
!!   &                                   oHz(i,j,k)
!!            DiaTwrk(i,j,k,itrc,iTrate)=DiaTwrk(i,j,k,itrc,iTrate)*    &
!!   &                                   oHz(i,j,k)
#  endif
#  ifndef SPLINES
!>            tl_t(i,j,k,nnew,itrc)=tl_t(i,j,k,nnew,itrc)*Hz(i,j,k)+    &
!>   &                              t(i,j,k,nnew,itrc)*tl_Hz(i,j,k)
!>
              ad_Hz(i,j,k)=ad_Hz(i,j,k)+                                &
     &                     t(i,j,k,nnew,itrc)*ad_t(i,j,k,nnew,itrc)
              ad_t(i,j,k,nnew,itrc)=ad_t(i,j,k,nnew,itrc)*Hz(i,j,k)
#  endif
!>            tl_t(i,j,k,nnew,itrc)=tl_t(i,j,k,nnew,itrc)-tl_cff1
!>
              ad_cff1=ad_cff1-ad_t(i,j,k,nnew,itrc)
!>            tl_cff1=CF(i,0)*((tl_FC(i,k)-tl_FC(i,k-1))*oHz(i,j,k)+    &
!>   &                         (FC(i,k)-FC(i,k-1))*tl_oHz(i,j,k))
!>
              adfac=CF(i,0)*ad_cff1
              adfac1=adfac*oHz(i,j,k)
              ad_oHz(i,j,k)=ad_oHz(i,j,k)+(FC(i,k)-FC(i,k-1))*adfac
              ad_FC(i,k-1)=ad_FC(i,k-1)-adfac1
              ad_FC(i,k  )=ad_FC(i,k  )-adfac1
              ad_cff1=0.0_r8
            END DO
          END DO
!
!  Compute adjoint anti-difussive corrected vertical advection flux.
!
          DO k=1,N(ng)-1
            DO i=Istr,Iend
!>            tl_FC(i,N(ng))=0.0_r8
!>
              ad_FC(i,N(ng))=0.0_r8
!>            tl_FC(i,0)=0.0_r8
!>
              ad_FC(i,0)=0.0_r8
            END DO
            DO i=Istr,Iend
              cff1=MAX(Wa(i,j,k),0.0_r8)
              cff2=MIN(Wa(i,j,k),0.0_r8)
!>            tl_FC(i,k)=tl_cff1*Ta(i,j,k  ,itrc)+                      &
!>   &                   cff1*tl_Ta(i,j,k  ,itrc)+                      &
!>   &                   tl_cff2*Ta(i,j,k+1,itrc)+                      &
!>   &                   cff2*tl_Ta(i,j,k+1,itrc)
!>
              ad_Ta(i,j,k  ,itrc)=ad_Ta(i,j,k  ,itrc)+cff1*ad_FC(i,k)
              ad_Ta(i,j,k+1,itrc)=ad_Ta(i,j,k+1,itrc)+cff2*ad_FC(i,k)
              ad_cff1=ad_cff1+Ta(i,j,k  ,itrc)*ad_FC(i,k)
              ad_cff2=ad_cff2+Ta(i,j,k+1,itrc)*ad_FC(i,k)
              ad_FC(i,k)=0.0_r8
!>            tl_cff2=(0.5_r8+SIGN(0.5,-Wa(i,j,k)))*tl_Wa(i,j,k)
!>            tl_cff1=(0.5_r8+SIGN(0.5, Wa(i,j,k)))*tl_Wa(i,j,k)
!>
              ad_Wa(i,j,k)=ad_Wa(i,j,k)+                                &
     &                     (0.5_r8+SIGN(0.5,-Wa(i,j,k)))*ad_cff2+       &
     &                     (0.5_r8+SIGN(0.5, Wa(i,j,k)))*ad_cff1
              ad_cff2=0.0_r8
              ad_cff1=0.0_r8
            END DO
          END DO
        END DO
!
!  Time-step adjoint corrected horizontal advection (Tunits).
!
        DO k=1,N(ng)
          DO j=Jstr,Jend
            DO i=Istr,Iend
#  ifdef DIAGNOSTICS_TS
!!            DiaTwrk(i,j,k,itrc,iThadv)=DiaTwrk(i,j,k,itrc,iThadv)-    &
!!   &                                   cff1
#  endif
!>            tl_t(i,j,k,nnew,itrc)=tl_Ta(i,j,k,itrc)-tl_cff1
!>
              ad_cff1=ad_cff1-ad_Ta(i,j,k,itrc)
              ad_t(i,j,k,nnew,itrc)=0.0_r8
!>            tl_cff1=dt(ng)*pm(i,j)*pn(i,j)*                           &
!>   &                (tl_oHz(i,j,k)*                                   &
!>   &                 (FX(i+1,j)-FX(i,j)+                              &
!>   &                  FE(i,j+1)-FE(i,j))+                             &
!>   &                 oHz(i,j,k)*                                      &
!>   &                 (tl_FX(i+1,j)-tl_FX(i,j)+                        &
!>   &                  tl_FE(i,j+1)-tl_FE(i,j)))
!>
              adfac=dt(ng)*pm(i,j)*pn(i,j)*ad_cff1
              adfac1=adfac*oHz(i,j,k)
              ad_FE(i,j  )=ad_FE(i,j  )-adfac1
              ad_FE(i,j+1)=ad_FE(i,j+1)+adfac1
              ad_FX(i,j  )=ad_FX(i,j  )-adfac1
              ad_FX(i+1,j)=ad_FX(i+1,j)+adfac1
              ad_oHz(i,j,k)=ad_oHz(i,j,k)+                              &
     &                      (FX(i+1,j)-FX(i,j)+                         &
     &                       FE(i,j+1)-FE(i,j))*adfac
              ad_cff1=0.0_r8
            END DO
          END DO
#  ifdef TS_PSOURCE
!
!  Apply tracers point sources to the horizontal advection terms.
!
          DO is=1,Nsrc
            i=Isrc(is)
            j=Jsrc(is)
            IF (INT(Dsrc(is)).eq.0) THEN
              IF (((Istr.le.i).and.(i.le.Iend+1)).and.                  &
     &            ((Jstr.le.j).and.(j.le.Jend))) THEN
                IF (Lsrc(is,itrc)) THEN
!>                tl_FX(i,j)=tl_Huon(i,j,k)*Tsrc(is,k,itrc)
!>
                  ad_Huon(i,j,k)=ad_Huon(i,j,k)+                        &
     &                           Tsrc(is,k,itrc)*ad_FX(i,j)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i  ,j).eq.0.0_r8).and.                     &
     &                (rmask(i-1,j).eq.1.0_r8)) THEN
!>                  tl_FX(i,j)=tl_Huon(i,j,k)*t(i-1,j,k,3,itrc)+        &
!>   &                         Huon(i,j,k)*tl_t(i-1,j,k,3,itrc)
!>
                    ad_t(i-1,j,k,3,itrc)=ad_t(i-1,j,k,3,itrc)+          &
     &                                   Huon(i,j,k)*ad_FX(i,j)
                    ad_Huon(i,j,k)=ad_Huon(i,j,k)+                      &
     &                             t(i-1,j,k,3,itrc)*ad_FX(i,j)
                    ad_FX(i,j)=0.0_r8
                  ELSE IF ((rmask(i  ,j).eq.1.0_r8).and.                &
     &                     (rmask(i-1,j).eq.0.0_r8)) THEN
!>                  tl_FX(i,j)=tl_Huon(i,j,k)*t(i  ,j,k,3,itrc)+        &
!>   &                         Huon(i,j,k)*tl_t(i  ,j,k,3,itrc)
!>
                    ad_t(i  ,j,k,3,itrc)=ad_t(i  ,j,k,3,itrc)+          &
     &                                   Huon(i,j,k)*ad_FX(i,j)
                    ad_Huon(i,j,k)=ad_Huon(i,j,k)+                      &
     &                             t(i  ,j,k,3,itrc)*ad_FX(i,j)
                    ad_FX(i,j)=0.0_r8
                  END IF
#   endif
                END IF
              END IF
            ELSE IF (INT(Dsrc(is)).eq.1) THEN
              IF (((Istr.le.i).and.(i.le.Iend)).and.                    &
     &            ((Jstr.le.j).and.(j.le.Jend+1))) THEN
                IF (Lsrc(is,itrc)) THEN
!>                tl_FE(i,j)=tl_Hvom(i,j,k)*Tsrc(is,k,itrc)
!>
                  ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                        &
     &                           Tsrc(is,k,itrc)*ad_FE(i,j)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i,j  ).eq.0.0_r8).and.                     &
     &                (rmask(i,j-1).eq.1.0_r8)) THEN
!>                  tl_FE(i,j)=tl_Hvom(i,j,k)*t(i,j-1,k,3,itrc)+        &
!>   &                         Hvom(i,j,k)*tl_t(i,j-1,k,3,itrc)
!>
                    ad_t(i,j-1,k,3,itrc)=ad_t(i,j-1,k,3,itrc)+          &
     &                                   Hvom(i,j,k)*ad_FE(i,j)
                    ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                      &
     &                             t(i,j-1,k,3,itrc)*ad_FE(i,j)
                    ad_FE(i,j)=0.0_r8
                  ELSE IF ((rmask(i,j  ).eq.1.0_r8).and.                &
     &                     (rmask(i,j-1).eq.0.0_r8)) THEN
!>                  tl_FE(i,j)=tl_Hvom(i,j,k)*t(i,j  ,k,3,itrc)+        &
!>   &                         Hvom(i,j,k)*tl_t(i,j  ,k,3,itrc)
!>
                    ad_t(i,j  ,k,3,itrc)=ad_t(i,j  ,k,3,itrc)+          &
     &                                   Hvom(i,j,k)*ad_FE(i,j)
                    ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                      &
     &                             t(i,j  ,k,3,itrc)*ad_FE(i,j)
                    ad_FE(i,j)=0.0_r8
                  END IF
#   endif
                END IF
              END IF
            END IF
          END DO
#  endif
!
!  Compute adjoint anti-difussive corrected advection fluxes.
!
          DO j=Jstr,Jend+1
            DO i=Istr,Iend
              cff1=MAX(Va(i,j,k),0.0_r8)
              cff2=MIN(Va(i,j,k),0.0_r8)
!>            tl_FE(i,j)=0.5_r8*om_v(i,j)*                              &
!>   &                   ((tl_Hz(i,j,k)+tl_Hz(i,j-1,k))*                &
!>   &                    (cff1*Ta(i,j-1,k,itrc)+                       &
!>   &                     cff2*Ta(i,j  ,k,itrc))+                      &
!>   &                    (Hz(i,j,k)+Hz(i,j-1,k))*                      &
!>   &                    (tl_cff1*Ta(i,j-1,k,itrc)+                    &
!>   &                     cff1*tl_Ta(i,j-1,k,itrc)+                    &
!>   &                     tl_cff2*Ta(i,j  ,k,itrc)+                    &
!>   &                     cff2*tl_Ta(i,j  ,k,itrc)))
!>
              adfac=0.5_r8*om_v(i,j)*ad_FE(i,j)
              adfac1=adfac*(cff1*Ta(i,j-1,k,itrc)+                      &
     &                      cff2*Ta(i,j  ,k,itrc))
              adfac2=adfac*(Hz(i,j,k)+Hz(i,j-1,k))
              ad_Hz(i,j-1,k)=ad_Hz(i,j-1,k)+adfac1
              ad_Hz(i,j  ,k)=ad_Hz(i,j,k)+adfac1
              ad_Ta(i,j-1,k,itrc)=ad_Ta(i,j-1,k,itrc)+cff1*adfac2
              ad_Ta(i,j  ,k,itrc)=ad_Ta(i,j  ,k,itrc)+cff2*adfac2
              ad_cff1=ad_cff1+Ta(i,j-1,k,itrc)*adfac2
              ad_cff2=ad_cff2+Ta(i,j  ,k,itrc)*adfac2
              ad_FE(i,j)=0.0_r8
!>            tl_cff2=(0.5_r8+SIGN(0.5_r8,-Va(i,j,k)))*tl_Va(i,j,k)
!>            tl_cff1=(0.5_r8+SIGN(0.5_r8, Va(i,j,k)))*tl_Va(i,j,k)
!>
              ad_Va(i,j,k)=ad_Va(i,j,k)+                                &
     &                     (0.5_r8+SIGN(0.5_r8,-Va(i,j,k)))*ad_cff2+    &
     &                     (0.5_r8+SIGN(0.5_r8, Va(i,j,k)))*ad_cff1
              ad_cff2=0.0_r8
              ad_cff1=0.0_r8
            END DO
          END DO
          DO j=Jstr,Jend
            DO i=Istr,Iend+1
              cff1=MAX(Ua(i,j,k),0.0_r8)
              cff2=MIN(Ua(i,j,k),0.0_r8)
!>            tl_FX(i,j)=0.5_r8*on_u(i,j)*                              &
!>   &                   ((tl_Hz(i,j,k)+tl_Hz(i-1,j,k))*                &
!>   &                    (cff1*Ta(i-1,j,k,itrc)+                       &
!>   &                     cff2*Ta(i  ,j,k,itrc))+                      &
!>   &                    (Hz(i,j,k)+Hz(i-1,j,k))*                      &
!>   &                    (tl_cff1*Ta(i-1,j,k,itrc)+                    &
!>   &                     cff1*tl_Ta(i-1,j,k,itrc)+                    &
!>   &                     tl_cff2*Ta(i  ,j,k,itrc)+                    &
!>   &                     cff2*tl_Ta(i  ,j,k,itrc)))
!>
              adfac=0.5_r8*on_u(i,j)*ad_FX(i,j)
              adfac1=adfac*(cff1*Ta(i-1,j,k,itrc)+                      &
     &                      cff2*Ta(i  ,j,k,itrc))
              adfac2=adfac*(Hz(i,j,k)+Hz(i-1,j,k))
              ad_Hz(i-1,j,k)=ad_Hz(i-1,j,k)+adfac1
              ad_Hz(i  ,j,k)=ad_Hz(i  ,j,k)+adfac1
              ad_Ta(i-1,j,k,itrc)=ad_Ta(i-1,j,k,itrc)+cff1*adfac2
              ad_Ta(i  ,j,k,itrc)=ad_Ta(i  ,j,k,itrc)+cff2*adfac2
              ad_cff1=ad_cff1+Ta(i-1,j,k,itrc)*adfac1
              ad_cff2=ad_cff2+Ta(i  ,j,k,itrc)*adfac2
              ad_FX(i,j)=0.0_r8
!>            tl_cff2=(0.5_r8+SIGN(0.5_r8,-Ua(i,j,k)))*tl_Ua(i,j,k)
!>            tl_cff1=(0.5_r8+SIGN(0.5_r8, Ua(i,j,k)))*tl_Ua(i,j,k)
!>
              ad_Ua(i,j,k)=ad_Ua(i,j,k)+                                &
     &                     (0.5_r8+SIGN(0.5_r8,-Ua(i,j,k)))*ad_cff2+    &
     &                     (0.5_r8+SIGN(0.5_r8, Ua(i,j,k)))*ad_cff1
              ad_cff2=0.0_r8
              ad_cff1=0.0_r8
            END DO
          END DO
        END DO
!
!  Compute adjoint anti-diffusive velocities to corrected advected
!  tracers using MPDATA recursive method.
!
!>      CALL tl_mpdata_adiff_tile (ng, Istr, Iend, Jstr, Jend, itrc,    &
!>   &                             LBi, UBi, LBj, UBj,                  &
!>   &                             ILB, IUB, JLB, JUB,                  &
!>   &                             nrhs,                                &
!>   &                             omn, om_u, on_v,                     &
!>   &                             Hz, tl_Hz,                           &
!>   &                             t, tl_t,                             &
!>   &                             u, tl_u,                             &
!>   &                             v, tl_v,                             &
!>   &                             W, tl_W,                             &
!>   &                             Ta(:,:,:,itrc),                      &
!>   &                             tl_Ta(:,:,:,itrc),                   &
!>   &                             Ua, tl_Ua,                           &
!>   &                             Va, tl_Va,                           &
!>   &                             Wa, tl_Wa)
!>
        CALL ad_mpdata_adiff_tile (ng, Istr, Iend, Jstr, Jend, itrc,    &
     &                             LBi, UBi, LBj, UBj,                  &
     &                             ILB, IUB, JLB, JUB,                  &
     &                             nrhs,                                &
     &                             omn, om_u, on_v,                     &
     &                             Hz, ad_Hz,                           &
     &                             t, ad_t,                             &
     &                             u, ad_u,                             &
     &                             v, ad_v,                             &
     &                             W, ad_W,                             &
     &                             Ta(:,:,:,itrc),                      &
     &                             ad_Ta(:,:,:,itrc),                   &
     &                             Ua, ad_Ua,                           &
     &                             Va, ad_Va,                           &
     &                             Wa, ad_Wa)
      END DO
# endif /* TS_MPDATA */
!
!-----------------------------------------------------------------------
!  Time-step adjoint vertical advection term.
!-----------------------------------------------------------------------
!
# ifdef TS_MPDATA
!
!  Compute BASIC STATE Ta due to horizontal advection only.
!
!  NOTE: the BASIC STATE retrived is already in Tunits so it is not
!        necessary to divide by Hz.
!
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-2,Iend+3
#  else
#   define I_RANGE MAX(Istr-2,1),MIN(Iend+3,Lm(ng)+1)
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-2,Jend+2
#  else
#   define J_RANGE MAX(Jstr-2,1),MIN(Jend+2,Mm(ng))
#  endif

      DO itrc=1,NT(ng)
        DO k=1,N(ng)
          DO j=J_RANGE
             DO i=I_RANGE
              cff1=MAX(Huon(i,j,k),0.0_r8)
              cff2=MIN(Huon(i,j,k),0.0_r8)
              FX(i,j)=cff1*t(i-1,j,k,3,itrc)+                           &
     &                cff2*t(i  ,j,k,3,itrc)
            END DO
          END DO
#  undef I_RANGE
#  undef J_RANGE
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-2,Iend+2
#  else
#   define I_RANGE MAX(Istr-2,1),MIN(Iend+2,Lm(ng))
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-2,Jend+3
#  else
#   define J_RANGE MAX(Jstr-2,1),MIN(Jend+3,Mm(ng)+1)
#  endif
          DO j=J_RANGE
            DO i=I_RANGE
              cff1=MAX(Hvom(i,j,k),0.0_r8)
              cff2=MIN(Hvom(i,j,k),0.0_r8)
              FE(i,j)=cff1*t(i,j-1,k,3,itrc)+                           &
     &                cff2*t(i,j  ,k,3,itrc)
            END DO
          END DO
#  undef I_RANGE
#  undef J_RANGE
#  ifdef TS_PSOURCE
!
!  Apply tracers point sources to the BASIC STATE horizontal advection
!  terms.
!
          DO is=1,Nsrc
            i=Isrc(is)
            j=Jsrc(is)
            IF (INT(Dsrc(is)).eq.0) THEN
              IF (((Istr.le.i).and.(i.le.Iend+1)).and.                  &
     &            ((Jstr.le.j).and.(j.le.Jend))) THEN
                IF (Lsrc(is,itrc)) THEN
                  FX(i,j)=Huon(i,j,k)*Tsrc(is,k,itrc)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i  ,j).eq.0.0_r8).and.                     &
     &                (rmask(i-1,j).eq.1.0_r8)) THEN
                    FX(i,j)=Huon(i,j,k)*t(i-1,j,k,3,itrc)
                  ELSE IF ((rmask(i  ,j).eq.1.0_r8).and.                &
     &                     (rmask(i-1,j).eq.0.0_r8)) THEN
                    FX(i,j)=Huon(i,j,k)*t(i  ,j,k,3,itrc)
                  END IF
#   endif
                END IF
              END IF
            ELSE IF (INT(Dsrc(is)).eq.1) THEN
              IF (((Istr.le.i).and.(i.le.Iend)).and.                    &
     &            ((Jstr.le.j).and.(j.le.Jend+1))) THEN
                IF (Lsrc(is,itrc)) THEN
                  FE(i,j)=Hvom(i,j,k)*Tsrc(is,k,itrc)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i,j  ).eq.0.0_r8).and.                     &
     &                (rmask(i,j-1).eq.1.0_r8)) THEN
                    FE(i,j)=Hvom(i,j,k)*t(i,j-1,k,3,itrc)

                  ELSE IF ((rmask(i,j  ).eq.1.0_r8).and.                &
     &                     (rmask(i,j-1).eq.0.0_r8)) THEN
                    FE(i,j)=Hvom(i,j,k)*t(i,j  ,k,3,itrc)
                  END IF
#   endif
                END IF
              END IF
            END IF
          END DO
#  endif
!
!  Time-step BASIC STATE horizontal advection term (Tunits).
!
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-2,Iend+2
#  else
#   define I_RANGE MAX(Istr-2,1),MIN(Iend+2,Lm(ng))
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-2,Jend+2
#  else
#   define J_RANGE MAX(Jstr-2,1),MIN(Jend+2,Mm(ng))
#  endif
          DO j=J_RANGE
            DO i=I_RANGE
              cff=dt(ng)*pm(i,j)*pn(i,j)*oHz(i,j,k)
              cff1=cff*(FX(i+1,j)-FX(i,j)+                              &
     &                  FE(i,j+1)-FE(i,j))
              Ta(i,j,k,itrc)=t(i,j,k,3,itrc)-cff1
            END DO
          END DO
        END DO
      END DO
#  undef I_RANGE
#  undef J_RANGE
# endif
!
!  Time-step adjoint vertical advection term. Compute first BASIC
!  STATE vertical advection flux, FC.
!
# ifdef TS_MPDATA
#   ifdef EW_PERIODIC
#    define I_RANGE Istr-2,Iend+2
#   else
#    define I_RANGE MAX(Istr-2,1),MIN(Iend+2,Lm(ng))
#   endif
#   ifdef NS_PERIODIC
#    define J_RANGE Jstr-2,Jend+2
#   else
#    define J_RANGE MAX(Jstr-2,1),MIN(Jend+2,Mm(ng))
#   endif
# else
#   define I_RANGE Istr,Iend
#   define J_RANGE Jstr,Jend
# endif

      DO j=J_RANGE
        DO itrc=1,NT(ng)

# if defined TS_SVADVECTION
!
!  Build conservative parabolic splines for the BASIC STATE vertical
!  derivatives "FC" of the tracer.  Then, the interfacial "FC" values
!  are converted to vertical advective flux.
!
          DO i=Istr,Iend
#  ifdef NEUMANN
            FC(i,0)=1.5_r8*t(i,j,1,3,itrc)
            CF(i,1)=0.5_r8
#  else
            FC(i,0)=2.0_r8*t(i,j,1,3,itrc)
            CF(i,1)=1.0_r8
#  endif
          END DO
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              cff=1.0_r8/(2.0_r8*Hz(i,j,k)+                             &
     &                    Hz(i,j,k+1)*(2.0_r8-CF(i,k)))
              CF(i,k+1)=cff*Hz(i,j,k)
              FC(i,k)=cff*(3.0_r8*(Hz(i,j,k  )*t(i,j,k+1,3,itrc)+       &
     &                             Hz(i,j,k+1)*t(i,j,k  ,3,itrc))-      &
     &                     Hz(i,j,k+1)*FC(i,k-1))
            END DO
          END DO
          DO i=Istr,Iend
#  ifdef NEUMANN
            FC(i,N(ng))=(3.0_r8*t(i,j,N(ng),3,itrc)-FC(i,N(ng)-1))/     &
     &                  (2.0_r8-CF(i,N(ng)))
#  else
            FC(i,N(ng))=(2.0_r8*t(i,j,N(ng),3,itrc)-FC(i,N(ng)-1))/     &
     &                  (1.0_r8-CF(i,N(ng)))
#  endif
          END DO
          DO k=N(ng)-1,0,-1
            DO i=Istr,Iend
              FC(i,k)=FC(i,k)-CF(i,k+1)*FC(i,k+1)
              FC(i,k+1)=W(i,j,k+1)*FC(i,k+1)
            END DO
          END DO
          DO i=Istr,Iend
            FC(i,N(ng))=0.0_r8
            FC(i,0)=0.0_r8
          END DO
# elif defined TS_A4VADVECTION
!
!  Fourth-order, BASIC STATE Akima vertical advective flux.
!
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              FC(i,k)=t(i,j,k+1,3,itrc)-                                &
     &                t(i,j,k  ,3,itrc)
            END DO
          END DO
          DO i=Istr,Iend
            FC(i,0)=FC(i,1)
            FC(i,N(ng))=FC(i,N(ng)-1)
          END DO
          DO k=1,N(ng)
            DO i=Istr,Iend
              cff=2.0_r8*FC(i,k)*FC(i,k-1)
              IF (cff.gt.eps) THEN
                CF(i,k)=cff/(FC(i,k)+FC(i,k-1))
              ELSE
                CF(i,k)=0.0_r8
              END IF
            END DO
          END DO
          cff1=1.0_r8/3.0_r8
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              FC(i,k)=W(i,j,k)*                                         &
     &                0.5_r8*(t(i,j,k  ,3,itrc)+                        &
     &                        t(i,j,k+1,3,itrc)-                        &
     &                        cff1*(CF(i,k+1)-CF(i,k)))
            END DO
          END DO
          DO i=Istr,Iend
            FC(i,0)=0.0_r8
            FC(i,N(ng))=0.0_r8
          END DO
# elif defined TS_C2VADVECTION
!
!  Second-order, BASIC STATE central differences vertical advective
!  flux.
!
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              FC(i,k)=W(i,j,k)*                                         &
     &                0.5_r8*(t(i,j,k  ,3,itrc)+                        &
     &                        t(i,j,k+1,3,itrc))
            END DO
          END DO
          DO i=Istr,Iend
            FC(i,0)=0.0_r8
            FC(i,N(ng))=0.0_r8
          END DO
# elif defined TS_MPDATA
!
!  First_order, BASIC STATE upstream differences vertical advective
!  flux.
!
          DO i=I_RANGE
            DO k=1,N(ng)-1
              cff1=MAX(W(i,j,k),0.0_r8)
              cff2=MIN(W(i,j,k),0.0_r8)
              FC(i,k)=cff1*t(i,j,k  ,3,itrc)*oHz(i,j,k  )+              &
     &                cff2*t(i,j,k+1,3,itrc)*oHz(i,j,k+1)
            END DO
            FC(i,0)=0.0_r8
            FC(i,N(ng))=0.0_r8
          END DO
# else
!!
!!  Fourth-order, BASIC STATE central differences vertical advective
!!  flux. (Not really needed, HGA).
!!
!!        cff1=0.5_r8
!!        cff2=7.0_r8/12.0_r8
!!        cff3=1.0_r8/12.0_r8
!!        DO k=2,N(ng)-2
!!          DO i=Istr,Iend
!!            FC(i,k)=W(i,j,k)*                                         &
!!   &                (cff2*(t(i,j,k  ,3,itrc)+                         &
!!   &                       t(i,j,k+1,3,itrc))-                        &
!!   &                 cff3*(t(i,j,k-1,3,itrc)+                         &
!!   &                       t(i,j,k+2,3,itrc)))
!!          END DO
!!        END DO
!!        DO i=Istr,Iend
!!          FC(i,0)=0.0_r8
!!          FC(i,1)=W(i,j,1)*                                           &
!!   &              (cff1*t(i,j,1,3,itrc)+                              &
!!   &               cff2*t(i,j,2,3,itrc)-                              &
!!   &               cff3*t(i,j,3,3,itrc))
!!          FC(i,N(ng)-1)=W(i,j,N(ng)-1)*                               &
!!   &                    (cff1*t(i,j,N(ng)  ,3,itrc)+                  &
!!   &                     cff2*t(i,j,N(ng)-1,3,itrc)-                  &
!!   &                     cff3*t(i,j,N(ng)-2,3,itrc))
!!          FC(i,N(ng))=0.0_r8
!!        END DO
# endif
!
!  Time-step vertical advection term.
# ifdef SPLINES
!  The BASIC STATE "t" used below must be in transport units, but "t"
!  retrived is in Tunits so we multiply by "Hz".
# endif
!
          DO i=Istr,Iend
            CF(i,0)=dt(ng)*pm(i,j)*pn(i,j)
          END DO
          DO k=1,N(ng)
            DO i=I_RANGE
!!            cff1=CF(i,0)*(FC(i,k)-FC(i,k-1))    ! not needed HGA
# ifdef TS_MPDATA
#  ifdef DIAGNOSTICS_TS
!!            DiaTwrk(i,j,k,itrc,iTvadv)=-cff1
#  endif
!>            tl_Ta(i,j,k,itrc)=(tl_Ta(i,j,k,itrc)-tl_cff1)*            &
!>   &                          oHz(i,j,k)+                             &
!>   &                          (Ta(i,j,k,itrc)-cff1)*                  &
!>   &                          tl_oHz(i,j,k)
!>
              adfac=ad_Ta(i,j,k,itrc)*oHz(i,j,k)
              ad_oHz(i,j,k)=ad_oHz(i,j,k)+                              &
     &                      (Ta(i,j,k,itrc)-cff1)*ad_Ta(i,j,k,itrc)
              ad_Ta(i,j,k,itrc)=adfac
              ad_cff1=ad_cff1-adfac
# else
#  ifdef DIAGNOSTICS_TS
!!            DiaTwrk(i,j,k,itrc,iTvadv)=-cff1
!!            DiaTwrk(i,j,k,itrc,iThadv)=DiaTwrk(i,j,k,itrc,iThadv)*    &
!!   &                                   oHz(i,j,k)
!!            DiaTwrk(i,j,k,itrc,iTvadv)=DiaTwrk(i,j,k,itrc,iTvadv)*    &
!!   &                                   oHz(i,j,k)
#   if defined TS_DIF2 || defined TS_DIF4
!!            DiaTwrk(i,j,k,itrc,iThdif)=DiaTwrk(i,j,k,itrc,iThdif)*    &
!!   &                                   oHz(i,j,k)
#   endif
!!            DiaTwrk(i,j,k,itrc,iTvdif)=DiaTwrk(i,j,k,itrc,iTvdif)*    &
!!   &                                   oHz(i,j,k)
!!            DiaTwrk(i,j,k,itrc,iTrate)=DiaTwrk(i,j,k,itrc,iTrate)*    &
!!   &                                   oHz(i,j,k)
#  endif
#  ifdef SPLINES
!>            tl_t(i,j,k,nnew,itrc)=tl_t(i,j,k,nnew,itrc)*              &
!>   &                              oHz(i,j,k)+                         &
!>   &                              (t(i,j,k,nnew,itrc)*Hz(i,j,k))*     &
!>   &                              tl_oHz(i,j,k)
!>
              ad_oHz(i,j,k)=ad_oHz(i,j,k)+                              &
     &                      (t(i,j,k,nnew,itrc)*Hz(i,j,k))*             &
     &                      ad_t(i,j,k,nnew,itrc)
              ad_t(i,j,k,nnew,itrc)=ad_t(i,j,k,nnew,itrc)*oHz(i,j,k)
#  endif
!>            tl_t(i,j,k,nnew,itrc)=tl_t(i,j,k,nnew,itrc)-tl_cff1
!>
              ad_cff1=ad_cff1-ad_t(i,j,k,nnew,itrc)
# endif
!>            tl_cff1=CF(i,0)*(tl_FC(i,k)-tl_FC(i,k-1))
!>
              adfac=CF(i,0)*ad_cff1
              ad_FC(i,k-1)=ad_FC(i,k-1)-adfac
              ad_FC(i,k  )=ad_FC(i,k  )+adfac
              ad_cff1=0.0_r8
            END DO
          END DO

# ifdef TS_SVADVECTION
!
!  Build conservative parabolic splines for the vertical derivatives
!  "FC" of the tracer.  Then, the interfacial "FC" values are
!  converted to vertical advective flux.
!
          DO i=Istr,Iend
#  ifdef NEUMANN
            FC(i,0)=1.5_r8*t(i,j,1,3,itrc)
            CF(i,1)=0.5_r8
#  else
            FC(i,0)=2.0_r8*t(i,j,1,3,itrc)
            CF(i,1)=1.0_r8
#  endif
          END DO
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              cff=1.0_r8/(2.0_r8*Hz(i,j,k)+                             &
     &                    Hz(i,j,k+1)*(2.0_r8-CF(i,k)))
              CF(i,k+1)=cff*Hz(i,j,k)
              FC(i,k)=cff*(3.0_r8*(Hz(i,j,k  )*t(i,j,k+1,3,itrc)+       &
     &                             Hz(i,j,k+1)*t(i,j,k  ,3,itrc))-      &
     &                     Hz(i,j,k+1)*FC(i,k-1))
            END DO
          END DO
          DO i=Istr,Iend
#  ifdef NEUMANN
            FC(i,N(ng))=(3.0_r8*t(i,j,N(ng),3,itrc)-FC(i,N(ng)-1))/     &
     &                  (2.0_r8-CF(i,N(ng)))
#  else
            FC(i,N(ng))=(2.0_r8*t(i,j,N(ng),3,itrc)-FC(i,N(ng)-1))/     &
     &                  (1.0_r8-CF(i,N(ng)))
#  endif
          END DO
          DO k=N(ng)-1,0,-1
            DO i=Istr,Iend
              FC(i,k)=FC(i,k)-CF(i,k+1)*FC(i,k+1)
            END DO
          END DO
!
!  Now the adjoint splines code.
!
          DO i=Istr,Iend
!>          tl_FC(i,N(ng))=0.0_r8
!>
            ad_FC(i,N(ng))=0.0_r8
!>          tl_FC(i,0)=0.0_r8   
!>
            ad_FC(i,0)=0.0_r8   
          END DO               
!
!   Adjoint back substitution.
!
          DO k=0,N(ng)-1
            DO i=Istr,Iend
!>           tl_FC(i,k+1)=tl_W(i,j,k+1)*FC(i,k+1)+                      &
!>   &                    W(i,j,k+1)*tl_FC(i,k+1)
!>
             ad_W(i,j,k+1)=ad_W(i,j,k+1)+FC(i,k+1)*ad_FC(i,k+1)
             ad_FC(i,k+1)=W(i,j,k+1)*ad_FC(i,k+1)
!>           tl_FC(i,k)=tl_FC(i,k)-CF(i,k+1)*tl_FC(i,k+1)
!>
             ad_FC(i,k+1)=ad_FC(i,k+1)-CF(i,k+1)*ad_FC(i,k)
            END DO
          END DO
          DO i=Istr,Iend
#  ifdef NEUMANN
!>          tl_FC(i,N(ng))=(3.0_r8*tl_t(i,j,N(ng),3,itrc)-              &
!>   &                      tl_FC(i,N(ng)-1))/                          &
!>   &                     (2.0_r8-CF(i,N(ng)))
!>
            adfac=ad_FC(i,N(ng))/(2.0_r8-CF(i,N(ng)))
            ad_t(i,j,N(ng),3,itrc)=ad_t(i,j,N(ng),3,itrc)+3.0_r8*adfac
            ad_FC(i,N(ng)-1)=ad_FC(i,N(ng)-1)-adfac
            ad_FC(i,N(ng))=0.0_r8
#  else      
!>          tl_FC(i,N(ng))=(2.0_r8*tl_t(i,j,N(ng),3,itrc)-              &
!>   &                      tl_FC(i,N(ng)-1))/                          &
!>   &                     (1.0_r8-CF(i,N(ng)))
!>
            adfac=ad_FC(i,N(ng))/(1.0_r8-CF(i,N(ng)))
            ad_t(i,j,N(ng),3,itrc)=ad_t(i,j,N(ng),3,itrc)+2.0_r8*adfac
            ad_FC(i,N(ng)-1)=ad_FC(i,N(ng)-1)-adfac
            ad_FC(i,N(ng))=0.0
#  endif     
          END DO
          DO k=N(ng)-1,1,-1
            DO i=Istr,Iend
              cff=1.0_r8/(2.0_r8*Hz(i,j,k)+                             &
     &                    Hz(i,j,k+1)*(2.0_r8-CF(i,k)))
!>            tl_FC(i,k)=cff*                                           &
!>   &                   (3.0_r8*(Hz(i,j,k  )*tl_t(i,j,k+1,3,itrc)+     &
!>   &                            Hz(i,j,k+1)*tl_t(i,j,k  ,3,itrc)+     &
!>   &                            tl_Hz(i,j,k  )*t(i,j,k+1,3,itrc)+     &
!>   &                            tl_Hz(i,j,k+1)*t(i,j,k  ,3,itrc))-    &
!>   &                    (tl_Hz(i,j,k+1)*FC(i,k-1)+                    &
!>   &                     2.0_r8*(tl_Hz(i,j,k  )+                      &
!>   &                             tl_Hz(i,j,k+1))*FC(i,k)+             &
!>   &                     tl_Hz(i,j,k  )*FC(i,k+1))-                   &
!>   &                    Hz(i,j,k+1)*tl_FC(i,k-1))
!>
              adfac=cff*ad_FC(i,k)
              adfac1=3.0_r8*adfac
              adfac2=2.0_r8*adfac
              ad_t(i,j,k  ,3,itrc)=ad_t(i,j,k  ,3,itrc)+                &
     &                             Hz(i,j,k+1)*adfac1
              ad_t(i,j,k+1,3,itrc)=ad_t(i,j,k+1,3,itrc)+                &
     &                             Hz(i,j,k  )*adfac1
              ad_Hz(i,j,k  )=ad_Hz(i,j,k  )+                            &
     &                       t(i,j,k+1,3,itrc)*adfac1-                  &
     &                       FC(i,k  )*adfac2-                          &
     &                       FC(i,k+1)*adfac
              ad_Hz(i,j,k+1)=ad_Hz(i,j,k+1)+                            &
     &                       t(i,j,k  ,3,itrc)*adfac1-                  &
     &                       FC(i,k-1)*adfac-                           &
     &                       FC(i,k  )*adfac2
              ad_FC(i,k-1)=ad_FC(i,k-1)-Hz(i,j,k+1)*adfac
              ad_FC(i,k)=0.0_r8
            END DO
          END DO 
          DO i=Istr,Iend
#  ifdef NEUMANN
!>          tl_FC(i,0)=1.5_r8*tl_t(i,j,1,3,itrc)
!>
            ad_t(i,j,1,3,itrc)=ad_t(i,j,1,3,itrc)+1.5_r8*ad_FC(i,0)
            ad_FC(i,0)=0.0_r8
#  else      
!>          tl_FC(i,0)=2.0_r8*tl_t(i,j,1,3,itrc)
!>
            ad_t(i,j,1,3,itrc)=ad_t(i,j,1,3,itrc)+2.0_r8*ad_FC(i,0)
            ad_FC(i,0)=0.0_r8
#  endif     
          END DO

# elif defined TS_A4VADVECTION
!
!  Fourth-order, Akima adjoint vertical advective flux.
!
          DO k=1,N(ng)-1
            DO i=Istr,Iend
              FC(i,k)=t(i,j,k+1,3,itrc)-                                &
     &                t(i,j,k  ,3,itrc)
            END DO
          END DO
          DO i=Istr,Iend
            FC(i,0)=FC(i,1)
            FC(i,N(ng))=FC(i,N(ng)-1)
          END DO
          DO k=1,N(ng)
            DO i=Istr,Iend
              cff=2.0_r8*FC(i,k)*FC(i,k-1)
              IF (cff.gt.eps) THEN
                CF(i,k)=cff/(FC(i,k)+FC(i,k-1))
              ELSE
                CF(i,k)=0.0_r8
              END IF
            END DO
          END DO
          DO i=Istr,Iend
!>          tl_FC(i,N(ng))=0.0_r8
!>
            ad_FC(i,N(ng))=0.0_r8
!>          tl_FC(i,0)=0.0_r8
!>
            ad_FC(i,0)=0.0_r8
          END DO
          cff1=1.0_r8/3.0_r8
          DO k=1,N(ng)-1
            DO i=Istr,Iend
!>            tl_FC(i,k)=0.5_r8*                                        &
!>   &                   (tl_W(i,j,k)*                                  &
!>   &                    (t(i,j,k  ,3,itrc)+                           &
!>   &                     t(i,j,k+1,3,itrc)-                           &
!>   &                     cff1*(CF(i,k+1)-CF(i,k)))+                   &
!>   &                    W(i,j,k)*                                     &
!>   &                    (tl_t(i,j,k  ,3,itrc)+                        &
!>   &                     tl_t(i,j,k+1,3,itrc)-                        &
!>   &                     cff1*(tl_CF(i,k+1)-tl_CF(i,k))))
!>
              adfac=0.5_r8*ad_FC(i,k)
              adfac1=adfac*W(i,j,k)
              adfac2=adfac1*cff1
              ad_CF(i,k  )=ad_CF(i,k  )+adfac2
              ad_CF(i,k+1)=ad_CF(i,k+1)-adfac2
              ad_t(i,j,k  ,3,itrc)=ad_t(i,j,k  ,3,itrc)+adfac1
              ad_t(i,j,k+1,3,itrc)=ad_t(i,j,k+1,3,itrc)+adfac1
              ad_W(i,j,k)=ad_W(i,j,k)+                                  &
     &                    (t(i,j,k  ,3,itrc)+                           &
     &                     t(i,j,k+1,3,itrc)-                           &
     &                     cff1*(CF(i,k+1)-CF(i,k)))*adfac
              ad_FC(i,k)=0.0_r8
            END DO
          END DO
          DO k=1,N(ng)
            DO i=Istr,Iend
              cff=2.0_r8*FC(i,k)*FC(i,k-1)
              IF (cff.gt.eps) THEN
!>              tl_CF(i,k)=((FC(i,k)+FC(i,k-1))*tl_cff-                 &
!>   &                      cff*(tl_FC(i,k)+tl_FC(i,k-1)))/             &
!>   &                     ((FC(i,k)+FC(i,k-1))*(FC(i,k)+FC(i,k-1)))
!>
                adfac=ad_CF(i,k)/                                       &
     &                ((FC(i,k)+FC(i,k-1))*(FC(i,k)+FC(i,k-1)))
                adfac1=adfac*cff
                ad_FC(i,k-1)=ad_FC(i,k-1)-adfac1
                ad_FC(i,k  )=ad_FC(i,k  )-adfac1
                ad_cff=ad_cff+(FC(i,k)+FC(i,k-1))*adfac
                ad_CF(i,k)=0.0_r8
              ELSE
!>              tl_CF(i,k)=0.0_r8
!>
                ad_CF(i,k)=0.0_r8
              END IF
!>            tl_cff=2.0_r8*(tl_FC(i,k)*FC(i,k-1)+                      &
!>   &                       FC(i,k)*tl_FC(i,k-1))
!>
              adfac=2.0_r8*ad_cff
              ad_FC(i,k-1)=ad_FC(i,k-1)+FC(i,k  )*adfac
              ad_FC(i,k  )=ad_FC(i,k  )+FC(i,k-1)*adfac
              ad_cff=0.0_r8
            END DO
          END DO
          DO i=Istr,Iend
!>          tl_FC(i,N(ng))=tl_FC(i,N(ng)-1)
!>
            ad_FC(i,N(ng)-1)=ad_FC(i,N(ng)-1)+ad_FC(i,N(ng))
            ad_FC(i,N(ng))=0.0_r8
!>          tl_FC(i,0)=tl_FC(i,1)
!>
            ad_FC(i,1)=ad_FC(i,1)+ad_FC(i,0)
            ad_FC(i,0)=0.0_r8
          END DO
          DO k=1,N(ng)-1
            DO i=Istr,Iend
!>            tl_FC(i,k)=tl_t(i,j,k+1,3,itrc)-                          &
!>   &                   tl_t(i,j,k  ,3,itrc)
!>
              ad_t(i,j,k  ,3,itrc)=ad_t(i,j,k  ,3,itrc)-ad_FC(i,k)
              ad_t(i,j,k+1,3,itrc)=ad_t(i,j,k+1,3,itrc)+ad_FC(i,k)
              ad_FC(i,k)=0.0_r8
            END DO
          END DO
# elif defined TS_C2VADVECTION
!
!  Second-order, central differences adjoint vertical advective flux.
!
          DO i=Istr,Iend
!>          tl_FC(i,N(ng))=0.0_r8
!>
            ad_FC(i,N(ng))=0.0_r8
!>          tl_FC(i,0)=0.0_r8
!>
            ad_FC(i,0)=0.0_r8
          END DO
          DO k=1,N(ng)-1
            DO i=Istr,Iend
!>            tl_FC(i,k)=0.5_r8*                                        &
!>   &                   (tl_W(i,j,k)*                                  &
!>   &                    (t(i,j,k  ,3,itrc)+                           &
!>   &                     t(i,j,k+1,3,itrc))+                          &
!>   &                    W(i,j,k)*                                     &
!>   &                    (tl_t(i,j,k  ,3,itrc)+                        &
!>   &                     tl_t(i,j,k+1,3,itrc)))
!>
              adfac=0.5_r8*ad_FC(i,k)
              adfac1=adfac*W(i,j,k)
              ad_t(i,j,k  ,3,itrc)=ad_t(i,j,k  ,3,itrc)+adfac1
              ad_t(i,j,k+1,3,itrc)=ad_t(i,j,k+1,3,itrc)+adfac1
              ad_W(i,j,k)=ad_W(i,j,k)+                                  &
     &                    (t(i,j,k  ,3,itrc)+                           &
     &                     t(i,j,k+1,3,itrc))*adfac
              ad_FC(i,k)=0.0_r8
            END DO
          END DO
# elif defined TS_MPDATA
!
!  First_order, upstream differences vertical advective flux.
!
          DO i=I_RANGE
!>          tl_FC(i,N(ng))=0.0_r8
!>
            ad_FC(i,N(ng))=0.0_r8
!>          tl_FC(i,0)=0.0_r8
!>
            ad_FC(i,0)=0.0_r8
            DO k=1,N(ng)-1
              cff1=MAX(W(i,j,k),0.0_r8)
              cff2=MIN(W(i,j,k),0.0_r8)
!>            tl_FC(i,k)=tl_cff1*t(i,j,k  ,3,itrc)*oHz(i,j,k  )+        &
!>   &                   cff1*(tl_t(i,j,k  ,3,itrc)*oHz(i,j,k  )+       &
!>   &                         t(i,j,k  ,3,itrc)*tl_oHz(i,j,k  ))+      &
!>   &                   tl_cff2*t(i,j,k+1,3,itrc)*oHz(i,j,k+1)+        &
!>   &                   cff2*(tl_t(i,j,k+1,3,itrc)*oHz(i,j,k+1)+       &
!>   &                         t(i,j,k+1,3,itrc)*tl_oHz(i,j,k+1))
!>
              adfac1=cff1*ad_FC(i,k)
              adfac2=cff2*ad_FC(i,k)
              ad_t(i,j,k  ,3,itrc)=ad_t(i,j,k  ,3,itrc)+                &
     &                             oHz(i,j,k  )*adfac1
              ad_t(i,j,k+1,3,itrc)=ad_t(i,j,k+1,3,itrc)+                &
     &                             oHz(i,j,k+1)*adfac2
              ad_oHz(i,j,k  )=ad_oHz(i,j,k  )+                          &
     &                        t(i,j,k  ,3,itrc)*adfac1
              ad_oHz(i,j,k+1)=ad_oHz(i,j,k+1)+                          &
     &                        t(i,j,k+1,3,itrc)*adfac2
              ad_cff1=ad_cff1+t(i,j,k  ,3,itrc)*oHz(i,j,k  )*ad_FC(i,k)
              ad_cff2=ad_cff2+t(i,j,k+1,3,itrc)*oHz(i,j,k+1)*ad_FC(i,k)
              ad_FC(i,k)=0.0_r8
!>            tl_cff2=(0.5_r8+SIGN(0.5_r8,-W(i,j,k)))*tl_W(i,j,k)
!>            tl_cff1=(0.5_r8+SIGN(0.5_r8, W(i,j,k)))*tl_W(i,j,k)
!>
              ad_W(i,j,k)=ad_W(i,j,k)+                                  &
     &                    (0.5_r8+SIGN(0.5_r8,-W(i,j,k)))*ad_cff2+      &
     &                    (0.5_r8+SIGN(0.5_r8, W(i,j,k)))*ad_cff1
              ad_cff2=0.0_r8
              ad_cff1=0.0_r8
            END DO
          END DO
# else
!
!  Fourth-order, central differences adjoint vertical advective flux.
!
          cff1=0.5_r8
          cff2=7.0_r8/12.0_r8
          cff3=1.0_r8/12.0_r8
          DO i=Istr,Iend
!>          tl_FC(i,N(ng))=0.0_r8
!>
            ad_FC(i,N(ng))=0.0_r8
!>          tl_FC(i,N(ng)-1)=tl_W(i,j,N(ng)-1)*                         &
!>   &                       (cff1*t(i,j,N(ng)  ,3,itrc)+               &
!>   &                        cff2*t(i,j,N(ng)-1,3,itrc)-               &
!>   &                        cff3*t(i,j,N(ng)-2,3,itrc))+              &
!>   &                       W(i,j,N(ng)-1)*                            &
!>   &                       (cff1*tl_t(i,j,N(ng)  ,3,itrc)+            &
!>   &                        cff2*tl_t(i,j,N(ng)-1,3,itrc)-            &
!>   &                        cff3*tl_t(i,j,N(ng)-2,3,itrc))
!>
            adfac=W(i,j,N(ng)-1)*ad_FC(i,N(ng)-1)
            ad_W(i,j,N(ng)-1)=ad_W(i,j,N(ng)-1)+                        &
     &                        (cff1*t(i,j,N(ng)  ,3,itrc)+              &
     &                         cff2*t(i,j,N(ng)-1,3,itrc)-              &
     &                         cff3*t(i,j,N(ng)-2,3,itrc))*             &
     &                        ad_FC(i,N(ng)-1)
            ad_t(i,j,N(ng)-2,3,itrc)=ad_t(i,j,N(ng)-2,3,itrc)-          &
     &                               cff3*adfac
            ad_t(i,j,N(ng)-1,3,itrc)=ad_t(i,j,N(ng)-1,3,itrc)+          &
     &                               cff2*adfac
            ad_t(i,j,N(ng)  ,3,itrc)=ad_t(i,j,N(ng)  ,3,itrc)+          &
     &                               cff1*adfac
            ad_FC(i,N(ng)-1)=0.0_r8
!>          tl_FC(i,1)=tl_W(i,j,1)*                                     &
!>   &                 (cff1*t(i,j,1,3,itrc)+                           &
!>   &                  cff2*t(i,j,2,3,itrc)-                           &
!>   &                  cff3*t(i,j,3,3,itrc))+                          &
!>   &                 W(i,j,1)*                                        &
!>   &                 (cff1*tl_t(i,j,1,3,itrc)+                        &
!>   &                  cff2*tl_t(i,j,2,3,itrc)-                        &
!>   &                  cff3*tl_t(i,j,3,3,itrc))
!>
            adfac=W(i,j,1)*ad_FC(i,1)
            ad_W(i,j,1)=ad_W(i,j,1)+                                    &
     &                  (cff1*t(i,j,1,3,itrc)+                          &
     &                   cff2*t(i,j,2,3,itrc)-                          &
     &                   cff3*t(i,j,3,3,itrc))*ad_FC(i,1)
            ad_t(i,j,1,3,itrc)=ad_t(i,j,1,3,itrc)+cff1*adfac
            ad_t(i,j,2,3,itrc)=ad_t(i,j,2,3,itrc)+cff2*adfac
            ad_t(i,j,3,3,itrc)=ad_t(i,j,3,3,itrc)-cff3*adfac
            ad_FC(i,1)=0.0_r8
!>          tl_FC(i,0)=0.0_r8
!>
            ad_FC(i,0)=0.0_r8
          END DO
          DO k=2,N(ng)-2
            DO i=Istr,Iend
!>            tl_FC(i,k)=tl_W(i,j,k)*                                   &
!>   &                   (cff2*(t(i,j,k  ,3,itrc)+                      &
!>   &                          t(i,j,k+1,3,itrc))-                     &
!>   &                    cff3*(t(i,j,k-1,3,itrc)+                      &
!>   &                          t(i,j,k+2,3,itrc)))+                    &
!>   &                   W(i,j,k)*                                      &
!>   &                   (cff2*(tl_t(i,j,k  ,3,itrc)+                   &
!>   &                          tl_t(i,j,k+1,3,itrc))-                  &
!>   &                    cff3*(tl_t(i,j,k-1,3,itrc)+                   &
!>   &                          tl_t(i,j,k+2,3,itrc)))
!>
              adfac=W(i,j,k)*ad_FC(i,k)
              adfac1=adfac*cff2
              adfac2=adfac*cff3
              ad_W(i,j,k)=ad_W(i,j,k)+                                  &
     &                    (cff2*(t(i,j,k  ,3,itrc)+                     &
     &                           t(i,j,k+1,3,itrc))-                    &
     &                     cff3*(t(i,j,k-1,3,itrc)+                     &
     &                           t(i,j,k+2,3,itrc)))*ad_FC(i,k)
              ad_t(i,j,k-1,3,itrc)=ad_t(i,j,k-1,3,itrc)-adfac2
              ad_t(i,j,k  ,3,itrc)=ad_t(i,j,k  ,3,itrc)+adfac1
              ad_t(i,j,k+1,3,itrc)=ad_t(i,j,k+1,3,itrc)+adfac1
              ad_t(i,j,k+2,3,itrc)=ad_t(i,j,k+2,3,itrc)-adfac2
              ad_FC(i,k)=0.0_r8
            END DO
          END DO
# endif
        END DO
      END DO
!
!-----------------------------------------------------------------------
!  Time-step adjoint horizontal advection term.
!-----------------------------------------------------------------------
!
      T_LOOP : DO itrc=1,NT(ng)
        K_LOOP : DO k=1,N(ng)
!
!  Time-step adjoint horizontal advection term.
!
# ifdef TS_MPDATA
#   ifdef EW_PERIODIC
#    define I_RANGE Istr-2,Iend+2
#   else
#    define I_RANGE MAX(Istr-2,1),MIN(Iend+2,Lm(ng))
#   endif
#   ifdef NS_PERIODIC
#    define J_RANGE Jstr-2,Jend+2
#   else
#    define J_RANGE MAX(Jstr-2,1),MIN(Jend+2,Mm(ng))
#   endif
# else
#   define I_RANGE Istr,Iend
#   define J_RANGE Jstr,Jend
# endif

          DO j=J_RANGE
            DO i=I_RANGE
              cff=dt(ng)*pm(i,j)*pn(i,j)
#  ifdef TS_MPDATA
!>            tl_Ta(i,j,k,itrc)=tl_t(i,j,k,3,itrc)-tl_cff1
!>
              ad_t(i,j,k,3,itrc)=ad_t(i,j,k,3,itrc)+ad_Ta(i,j,k,itrc)
              ad_cff1=ad_cff1-ad_Ta(i,j,k,itrc)
              ad_Ta(i,j,k,itrc)=0.0_r8
#  else
#   ifdef DIAGNOSTICS_TS
!!            DiaTwrk(i,j,k,itrc,iThadv)=-cff1
#   endif
!>            tl_t(i,j,k,nnew,itrc)=tl_t(i,j,k,nnew,itrc)-tl_cff1
!>
              ad_cff1=ad_cff1-ad_t(i,j,k,nnew,itrc)
#  endif
!>            tl_cff1=cff*(tl_FX(i+1,j)-tl_FX(i,j)+                      &
!>   &                     tl_FE(i,j+1)-tl_FE(i,j))
!>
              adfac=cff*ad_cff1
              ad_FE(i,j  )=ad_FE(i,j  )-adfac
              ad_FE(i,j+1)=ad_FE(i,j+1)+adfac
              ad_FX(i  ,j)=ad_FX(i  ,j)-adfac
              ad_FX(i+1,j)=ad_FX(i+1,j)+adfac
              ad_cff1=0.0_r8
            END DO
          END DO
# undef I_RANGE
# undef J_RANGE

# ifdef TS_PSOURCE
!
!  Apply adjoint tracers point sources to the horizontal advection
!  terms.
!
#  ifdef TS_MPDATA
          DO is=1,Nsrc
            i=Isrc(is)
            j=Jsrc(is)
            IF (INT(Dsrc(is)).eq.0) THEN
              IF (((Istr-2.le.i).and.(i.le.Iend+2)).and.                &
     &            ((Jstr-2.le.j).and.(j.le.Jend+2))) THEN
                IF (Lsrc(is,itrc)) THEN
!>                tl_FX(i,j)=tl_Huon(i,j,k)*Tsrc(is,k,itrc)
!>
                  ad_Huon(i,j,k)=ad_Huon(i,j,k)+                        &
     &                           Tsrc(is,k,itrc)*ad_FX(i,j)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i  ,j).eq.0.0_r8).and.                     &
     &                (rmask(i-1,j).eq.1.0_r8)) THEN
!>                  tl_FX(i,j)=tl_Huon(i,j,k)*                          &
!>   &                         t(i-1,j,k,3,itrc)*oHz(i-1,j,k)+          &
!>   &                         Huon(i,j,k)*                             &
!>   &                         (tl_t(i-1,j,k,3,itrc)*oHz(i-1,j,k)+      &
!>   &                          t(i-1,j,k,3,itrc)*tl_oHz(i-1,j,k))
!>
                    adfac=Huon(i,j,k)*ad_FX(i,j)
                    ad_t(i-1,j,k,3,itrc)=ad_t(i-1,j,k,3,itrc)+          &
     &                                   oHz(i-1,j,k)*adfac
                    ad_oHz(i-1,j,k)=ad_oHz(i-1,j,k)+                    &
     &                              t(i-1,j,k,3,itrc)*adfac
                    ad_Huon(i,j,k)=ad_Huon(i,j,k)+                      &
     &                             t(i-1,j,k,3,itrc)*oHz(i-1,j,k)*      &
     &                             ad_FX(i,j)
                    ad_FX(i,j)=0.0_r8
                  ELSE IF ((rmask(i  ,j).eq.1.0_r8).and.                &
     &                     (rmask(i-1,j).eq.0.0_r8)) THEN
!>                  tl_FX(i,j)=tl_Huon(i,j,k)*                          &
!>   &                         t(i  ,j,k,3,itrc)*oHz(i  ,j,k)+          &
!>   &                         Huon(i,j,k)*                             &
!>   &                         (tl_t(i  ,j,k,3,itrc)*oHz(i  ,j,k)+      &
!>   &                          t(i  ,j,k,3,itrc)*tl_oHz(i  ,j,k))
!>
                    adfac=Huon(i,j,k)*ad_FX(i,j)
                    ad_t(i  ,j,k,3,itrc)=ad_t(i  ,j,k,3,itrc)+          &
     &                                   oHz(i  ,j,k)*adfac
                    ad_oHz(i  ,j,k)=ad_oHz(i  ,j,k)+                    &
     &                              t(i  ,j,k,3,itrc)*adfac
                    ad_Huon(i,j,k)=ad_Huon(i,j,k)+                      &
     &                             t(i  ,j,k,3,itrc)*oHz(i  ,j,k)*      &
     &                             ad_FX(i,j)
                    ad_FX(i,j)=0.0_r8
                  END IF
#   endif
                END IF
              END IF
            ELSE IF (INT(Dsrc(is)).eq.1) THEN
              IF (((Istr-2.le.i).and.(i.le.Iend+2)).and.                &
     &            ((Jstr-2.le.j).and.(j.le.Jend+2))) THEN
                IF (Lsrc(is,itrc)) THEN
!>                tl_FE(i,j)=tl_Hvom(i,j,k)*Tsrc(is,k,itrc)
!>
                  ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                        &
     &                           Tsrc(is,k,itrc)*ad_FE(i,j)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i,j  ).eq.0.0_r8).and.                     &
     &                (rmask(i,j-1).eq.1.0_r8)) THEN
!>                  tl_FE(i,j)=tl_Hvom(i,j,k)*                          &
!>   &                         t(i,j-1,k,3,itrc)*oHz(i,j-1,k)+          &
!>   &                         Hvom(i,j,k)*                             &
!>   &                         (tl_t(i,j-1,k,3,itrc)*oHz(i,j-1,k)+      &
!>   &                          t(i,j-1,k,3,itrc)*tl_oHz(i,j-1,k))
!>
                    adfac=Hvom(i,j,k)*ad_FE(i,j)
                    ad_t(i,j-1,k,3,itrc)=ad_t(i,j-1,k,3,itrc)+          &
     &                                   oHz(i,j-1,k)*adfac
                    ad_oHz(i,j-1,k)=ad_oHz(i,j-1,k)+                    &
     &                              t(i,j-1,k,3,itrc)*adfac
                    ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                      &
     &                             t(i,j-1,k,3,itrc)*oHz(i,j-1,k)*      &
     &                             ad_FE(i,j)
                    ad_FE(i,j)=0.0_r8
                  ELSE IF ((rmask(i,j  ).eq.1.0_r8).and.                &
     &                     (rmask(i,j-1).eq.0.0_r8)) THEN
!>                  tl_FE(i,j)=tl_Hvom(i,j,k)*                          &
!>   &                         t(i,j  ,k,3,itrc)*oHz(i,j  ,k)+          &
!>   &                         Hvom(i,j,k)*                             &
!>   &                         (tl_t(i,j  ,k,3,itrc)*oHz(i,j  ,k)+      &
!>   &                          t(i,j  ,k,3,itrc)*tl_oHz(i,j  ,k))
!>
                    adfac=Hvom(i,j,k)*ad_FE(i,j)
                    ad_t(i,j  ,k,3,itrc)=ad_t(i,j  ,k,3,itrc)+          &
     &                                   oHz(i,j  ,k)*adfac
                    ad_oHz(i,j  ,k)=ad_oHz(i,j  ,k)+                    &
     &                              t(i,j  ,k,3,itrc)*adfac
                    ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                      &
     &                             t(i,j  ,k,3,itrc)*oHz(i,j  ,k)*      &
     &                             ad_FE(i,j)
                    ad_FE(i,j)=0.0_r8
                  END IF
#   endif
                END IF
              END IF
            END IF
          END DO
#  else
          DO is=1,Nsrc
            i=Isrc(is)
            j=Jsrc(is)
            IF (INT(Dsrc(is)).eq.0) THEN
              IF (((Istr.le.i).and.(i.le.Iend+1)).and.                  &
     &            ((Jstr.le.j).and.(j.le.Jend))) THEN
                IF (Lsrc(is,itrc)) THEN
!>                tl_FX(i,j)=tl_Huon(i,j,k)*Tsrc(is,k,itrc)
!>
                  ad_Huon(i,j,k)=ad_Huon(i,j,k)+                        &
     &                           Tsrc(is,k,itrc)*ad_FX(i,j)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i  ,j).eq.0.0_r8).and.                     &
     &                (rmask(i-1,j).eq.1.0_r8)) THEN
!>                  tl_FX(i,j)=tl_Huon(i,j,k)*t(i-1,j,k,3,itrc)+        &
!>   &                         Huon(i,j,k)*tl_t(i-1,j,k,3,itrc)
!>
                    ad_t(i-1,j,k,3,itrc)=ad_t(i-1,j,k,3,itrc)+          &
     &                                   Huon(i,j,k)*ad_FX(i,j)
                    ad_Huon(i,j,k)=ad_Huon(i,j,k)+                      &
     &                             t(i-1,j,k,3,itrc)*ad_FX(i,j)
                    ad_FX(i,j)=0.0_r8
                  ELSE IF ((rmask(i  ,j).eq.1.0_r8).and.                &
     &                     (rmask(i-1,j).eq.0.0_r8)) THEN
!>                  tl_FX(i,j)=tl_Huon(i,j,k)*t(i  ,j,k,3,itrc)+        &
!>   &                         Huon(i,j,k)*tl_t(i  ,j,k,3,itrc)
!>
                    ad_t(i  ,j,k,3,itrc)=ad_t(i  ,j,k,3,itrc)+          &
     &                                   Huon(i,j,k)*ad_FX(i,j)
                    ad_Huon(i,j,k)=ad_Huon(i,j,k)+                      &
     &                             t(i  ,j,k,3,itrc)*ad_FX(i,j)
                    ad_FX(i,j)=0.0_r8
                  END IF
#   endif
                END IF
              END IF
            ELSE IF (INT(Dsrc(is)).eq.1) THEN
              IF (((Istr.le.i).and.(i.le.Iend)).and.                    &
     &            ((Jstr.le.j).and.(j.le.Jend+1))) THEN
                IF (Lsrc(is,itrc)) THEN
!>                tl_FE(i,j)=tl_Hvom(i,j,k)*Tsrc(is,k,itrc)
!>
                  ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                        &
     &                           Tsrc(is,k,itrc)*ad_FE(i,j)
#   ifdef MASKING
                ELSE
                  IF ((rmask(i,j  ).eq.0.0_r8).and.                     &
     &                (rmask(i,j-1).eq.1.0_r8)) THEN
!>                  tl_FE(i,j)=tl_Hvom(i,j,k)*t(i,j-1,k,3,itrc)+        &
!>   &                         Hvom(i,j,k)*tl_t(i,j-1,k,3,itrc)
!>
                    ad_t(i,j-1,k,3,itrc)=ad_t(i,j-1,k,3,itrc)+          &
     &                                   Hvom(i,j,k)*ad_FE(i,j)
                    ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                      &
     &                             t(i,j-1,k,3,itrc)*ad_FE(i,j)
                    ad_FE(i,j)=0.0_r8
                  ELSE IF ((rmask(i,j  ).eq.1.0_r8).and.                &
     &                     (rmask(i,j-1).eq.0.0_r8)) THEN
!>                  tl_FE(i,j)=tl_Hvom(i,j,k)*t(i,j  ,k,3,itrc)+        &
!>   &                         Hvom(i,j,k)*tl_t(i,j  ,k,3,itrc)
!>
                    ad_t(i,j  ,k,3,itrc)=ad_t(i,j  ,k,3,itrc)+          &
     &                                   Hvom(i,j,k)*ad_FE(i,j)
                    ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                      &
     &                             t(i,j  ,k,3,itrc)*ad_FE(i,j)
                    ad_FE(i,j)=0.0_r8
                  END IF
#   endif
                END IF
              END IF
            END IF
          END DO
#  endif
# endif

# if defined TS_C2HADVECTION
!
!  Second-order, centered differences adjoint horizontal advective fluxes.
!
          DO j=Jstr,Jend+1
            DO i=Istr,Iend
!>            tl_FE(i,j)=0.5_r8*                                        &
!>   &                   (tl_Hvom(i,j,k)*(t(i,j-1,k,3,itrc)+            &
!>   &                                    t(i,j  ,k,3,itrc))+           &
!>   &                    Hvom(i,j,k)*(tl_t(i,j-1,k,3,itrc)+            &
!>   &                                 tl_t(i,j  ,k,3,itrc)))
!>
              adfac=0.5_r8*ad_FE(i,j)
              adfac1=adfac*Hvom(i,j,k)
              ad_t(i,j-1,k,3,itrc)=ad_t(i,j-1,k,3,itrc)+adfac1
              ad_t(i,j  ,k,3,itrc)=ad_t(i,j  ,k,3,itrc)+adfac1
              ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                            &
     &                       (t(i,j-1,k,3,itrc)+                        &
     &                        t(i,j  ,k,3,itrc))*adfac
              ad_FE(i,j)=0.0_r8
            END DO
          END DO
          DO j=Jstr,Jend
            DO i=Istr,Iend+1
!>            tl_FX(i,j)=0.5_r8*                                        &
!>   &                   (tl_Huon(i,j,k)*(t(i-1,j,k,3,itrc)+            &
!>   &                                    t(i  ,j,k,3,itrc))+           &
!>   &                    Huon(i,j,k)*(tl_t(i-1,j,k,3,itrc)+            &
!>   &                                 tl_t(i  ,j,k,3,itrc)))
!>
              adfac=0.5_r8*ad_FX(i,j)
              adfac1=adfac*Huon(i,j,k)
              ad_t(i-1,j,k,3,itrc)=ad_t(i-1,j,k,3,itrc)+adfac1
              ad_t(i  ,j,k,3,itrc)=ad_t(i  ,j,k,3,itrc)+adfac1
              ad_Huon(i,j,k)=ad_Huon(i,j,k)+                            &
     &                       (t(i-1,j,k,3,itrc)+                        &
     &                        t(i  ,j,k,3,itrc))*adfac
              ad_FX(i,j)=0.0_r8
            END DO
          END DO
# elif defined TS_MPDATA
!
!  First-order, upstream differences adjoint horizontal advective
!  fluxes.
!
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-2,Iend+2
#  else
#   define I_RANGE MAX(Istr-2,1),MIN(Iend+2,Lm(ng))
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-2,Jend+3
#  else
#   define J_RANGE MAX(Jstr-2,1),MIN(Jend+3,Mm(ng)+1)
#  endif
          DO j=J_RANGE
            DO i=I_RANGE
              cff1=MAX(Hvom(i,j,k),0.0_r8)
              cff2=MIN(Hvom(i,j,k),0.0_r8)
!>            tl_FE(i,j)=tl_cff1*t(i,j-1,k,3,itrc)*oHz(i,j-1,k)+        &
!>   &                   cff1*(tl_t(i,j-1,k,3,itrc)*oHz(i,j-1,k)+       &
!>   &                         t(i,j-1,k,3,itrc)*tl_oHz(i,j-1,k))+      &
!>   &                   tl_cff2*t(i,j  ,k,3,itrc)*oHz(i,j  ,k)+        &
!>   &                   cff2*(tl_t(i,j  ,k,3,itrc)*oHz(i,j  ,k)+       &
!>   &                         t(i,j  ,k,3,itrc)*tl_oHz(i,j  ,k))
!>
              adfac1=cff1*ad_FE(i,j)
              adfac2=cff2*ad_FE(i,j)
              ad_t(i,j-1,k,3,itrc)=ad_t(i,j-1,k,3,itrc)+                &
     &                             oHz(i,j-1,k)*adfac1
              ad_t(i,j  ,k,3,itrc)=ad_t(i,j  ,k,3,itrc)+                &
     &                             oHz(i,j  ,k)*adfac2
              ad_oHz(i,j  ,k)=ad_oHz(i,j  ,k)+                          &
     &                        t(i,j  ,k,3,itrc)*adfac2
              ad_oHz(i,j-1,k)=ad_oHz(i,j-1,k)+                          &
     &                        t(i,j-1,k,3,itrc)*adfac1
              ad_cff1=ad_cff1+t(i,j-1,k,3,itrc)*oHz(i,j-1,k)*ad_FE(i,j)
              ad_cff2=ad_cff2+t(i,j  ,k,3,itrc)*oHz(i,j  ,k)*ad_FE(i,j)
              ad_FE(i,j)=0.0_r8
!>            tl_cff2=(0.5_r8+SIGN(0.5_r8,-Hvom(i,j,k)))*tl_Hvom(i,j,k)
!>            tl_cff1=(0.5_r8+SIGN(0.5_r8, Hvom(i,j,k)))*tl_Hvom(i,j,k)
!>
              ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                            &
     &                       (0.5_r8+SIGN(0.5_r8,-Hvom(i,j,k)))*        &
     &                       ad_cff2+                                   &
     &                       (0.5_r8+SIGN(0.5_r8, Hvom(i,j,k)))*        &
     &                       ad_cff1
              ad_cff2=0.0_r8
              ad_cff1=0.0_r8
            END DO
          END DO
#  undef I_RANGE
#  undef J_RANGE
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-2,Iend+3
#  else
#   define I_RANGE MAX(Istr-2,1),MIN(Iend+3,Lm(ng)+1)
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-2,Jend+2
#  else
#   define J_RANGE MAX(Jstr-2,1),MIN(Jend+2,Mm(ng))
#  endif
          DO j=J_RANGE
             DO i=I_RANGE
              cff1=MAX(Huon(i,j,k),0.0_r8)
              cff2=MIN(Huon(i,j,k),0.0_r8)
!>            tl_FX(i,j)=tl_cff1*t(i-1,j,k,3,itrc)*oHz(i-1,j,k)+        &
!>   &                   cff1*(tl_t(i-1,j,k,3,itrc)*oHz(i-1,j,k)+       &
!>   &                         t(i-1,j,k,3,itrc)*tl_oHz(i-1,j,k))+      &
!>   &                   tl_cff2*t(i  ,j,k,3,itrc)*oHz(i  ,j,k)+        &
!>   &                   cff2*(tl_t(i  ,j,k,3,itrc)*oHz(i  ,j,k)+       &
!>   &                         t(i  ,j,k,3,itrc)*tl_oHz(i  ,j,k))
!>
              adfac1=cff1*ad_FX(i,j)
              adfac2=cff2*ad_FX(i,j)
              ad_t(i  ,j,k,3,itrc)=ad_t(i  ,j,k,3,itrc)+                &
     &                             oHz(i  ,j,k)*adfac2
              ad_t(i-1,j,k,3,itrc)=ad_t(i-1,j,k,3,itrc)+                &
     &                             oHz(i-1,j,k)*adfac1
              ad_oHz(i  ,j,k)=ad_oHz(i  ,j,k)+                          &
     &                        t(i  ,j,k,3,itrc)*adfac2
              ad_oHz(i-1,j,k)=ad_oHz(i-1,j,k)+                          &
     &                        t(i-1,j,k,3,itrc)*adfac1
              ad_cff1=ad_cff1+t(i-1,j,k,3,itrc)*oHz(i-1,j,k)*ad_FX(i,j)
              ad_cff2=ad_cff2+t(i  ,j,k,3,itrc)*oHz(i  ,j,k)*ad_FX(i,j)
              ad_FX(i,j)=0.0_r8
!>            tl_cff2=(0.5_r8+SIGN(0.5_r8,-Huon(i,j,k)))*tl_Huon(i,j,k)
!>            tl_cff1=(0.5_r8+SIGN(0.5_r8, Huon(i,j,k)))*tl_Huon(i,j,k)
!>
              ad_Huon(i,j,k)=ad_Huon(i,j,k)+                            &
     &                       (0.5_r8+SIGN(0.5_r8,-Huon(i,j,k)))*        &
     &                       ad_cff2+                                   &
     &                       (0.5_r8+SIGN(0.5_r8, Huon(i,j,k)))*        &
     &                       ad_cff1
              ad_cff2=0.0_r8
              ad_cff1=0.0_r8
            END DO
          END DO
#  undef I_RANGE
#  undef J_RANGE
# else
!
#  if defined TS_U3HADVECTION
!  Third-order, uptream-biased horizontal advective fluxes.
#  elif defined TS_A4HADVECTION
!  Fourth-order, Akima horizontal advective fluxes.
#  else
!  Fourth-order, centered differences horizontal advective fluxes.
#  endif
!  Compute BASIC STATE "curv" and "grad" scratch arrays.
!
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-1,Jend+2
#  else
#   define J_RANGE MAX(Jstr-1,1),MIN(Jend+2,Mm(ng)+1)
#  endif
          DO j=J_RANGE
            DO i=Istr,Iend
              FE(i,j)=t(i,j  ,k,3,itrc)-                                &
     &                t(i,j-1,k,3,itrc)
#  ifdef MASKING
              FE(i,j)=FE(i,j)*vmask(i,j)
#  endif
            END DO
          END DO
#  ifndef NS_PERIODIC
          IF (SOUTHERN_EDGE) THEN
            DO i=Istr,Iend
              FE(i,Jstr-1)=FE(i,Jstr)
            END DO
          END IF
          IF (NORTHERN_EDGE) THEN
            DO i=Istr,Iend
              FE(i,Jend+2)=FE(i,Jend+1)
            END DO
          END IF
#  endif
!
          DO j=Jstr-1,Jend+1
            DO i=Istr,Iend
#  if defined TS_U3HADVECTION
              curv(i,j)=FE(i,j+1)-FE(i,j)
#  elif defined TS_A4HADVECTION
              cff=2.0_r8*FE(i,j+1)*FE(i,j)
              IF (cff.gt.eps) THEN
                grad(i,j)=cff/(FE(i,j+1)+FE(i,j))
              ELSE
                grad(i,j)=0.0_r8
              END IF
#  else
              grad(i,j)=0.5_r8*(FE(i,j+1)+FE(i,j))
#  endif
            END DO
          END DO
!
          cff1=1.0_r8/6.0_r8
          cff2=1.0_r8/3.0_r8
          DO j=Jstr,Jend+1
            DO i=Istr,Iend
#  ifdef TS_U3HADVECTION
!>            tl_FE(i,j)=0.5_r8*                                        &
!>   &                   (tl_Hvom(i,j,k)*                               &
!>   &                    (t(i,j-1,k,3,itrc)+                           &
!>   &                     t(i,j  ,k,3,itrc))+                          &
!>   &                    Hvom(i,j,k)*                                  &
!>   &                    (tl_t(i,j-1,k,3,itrc)+                        &
!>   &                     tl_t(i,j  ,k,3,itrc)))-                      &
!>   &                   cff1*                                          &
!>   &                   (tl_curv(i,j-1)*MAX(Hvom(i,j,k),0.0_r8)+       &
!>   &                    curv(i,j-1)*                                  &
!>   &                    (0.5_r8+SIGN(0.5_r8, Hvom(i,j,k)))*           &
!>   &                    tl_Hvom(i,j,k)+                               &
!>   &                    tl_curv(i,j  )*MIN(Hvom(i,j,k),0.0_r8)+       &
!>   &                    curv(i,j  )*                                  &
!>   &                    (0.5_r8+SIGN(0.5_r8,-Hvom(i,j,k)))*           &
!>   &                    tl_Hvom(i,j,k))
!>
              adfac=0.5_r8*ad_FE(i,j)
              adfac1=adfac*Hvom(i,j,k)
              adfac2=cff1*ad_FE(i,j)
              ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                            &
     &                       (t(i,j-1,k,3,itrc)+                        &
     &                        t(i,j  ,k,3,itrc))*adfac-                 &
     &                       (curv(i,j-1)*                              &
     &                        (0.5_r8+SIGN(0.5_r8, Hvom(i,j,k)))+       &
     &                        curv(i,j  )*                              &
     &                        (0.5_r8+SIGN(0.5_r8,-Hvom(i,j,k))))*      &
     &                       adfac2
              ad_t(i,j-1,k,3,itrc)=ad_t(i,j-1,k,3,itrc)+adfac1
              ad_t(i,j  ,k,3,itrc)=ad_t(i,j  ,k,3,itrc)+adfac1
              ad_curv(i,j-1)=ad_curv(i,j-1)-                            &
     &                       MAX(Hvom(i,j,k),0.0_r8)*adfac2
              ad_curv(i,j  )=ad_curv(i,j  )-                            &
     &                       MIN(Hvom(i,j,k),0.0_r8)*adfac2
              ad_FE(i,j)=0.0_r8
#  else
!>            tl_FE(i,j)=0.5_r8*                                        &
!>   &                   (tl_Hvom(i,j,k)*                               &
!>   &                    (t(i,j-1,k,3,itrc)+                           &
!>   &                     t(i,j  ,k,3,itrc)-                           &
!>   &                     cff2*(grad(i,j  )-                           &
!>   &                           grad(i,j-1)))+                         &
!>   &                    Hvom(i,j,k)*                                  &
!>   &                    (tl_t(i,j-1,k,3,itrc)+                        &
!>   &                     tl_t(i,j  ,k,3,itrc)-                        &
!>   &                     cff2*(tl_grad(i,j  )-                        &
!>   &                           tl_grad(i,j-1))))
!>
              adfac=0.5_r8*ad_FE(i,j)
              adfac1=adfac*Hvom(i,j,k)
              adfac2=adfac1*cff2
              ad_Hvom(i,j,k)=ad_Hvom(i,j,k)+                            &
     &                       adfac*(t(i,j-1,k,3,itrc)+                  &
     &                              t(i,j  ,k,3,itrc)-                  &
     &                              cff2*(grad(i,j  )-                  &
     &                                    grad(i,j-1)))
              ad_t(i,j-1,k,3,itrc)=ad_t(i,j-1,k,3,itrc)+adfac1
              ad_t(i,j  ,k,3,itrc)=ad_t(i,j  ,k,3,itrc)+adfac1
              ad_grad(i,j-1)=ad_grad(i,j-1)+adfac2
              ad_grad(i,j  )=ad_grad(i,j  )-adfac2
              ad_FE(i,j)=0.0_r8
#  endif
            END DO
          END DO
!
          DO j=Jstr-1,Jend+1
            DO i=Istr,Iend
#  if defined TS_U3HADVECTION
!>            tl_curv(i,j)=tl_FE(i,j+1)-tl_FE(i,j)
!>
              ad_FE(i,j  )=ad_FE(i,j  )-ad_curv(i,j)
              ad_FE(i,j+1)=ad_FE(i,j+1)+ad_curv(i,j)
              ad_curv(i,j)=0.0_r8
#  elif defined TS_A4HADVECTION
              cff=2.0_r8*FE(i,j+1)*FE(i,j)
              IF (cff.gt.eps) THEN
!>              tl_grad(i,j)=((FE(i,j+1)+FE(i,j))*tl_cff-               &
!>   &                        cff*(tl_FE(i,j+1)+tl_FE(i,j)))/           &
!>   &                       ((FE(i,j+1)+FE(i,j))*(FE(i,j+1)+FE(i,j)))
!>
                adfac=ad_grad(i,j)/                                     &
     &                ((FE(i,j+1)+FE(i,j))*(FE(i,j+1)+FE(i,j)))
                adfac1=adfac*cff
                ad_FE(i,j  )=ad_FE(i,j)-adfac1
                ad_FE(i,j+1)=ad_FE(i,j+1)-adfac1
                ad_cff=ad_cff+(FE(i,j+1)+FE(i,j))*adfac
                ad_grad(i,j)=0.0_r8
              ELSE
!>              tl_grad(i,j)=0.0_r8
!>
                ad_grad(i,j)=0.0_r8
              END IF
!>            tl_cff=2.0_r8*(tl_FE(i,j+1)*FE(i,j)+                      &
!>   &                       FE(i,j+1)*tl_FE(i,j))
!>
              adfac=2.0_r8*ad_cff
              ad_FE(i,j  )=ad_FE(i,j  )+FE(i,j+1)*adfac
              ad_FE(i,j+1)=ad_FE(i,j+1)+FE(i,j  )*adfac
              ad_cff=0.0_r8
#  else
!>            tl_grad(i,j)=0.5_r8*(tl_FE(i,j+1)+tl_FE(i,j))
!>
              adfac=0.5_r8*ad_grad(i,j)
              ad_FE(i,j  )=ad_FE(i,j  )+adfac
              ad_FE(i,j+1)=ad_FE(i,j+1)+adfac
              ad_grad(i,j)=0.0_r8
#  endif
            END DO
          END DO
#  ifndef NS_PERIODIC
          IF (NORTHERN_EDGE) THEN
            DO i=Istr,Iend
!>            tl_FE(i,Jend+2)=tl_FE(i,Jend+1)
!>
              ad_FE(i,Jend+1)=ad_FE(i,Jend+1)+ad_FE(i,Jend+2)
              ad_FE(i,Jend+2)=0.0_r8
            END DO
          END IF
          IF (SOUTHERN_EDGE) THEN
            DO i=Istr,Iend
!>            tl_FE(i,Jstr-1)=tl_FE(i,Jstr)
!>
              ad_FE(i,Jstr)=ad_FE(i,Jstr)+ad_FE(i,Jstr-1)
              ad_FE(i,Jstr-1)=0.0_r8
            END DO
          END IF
#  endif
!
          DO j=J_RANGE
            DO i=Istr,Iend
#  ifdef MASKING
!>            tl_FE(i,j)=tl_FE(i,j)*vmask(i,j)
!>
              ad_FE(i,j)=ad_FE(i,j)*vmask(i,j)
#  endif
!>            tl_FE(i,j)=tl_t(i,j  ,k,3,itrc)-                          &
!>   &                   tl_t(i,j-1,k,3,itrc)
!>
              ad_t(i,j-1,k,3,itrc)=ad_t(i,j-1,k,3,itrc)-ad_FE(i,j)
              ad_t(i,j  ,k,3,itrc)=ad_t(i,j  ,k,3,itrc)+ad_FE(i,j)
              ad_FE(i,j)=0.0_r8
            END DO
          END DO
#  undef J_RANGE
!
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-1,Iend+2
#  else
#   define I_RANGE MAX(Istr-1,1),MIN(Iend+2,Lm(ng)+1)
#  endif
          DO j=Jstr,Jend
            DO i=I_RANGE
              FX(i,j)=t(i  ,j,k,3,itrc)-                                &
     &                t(i-1,j,k,3,itrc)
#  ifdef MASKING
              FX(i,j)=FX(i,j)*umask(i,j)
#  endif
            END DO
          END DO
#  ifndef EW_PERIODIC
          IF (WESTERN_EDGE) THEN
            DO j=Jstr,Jend
              FX(Istr-1,j)=FX(Istr,j)
            END DO
          END IF
          IF (EASTERN_EDGE) THEN
            DO j=Jstr,Jend
              FX(Iend+2,j)=FX(Iend+1,j)
            END DO
          END IF
#  endif
!
          DO j=Jstr,Jend
            DO i=Istr-1,Iend+1
#  if defined TS_U3HADVECTION
              curv(i,j)=FX(i+1,j)-FX(i,j)
#  elif defined TS_A4HADVECTION
              cff=2.0_r8*FX(i+1,j)*FX(i,j)
              IF (cff.gt.eps) THEN
                grad(i,j)=cff/(FX(i+1,j)+FX(i,j))
              ELSE
                grad(i,j)=0.0_r8
              END IF
#  else
              grad(i,j)=0.5_r8*(FX(i+1,j)+FX(i,j))
#  endif
            END DO
          END DO
!
          cff1=1.0_r8/6.0_r8
          cff2=1.0_r8/3.0_r8
          DO j=Jstr,Jend
            DO i=Istr,Iend+1
#  ifdef TS_U3HADVECTION
!>            tl_FX(i,j)=0.5_r8*                                        &
!>   &                   (tl_Huon(i,j,k)*                               &
!>   &                    (t(i-1,j,k,3,itrc)+                           &
!>   &                     t(i  ,j,k,3,itrc))+                          &
!>   &                    Huon(i,j,k)*                                  &
!>   &                    (tl_t(i-1,j,k,3,itrc)+                        &
!>   &                     tl_t(i  ,j,k,3,itrc)))-                      &
!>   &                   cff1*                                          &
!>   &                   (tl_curv(i-1,j)*MAX(Huon(i,j,k),0.0_r8)+       &
!>   &                    curv(i-1,j)*                                  &
!>   &                    (0.5_r8+SIGN(0.5_r8, Huon(i,j,k)))*           &
!>   &                    tl_Huon(i,j,k)+                               &
!>   &                    tl_curv(i  ,j)*MIN(Huon(i,j,k),0.0_r8)+       &
!>   &                    curv(i  ,j)*                                  &
!>   &                    (0.5_r8+SIGN(0.5_r8,-Huon(i,j,k)))*           &
!>   &                    tl_Huon(i,j,k))
!>
              adfac=0.5_r8*ad_FX(i,j)
              adfac1=adfac*Huon(i,j,k)
              adfac2=cff1*ad_FX(i,j)
              ad_Huon(i,j,k)=ad_Huon(i,j,k)+                            &
     &                       (t(i-1,j,k,3,itrc)+                        &
     &                        t(i  ,j,k,3,itrc))*adfac-                 &
     &                       (curv(i-1,j)*                              &
     &                        (0.5_r8+SIGN(0.5_r8, Huon(i,j,k)))+       &
     &                        curv(i  ,j)*                              &
     &                        (0.5_r8+SIGN(0.5_r8,-Huon(i,j,k))))*      &
     &                       adfac2
              ad_t(i-1,j,k,3,itrc)=ad_t(i-1,j,k,3,itrc)+adfac1
              ad_t(i  ,j,k,3,itrc)=ad_t(i  ,j,k,3,itrc)+adfac1
              ad_curv(i-1,j)=ad_curv(i-1,j)-                            &
     &                       MAX(Huon(i,j,k),0.0_r8)*adfac2
              ad_curv(i  ,j)=ad_curv(i  ,j)-                            &
     &                       MIN(Huon(i,j,k),0.0_r8)*adfac2
              ad_FX(i,j)=0.0_r8
#  else
!>            tl_FX(i,j)=0.5_r8*                                        &
!>   &                   (tl_Huon(i,j,k)*                               &
!>   &                    (t(i-1,j,k,3,itrc)+                           &
!>   &                     t(i  ,j,k,3,itrc)-                           &
!>   &                     cff2*(grad(i  ,j)-                           &
!>   &                           grad(i-1,j)))+                         &
!>   &                    Huon(i,j,k)*                                  &
!>   &                    (tl_t(i-1,j,k,3,itrc)+                        &
!>   &                     tl_t(i  ,j,k,3,itrc)-                        &
!>   &                     cff2*(tl_grad(i  ,j)-                        &
!>   &                           tl_grad(i-1,j))))
!>
              adfac=0.5_r8*ad_FX(i,j)
              adfac1=adfac*Huon(i,j,k)
              adfac2=adfac1*cff2
              ad_Huon(i,j,k)=ad_Huon(i,j,k)+                            &
     &                       adfac*(t(i-1,j,k,3,itrc)+                  &
     &                              t(i  ,j,k,3,itrc)-                  &
     &                              cff2*(grad(i  ,j)-                  &
     &                                    grad(i-1,j)))
              ad_t(i-1,j,k,3,itrc)=ad_t(i-1,j,k,3,itrc)+adfac1
              ad_t(i  ,j,k,3,itrc)=ad_t(i  ,j,k,3,itrc)+adfac1
              ad_grad(i-1,j)=ad_grad(i-1,j)+adfac2
              ad_grad(i  ,j)=ad_grad(i  ,j)-adfac2
              ad_FX(i,j)=0.0_r8
#  endif
            END DO
          END DO
!
          DO j=Jstr,Jend
            DO i=Istr-1,Iend+1
#  if defined TS_U3HADVECTION
!>            tl_curv(i,j)=tl_FX(i+1,j)-tl_FX(i,j)
!>
              ad_FX(i  ,j)=ad_FX(i  ,j)-ad_curv(i,j)
              ad_FX(i+1,j)=ad_FX(i+1,j)+ad_curv(i,j)
              ad_curv(i,j)=0.0_r8
#  elif defined TS_A4HADVECTION
              cff=2.0_r8*FX(i+1,j)*FX(i,j)
              IF (cff.gt.eps) THEN
!>              tl_grad(i,j)=((FX(i+1,j)+FX(i,j))*tl_cff-               &
!>   &                        cff*(tl_FX(i+1,j)+tl_FX(i,j)))/           &
!>   &                       ((FX(i+1,j)+FX(i,j))*(FX(i+1,j)+FX(i,j)))
!>
                adfac=ad_grad(i,j)/                                     &
     &                ((FX(i+1,j)+FX(i,j))*(FX(i+1,j)+FX(i,j)))
                adfac1=adfac*cff
                ad_FX(i  ,j)=ad_FX(i  ,j)-adfac1        
                ad_FX(i+1,j)=ad_FX(i+1,j)-adfac1
                ad_cff=ad_cff+(FX(i+1,j)+FX(i,j))*adfac
                ad_grad(i,j)=0.0_r8
              ELSE
!>              tl_grad(i,j)=0.0_r8
!>
                ad_grad(i,j)=0.0_r8
              END IF
#  else
!>            tl_grad(i,j)=0.5_r8*(tl_FX(i+1,j)+tl_FX(i,j))
!>
              adfac=0.5_r8*ad_grad(i,j)
              ad_FX(i  ,j)=ad_FX(i  ,j)+adfac
              ad_FX(i+1,j)=ad_FX(i+1,j)+adfac
              ad_grad(i,j)=0.0_r8
#  endif
!>            tl_cff=2.0_r8*(tl_FX(i+1,j)*FX(i,j)+                      &
!>   &                       FX(i+1,j)*tl_FX(i,j))
!>
              adfac=2.0_r8*ad_cff
              ad_FX(i  ,j)=ad_FX(i  ,j)+FX(i+1,j)*adfac
              ad_FX(i+1,j)=ad_FX(i+1,j)+FX(i  ,j)*adfac
              ad_cff=0.0_r8
            END DO
          END DO
#  ifndef EW_PERIODIC
          IF (EASTERN_EDGE) THEN
            DO j=Jstr,Jend
!>            tl_FX(Iend+2,j)=tl_FX(Iend+1,j)
!>
              ad_FX(Iend+1,j)=ad_FX(Iend+1,j)+ad_FX(Iend+2,j)
              ad_FX(Iend+2,j)=0.0_r8
            END DO
          END IF
          IF (WESTERN_EDGE) THEN
            DO j=Jstr,Jend
!>            tl_FX(Istr-1,j)=tl_FX(Istr,j)
!>
              ad_FX(Istr,j)=ad_FX(Istr,j)+ad_FX(Istr-1,j)
              ad_FX(Istr-1,j)=0.0_r8
            END DO
          END IF
#  endif
!
          DO j=Jstr,Jend
            DO i=I_RANGE
#  ifdef MASKING
!>            tl_FX(i,j)=tl_FX(i,j)*umask(i,j)
!>
              ad_FX(i,j)=ad_FX(i,j)*umask(i,j)
#  endif
!>            tl_FX(i,j)=tl_t(i  ,j,k,nstp,itrc)-                       &
!>   &                   tl_t(i-1,j,k,nstp,itrc)
!>
              ad_t(i-1,j,k,3,itrc)=ad_t(i-1,j,k,3,itrc)-ad_FX(i,j)
              ad_t(i  ,j,k,3,itrc)=ad_t(i  ,j,k,3,itrc)+ad_FX(i,j)
              ad_FX(i,j)=0.0_r8
            END DO
          END DO
#  undef I_RANGE
# endif
        END DO K_LOOP
      END DO T_LOOP
!
!  Compute adjoint inverse thickness.
!
# ifdef TS_MPDATA
#  ifdef EW_PERIODIC
#   define I_RANGE Istr-3,Iend+2
#  else
#   define I_RANGE MAX(Istr-3,1),MIN(Iend+2,Lm(ng))
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr-3,Jend+2
#  else
#   define J_RANGE MAX(Jstr-3,1),MIN(Jend+2,Mm(ng))
#  endif
# else
#  define I_RANGE Istr,Iend
#  define J_RANGE Jstr,Jend
# endif

      DO k=1,N(ng)
        DO j=J_RANGE
          DO i=I_RANGE
            oHz(i,j,k)=1.0_r8/Hz(i,j,k)
!>          tl_oHz(i,j,k)=-oHz(i,j,k)*oHz(i,j,k)*tl_Hz(i,j,k)
!>
            ad_Hz(i,j,k)=ad_Hz(i,j,k)-                                  &
     &                   oHz(i,j,k)*oHz(i,j,k)*ad_oHz(i,j,k)
            ad_oHz(i,j,k)=0.0_r8
          END DO
        END DO
      END DO
#undef I_RANGE
#undef J_RANGE
      RETURN
      END SUBROUTINE ad_step3d_t_tile
#endif
      END MODULE ad_step3d_t_mod
