#include "cppdefs.h"
      SUBROUTINE mod_arrays (allocate_vars)
!
!=======================================================================
!  Copyright (c) 2005 ROMS/TOMS Group                                  !
!================================================== Hernan G. Arango ===
!                                                                      !
!  This routine routine allocates and initializa model state arrays    !
!  for each nested and/or multiple connected grids.                    !
!                                                                      !
!=======================================================================
!
      USE mod_param
      USE mod_parallel
!
#ifdef AVERAGES
      USE mod_average, ONLY : allocate_average, initialize_average
#endif
      USE mod_boundary, ONLY : allocate_boundary, initialize_boundary
#if defined CLIMATOLOGY || defined AD_SENSITIVITY
      USE mod_clima, ONLY : allocate_clima, initialize_clima
#endif
#ifdef SOLVE3D
      USE mod_coupling, ONLY : allocate_coupling, initialize_coupling
#endif
#ifdef DIAGNOSTICS
      USE mod_diags, ONLY : allocate_diags, initialize_diags
#endif
      USE mod_forces, ONLY : allocate_forces, initialize_forces
      USE mod_grid, ONLY : allocate_grid, initialize_grid
      USE mod_mixing, ONLY : allocate_mixing, initialize_mixing
#if defined ASSIMILATION || defined NUDGING
      USE mod_obs, ONLY : allocate_obs, initialize_obs
#endif
      USE mod_ocean, ONLY : allocate_ocean, initialize_ocean
#if defined UV_PSOURCE || defined TS_PSOURCE
      USE mod_sources, ONLY : allocate_sources
#endif
#if defined SSH_TIDES || defined UV_TIDES
      USE mod_tides, ONLY : allocate_tides, initialize_tides
#endif
#ifdef BBL_MODEL
      USE mod_bbl, ONLY : allocate_bbl, initialize_bbl
#endif
!
      implicit none
!
!  Imported variable declarations
!
      logical, intent(in) :: allocate_vars
!
!  Local variable declarations.
!
      integer :: ng, gtype
      integer :: LBi, UBi, LBj, UBj
      integer :: Itile, Jtile, tile, subs, thread

      integer, parameter :: model = 0
!
!-----------------------------------------------------------------------
!  Allocate model structures.
!-----------------------------------------------------------------------
!
      IF (allocate_vars) then
        tile=-1
        gtype=0
        DO ng=1,Ngrids
          CALL get_bounds (ng, TILE, gtype, NghostPoints,               &
     &                     Itile, Jtile, LBi, UBi, LBj, UBj)
#ifdef AVERAGES
          CALL allocate_average (ng, LBi, UBi, LBj, UBj)
#endif
          CALL allocate_boundary (ng)
#ifdef BBL_MODEL
          CALL allocate_bbl (ng, LBi, UBi, LBj, UBj)
#endif
#if defined CLIMATOLOGY || defined AD_SENSITIVITY
          CALL allocate_clima (ng, LBi, UBi, LBj, UBj)
#endif
#ifdef SOLVE3D
          CALL allocate_coupling (ng, LBi, UBi, LBj, UBj)
#endif
#ifdef DIAGNOSTICS
          CALL allocate_diags (ng, LBi, UBi, LBj, UBj)
#endif
          CALL allocate_forces (ng, LBi, UBi, LBj, UBj)
          CALL allocate_grid (ng, LBi, UBi, LBj, UBj)
          CALL allocate_mixing (ng, LBi, UBi, LBj, UBj)
#if defined ASSIMILATION || defined NUDGING
          CALL allocate_obs (ng, LBi, UBi, LBj, UBj)
#endif
          CALL allocate_ocean  (ng, LBi, UBi, LBj, UBj)
#if defined SSH_TIDES || defined UV_TIDES
          CALL allocate_tides (ng, LBi, UBi, LBj, UBj)
#endif
#if defined UV_PSOURCE || defined TS_PSOURCE
          CALL allocate_sources (ng)
#endif
        END DO
      END IF
!
!-----------------------------------------------------------------------
!  Allocate and intialize variables within structures for each grid.
!-----------------------------------------------------------------------
!
!$OMP PARALLEL DO PRIVATE(ng,thread,subs,tile) SHARED(numthreads)
      DO thread=0,numthreads-1
        DO ng=1,Ngrids
#if defined _OPENMP || defined DISTRIBUTE
          subs=NtileX(ng)*NtileE(ng)/numthreads
#else
          subs=1
#endif
          DO tile=subs*thread,subs*(thread+1)-1
#ifdef AVERAGES
            CALL initialize_average (ng, TILE)
#endif
#ifdef BBL_MODEL
            CALL initialize_bbl (ng, TILE)
#endif
            CALL initialize_boundary (ng, TILE)
#if defined CLIMATOLOGY || defined AD_SENSITIVITY
            CALL initialize_clima (ng, TILE)
#endif
#ifdef SOLVE3D
            CALL initialize_coupling (ng, TILE)
#endif
#ifdef DIAGNOSTICS
            CALL initialize_diags (ng, TILE)
#endif
            CALL initialize_forces (ng, TILE)
            CALL initialize_grid (ng, TILE)
            CALL initialize_mixing (ng, TILE)
#if defined ASSIMILATION || defined NUDGING
            CALL initialize_obs (ng, TILE)
#endif
            CALL initialize_ocean (ng, TILE, model)
#if defined SSH_TIDES || defined UV_TIDES
            CALL initialize_tides (ng, TILE)
#endif
          END DO
        END DO
      END DO
!$OMP END PARALLEL DO

      RETURN
      END SUBROUTINE mod_arrays
