#include "cppdefs.h"
      SUBROUTINE biology (ng,tile)
!
!**************************************************** Ruoying He     ***
!  Copyright (c) 2005 ROMS/TOMS Group                                  !
!************************************************** Hernan G. Arango ***
!                                                                      !
!  Alexandrium Fundyense (red tides) Model for the Gulf of Maine       !
!                                                                      !
!  This routine computes the biological sources and sinks and adds     !
!  to the global biological fields.                                    !
!                                                                      !
!  Reference:                                                          !
!                                                                      !
!    Stock et al, 2005: Evaluating hypotheses for the initiation and   !
!      development of alexandrium fundyense blooms in the western gulf !
!      of Maine using a coupled physical-biological model              !
!      Deep Sea Reserach II, 52,2715-2744.                             !
!                                                                      !
!                                                                      !
!***********************************************************************
!
      USE mod_param
      USE mod_grid
      USE mod_ocean
      USE mod_stepping
      USE mod_forces
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, tile
!
!  Local variable declarations.
!
#include "tile.h"
!
#ifdef PROFILE
      CALL wclock_on (ng, iNLM, 15)
#endif
      CALL biology_tile (ng, Istr, Iend, Jstr, Jend,                    &
     &                   LBi, UBi, LBj, UBj, N(ng), NT(ng),             &
     &                   nnew(ng),                                      &
#ifdef MASKING
     &                   GRID(ng) % rmask,                              &
#endif
     &                   GRID(ng) % Hz,                                 &
     &                   GRID(ng) % z_r,                                &
     &                   GRID(ng) % z_w,                                &
     &                   GRID(ng) % pm,                                 &
     &                   GRID(ng) % pn,                                 &
     &                   OCEAN(ng) % t,                                 & 
     &                   OCEAN(ng) % W,                                 &
     &                   OCEAN(ng) % WCELL,                             &
     &                   FORCES(ng) % Asrflx,                           & 
     &                   FORCES(ng) % DINOINI,                          &
     &                   FORCES(ng) % DINOBS )                           

#ifdef PROFILE
      CALL wclock_off (ng, iNLM, 15)
#endif
      RETURN
      END SUBROUTINE biology
!
!-----------------------------------------------------------------------
      SUBROUTINE biology_tile (ng, Istr, Iend, Jstr, Jend,              &
     &                         LBi, UBi, LBj, UBj, UBk, UBt,            &
     &                         nnew,                                    &
#ifdef MASKING
     &                         rmask,                                   &
#endif
     &                         Hz,                                      &
     &                         z_r,                                     &
     &                         z_w,                                     &
     &                         pm,                                      &
     &                         pn,                                      &
     &                         t,                                       &
     &                         W,                                       &
     &                         WCELL,                                   &
     &                         DASWR,                                   &
     &                         CYSTINIT,                                &
     &                         DIN_OBS )                              
!-----------------------------------------------------------------------
!
      USE mod_param
      USE mod_biology
      USE mod_ncparam
      USE mod_scalars
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, Iend, Istr, Jend, Jstr
      integer, intent(in) :: LBi, UBi, LBj, UBj, UBk, UBt
      integer, intent(in) :: nnew

#ifdef ASSUMED_SHAPE
# ifdef MASKING
      real(r8), intent(in) :: rmask(LBi:,LBj:)
# endif
      real(r8), intent(in) :: Hz(LBi:,LBj:,:)
      real(r8), intent(in) :: z_r(LBi:,LBj:,:)
      real(r8), intent(in) :: z_w(LBi:,LBj:,0:)
      real(r8), intent(in) :: pm(LBi:,LBj:)
      real(r8), intent(in) :: pn(LBi:,LBj:)
      real(r8), intent(inout) :: t(LBi:,LBj:,:,:,:)
      real(r8), intent(inout) :: W(LBi:,LBj:,0:)
      real(r8), intent(inout) :: WCELL(LBi:,LBj:,0:)
      real(r8), intent(in) :: DASWR(LBi:,LBj:)
      real(r8), intent(in) :: CYSTINIT(LBi:,LBj:)
      real(r8), intent(in) :: DIN_OBS(LBi:,LBj:,:)
#else
# ifdef MASKING
      real(r8), intent(in) :: rmask(LBi:UBi,LBj:UBj)
# endif
      real(r8), intent(in) :: Hz(LBi:UBi,LBj:UBj,UBk)
      real(r8), intent(in) :: z_r(LBi:UBi,LBj:UBj,UBk)
      real(r8), intent(in) :: z_w(LBi:UBi,LBj:UBj,0:UBk)
      real(r8), intent(in) :: pm(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: pn(LBi:UBi,LBj:UBj)
      real(r8), intent(inout) :: t(LBi:UBi,LBj:UBj,UBk,3,UBt)
      real(r8), intent(inout) :: W(LBi:UBi,LBj:UBj,0:UBk)
      real(r8), intent(inout) :: WCELL(LBi:UBi,LBj:UBj,0:UBk)
      real(r8), intent(in) :: DASWR(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: CYSTINIT(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: DIN_OBS(LBi:UBi,LBj:UBj,UBk)
#endif
!
!  Local variable declarations.
!
      integer :: IstrR, IendR, JstrR, JendR, IstrU, JstrV
      integer :: Iter, i, j, k, itrc, ibio

      real(r8), parameter :: eps = 1.0e-16_r8

      real(r8) :: cff, cff1, cff2, cff3, dtdays
      real(r8) :: cffL, cffR, cu, dltL, dltR

      real(r8), dimension(PRIVATE_1D_SCRATCH_ARRAY,N(ng),NT(ng)) :: Bio

      real(r8), dimension(LBi:UBi,LBj:UBj,N(ng)) :: Hz_inv
      real(r8), dimension(LBi:UBi,LBj:UBj,N(ng)) :: Hz_inv2
      real(r8), dimension(LBi:UBi,LBj:UBj,N(ng)) :: Hz_inv3

!==== Redtide Local Arrays

      real(r8), dimension(12) :: GP    ! Monthly median Germination Potentials
      real(r8), dimension(12) :: DAYMM ! Mid-point yeardays of each month 
      real(r8), dimension(12) :: GPN   ! Normalized Germination Potentials
      real(r8), dimension(11) :: BIOMOD! Parameters
      real(r8), dimension(LBi:UBi,LBj:UBj,N(ng)) :: CW
      real(r8), dimension(LBi:UBi,LBj:UBj,N(ng)) :: GROW
      real(r8), dimension(LBi:UBi,LBj:UBj,10) :: EFLUX
      real(r8), dimension(LBi:UBi,LBj:UBj) :: CELLFLX

      real(r8) :: EC, FAC, ATTENK, ATTENS, EFLUXL, EFLUXD, DGERM, EN

      real(r8) :: GERML(LBi:UBi,LBj:UBj), GERMD(LBi:UBi,LBj:UBj)
      real(r8) :: GERM(LBi:UBi,LBj:UBj), FFLUX(LBi:UBi,LBj:UBj)

      real(r8) :: C_DEPTH,MORT,AKSN
      real(r8) :: DASWR_AVG,RATE,GROWTHFAC
      real(r8) :: TTT,SSS,T_FAC,S_FAC,G_FAC,WMIG, WMIGA
      real(r8) :: RAD,DIN,G_LIGHT,G_DIN, TDAY, ENDOSCALE, THOUR
      integer :: year, month, iday
      real(r8) :: yday, hour, ITDAY, YEARDAY

#include "set_bounds.h"

      dtdays=dt(ng)*sec2day/REAL(BioIter(ng),r8)

!
!  Compute inverse thickness to avoid repeated divisions.
!
       DO j=Jstr,Jend
        DO k=1,N(ng)
          DO i=Istr,Iend
            Hz_inv(i,j,k)=1.0_r8/Hz(i,j,k)
          END DO
        END DO
        DO k=1,N(ng)-1
          DO i=Istr,Iend
            Hz_inv2(i,j,k)=1.0_r8/(Hz(i,j,k)+Hz(i,j,k+1))
          END DO
        END DO
        DO k=2,N(ng)-1
          DO i=Istr,Iend
            Hz_inv3(i,j,k)=1.0_r8/(Hz(i,j,k-1)+Hz(i,j,k)+Hz(i,j,k+1))
          END DO
        END DO
      END DO

!  Extract biological variables from tracer arrays, place them into
!  scratch arrays CW

        DO itrc=1,NBT
          ibio=idbio(itrc)
          DO k=1,N(ng)
           DO j=Jstr,Jend
            DO i=Istr,Iend
              CW(i,j,k)=t(i,j,k,nnew,ibio)*Hz_inv(i,j,k)
            END DO
           END DO
          END DO
        END DO
!
!----------------------------------------------------------------------
! Define input parameter data set
!----------------------------------------------------------------------
      BIOMOD(1) = Gmax(ng)   ! 1/day
      BIOMOD(2) = Dg(ng)     ! cm
      BIOMOD(3) = Kn(ng)     ! umol
      BIOMOD(4) = Mor(ng)    ! Mortality
      BIOMOD(5) = G_eff(ng)  ! m2/(watts*day)
      BIOMOD(6) = G_r(ng)    ! 1/day
      BIOMOD(7) = Kw(ng)     ! 1/m
      BIOMOD(8) = Ks(ng)     ! 1/mm
      BIOMOD(9) = Elgt(ng)   ! watt/m2
      BIOMOD(10)= Edrk(ng)   ! watt/m2
      BIOMOD(11)= Wa(ng)     ! m/day

! Monthly Median Germination Potential Values
      GP(1)  = 21.90_r8
      GP(2)  = 11.25_r8
      GP(3)  = 78.0_r8
      GP(4)  = 85.0_r8
      GP(5)  = 96.8_r8
      GP(6)  = 93.0_r8
      GP(7)  = 60.0_r8
      GP(8)  = 50.0_r8
      GP(9)  = 10.0_r8
      GP(10) = 11.5_r8
      GP(11) = 17.0_r8
      GP(12) = 34.5_r8

! Mid-point yeardays of each month (non leap-year, 1/1 00:00 = yearday 0)
      DAYMM(1)  = 15.5_r8
      DAYMM(2)  = 45.0_r8
      DAYMM(3)  = 74.5_r8
      DAYMM(4)  = 105.0_r8
      DAYMM(5)  = 135.5_r8
      DAYMM(6)  = 166.0_r8
      DAYMM(7)  = 196.5_r8
      DAYMM(8)  = 227.5_r8
      DAYMM(9)  = 258.0_r8
      DAYMM(10) = 288.5_r8
      DAYMM(11) = 319.0_r8
      DAYMM(12) = 349.5_r8

! Normalize to 1 (logic: germination rate experiments were done
! during peak germination potential, which is a bit under 100%.
! Those full rates should be realized).

      DO I = 1,12
        GPN(I) = GP(I)
        GPN(I) = GPN(I)/(MAX(GP(1),GP(2),GP(3),GP(4),GP(5),GP(6),       &
     &                   GP(7),GP(8),GP(9),GP(10),GP(11),GP(12)))
      END DO

! Define new vertcial velocity, which equals the sum of
! hydrodyanmic vertical vel. plus biological upward swimming vel.

      WMIG = BIOMOD(11)/(24.0*60.0*60.0)  ! m/s
      DO i=Istr,Iend
       DO j=Jstr,Jend
         WCELL(i,j,:)=0.0_r8
         DO k=0, N(ng)-1
           IF (-1*z_w(i,j,k).LT.21.0_r8) THEN
             WMIGA = WMIG*(1.0-TANH((21.0+z_w(i,j,k))/10.0)) ! m/s
           ELSE
             WMIGA = WMIG
           ENDIF
           WCELL(i,j,k) = W(I,J,K)+WMIGA/(pm(I,J)*pn(I,J)) ! vertical flux m3/s
         END DO
       END DO
      END DO
     
!      do k=0,N(ng)
!         print*,'WCELL= ',k,W(100,80,k), WCELL(100,80,k)
!      end do
!      stop

!------------------------------------------------------------------------
! Calculate Endogenous Factor
!------------------------------------------------------------------------
!
!
      CALL caldate (r_date, tdays(ng), year, yday, month, iday, hour)
      YEARDAY=yday
      THOUR  =hour

      IF (YEARDAY.LT.DAYMM(1)) THEN
        FAC=((365.0_r8-DAYMM(12))+YEARDAY)/                             &     
     &      ((365.0_r8-DAYMM(12))+DAYMM(1))
        EC = GPN(12) + FAC*(GPN(1) - GPN(12))
      ELSE IF (YEARDAY.GE.DAYMM(12)) THEN
        FAC = (YEARDAY - DAYMM(12))/((365.0_r8-DAYMM(12))+DAYMM(1))
        EC = GPN(12) + FAC*(GPN(1) - GPN(12))
      ELSE
        DO I = 1,11
          IF ((YEARDAY.GE.DAYMM(I)).AND.(YEARDAY.LT.DAYMM(I+1))) THEN
            FAC = (YEARDAY - DAYMM(I))/(DAYMM(I+1) - DAYMM(I))
            EC = GPN(I) + FAC*(GPN(I+1) - GPN(I))
          END IF
        END DO
      END IF

      ENDOSCALE = EC
!      print*,'EC=', yday,hour,EC
!-----------------------------------------------------------------------
! Calculate bottom vegatation flux from the top sediment layer: Fg
!-----------------------------------------------------------------------

      DO  I = Istr,Iend
        DO  J = Jstr,Jend
          GERM(I,J)  = 0.0_r8
          GERMD(I,J) = 0.0_r8
          GERML(I,J) = 0.0_r8
          DO  K = 1,10
            EFLUX(I,J,K)= 0.0_r8
          END DO
        END DO
      END DO

! setting light attenuation constant, the values that differentiate 
! the "light", "dark", and "transitional" conditions for germination,
! and the germination depth based on the input from the array BIOMOD

      ATTENK = BIOMOD(7)             !m-1
      ATTENS = BIOMOD(8)*10.0        !convert from mm-1 to cm-1
      EFLUXL = BIOMOD(9)             !watts/m2 
      EFLUXD = BIOMOD(10)            !watts/m2 
      DGERM  = BIOMOD(2)             !cm

! begin loop through all grid points to calculate the germination rate 
! and fluxes

      DO  I = Istr, Iend
        DO  J = Jstr, Jend
! calculate light/dark germination rate
          TTT=Min(t(i,j,1,nnew,itemp)*Hz_inv(i,j,1),36.0_r8)
          GERML(I,J) = 1.50_r8 + (8.72_r8 - 1.50_r8) * 0.5_r8 *         &
     &         (tanh(0.790_r8 * TTT - 6.27_r8) + 1.0_r8)
          GERMD(I,J) = 1.04_r8 + (4.26_r8 - 1.04_r8) * 0.5_r8 *         &
     &         (tanh(0.394_r8 * TTT - 3.33_r8) + 1.0_r8)

            DO K = 1,10
              EFLUX(I,J,K)=DASWR(i,j)*EXP(ATTENK*z_w(i,j,0))*           &
     &              EXP(-ATTENS*(DGERM/10.0_r8*K - DGERM/20.0_r8))

! checking to see which light regime applies: light, dark, or transitional

              IF (EFLUX(I,J,K).GT.EFLUXL) THEN
                GERM(I,J) = GERM(I,J)+GERML(I,J)*0.1_r8
              ELSE IF (EFLUX(I,J,K).LT.EFLUXD) THEN
                GERM(I,J) = GERM(I,J)+GERMD(I,J)*0.1_r8
              ELSE
                GERM(I,J)=GERM(I,J)+((GERML(I,J)-GERMD(I,J))*           &
     &          ((EFLUX(I,J,K)-EFLUXD)/(EFLUXL-EFLUXD))                 &
     &          + GERMD(I,J)) * 0.1_r8
              END IF
            END DO

! The Endogenous Clock: scale the germination rate by the factor 
! ENDOSCALE

            GERM(I,J) = GERM(I,J)*ENDOSCALE

! convert % cysts/day into decimal fraction of cysts

            GERM(I,J) = GERM(I,J)/100.0_r8

! calculate the flux of cells away from the bottom. It is referenced
! to the initial number of cysts to be consistent with laboratory 
! experiments. 
            CELLFLX(I,J) = CYSTINIT(I,J)*GERM(I,J)      ! cell/m2
            CELLFLX(I,J) = CELLFLX(I,J)*Hz_inv(I,J,1)   ! cell/m3      
        END DO
      END DO
!      print*,  GERM(100,80),CELLFLX(100,80), Hz_inv(100,80,1)
!-----------------------------------------------------------------------
! Now, Calculate the Growth and Mortality rates
!-----------------------------------------------------------------------

! set parameter values 
      MORT   = BIOMOD(4)
      AKSN   = BIOMOD(3)
      ATTENK = BIOMOD(7)

! assume that deep newly germinated cells can get up to the
! compensation depth without losses. tanh(x) --> x as x gets
! small - use mean day-averaged  irradiance (watts) to get
! the compensation depth (C_DEPTH)

      DASWR_AVG = 345.5_r8
      C_DEPTH = 1.0_r8/ATTENK*LOG(G_r(ng)/(G_eff(ng)*DASWR_AVG))
      

      Do  I = Istr,Iend
        Do  J = Jstr, Jend
          Do  K = 1,N(ng)
            TTT=Min(t(i,j,k,nnew,itemp)*Hz_inv(i,j,k),36.0_r8)
            SSS=MAX(t(i,j,k,nnew,isalt)*Hz_inv(i,j,k),0.0_r8)

! to compensate physical model bias, which came from HyCOM.
!           TTT=Min(t(i,j,k,nnew,itemp)*Hz_inv(i,j,k)+1,36.0_r8)
!           SSS=MAX(t(i,j,k,nnew,isalt)*Hz_inv(i,j,k)-5,0.0_r8)


! Temperature factor, polynomial for < 5, 1 term Taylor Expansion < 5
! 0.343 = first derivative at 5
!             IF (TTT.GE.5.0_r8) THEN
!               T_FAC=-0.000513_r8*TTT**3+0.0160_r8*TTT**2-              &
!     &               0.0867_r8*TTT+0.382_r8
!             ELSE
!               T_FAC=-0.000513_r8*5.0_r8**3+0.0160_r8*5.0_r8**2         &
!     &              -0.0867_r8*5.0+0.382_r8
!               T_FAC=T_FAC - 0.0343_r8*(5.0_r8-TTT)
!             END IF
! Below is the temperature correction made by Stock (8/15/2006)
             IF (TTT.GE.5.0_r8) THEN
               T_FAC=-0.000536_r8*TTT**3+0.0169_r8*TTT**2-              &
     &               0.0961_r8*TTT+0.379_r8
             ELSE
               T_FAC=0.254_r8 - 0.0327_r8*(5.0_r8-TTT)
             END IF

! Salinity adjustment factor
!             S_FAC=0.0000882_r8*SSS**3-0.00808_r8*SSS**2+0.220_r8*SSS   &
!     &             -0.872_r8
! Below is the salinity correction made by Stock (8/15/2006)
             S_FAC=0.0000557_r8*SSS**3-0.00622_r8*SSS**2+0.186_r8*SSS   &
     &             -0.693_r8

             G_FAC = S_FAC*T_FAC
             RAD = DASWR(i,j)*exp(ATTENK*z_r(i,j,k))

             IF (z_r(i,j,k).GT.C_DEPTH) THEN
               G_LIGHT = (GMAX(ng)*G_FAC+G_r(ng))*                      &
     &        TANH(G_eff(ng)*RAD/(GMAX(ng)*G_FAC+G_r(ng))) - G_r(ng)
             ELSE
               G_LIGHT = 0.0_r8
             END IF

! setting DIN from observations 

             IF (z_r(i,j,k).GT.C_DEPTH) THEN
               DIN = DIN_OBS(I,J,K)
             ELSE
               DIN = 15.0_r8
             END IF

             IF (AKSN.GT.0.0_r8) THEN
               G_DIN = GMAX(ng)*G_FAC*DIN/(AKSN+DIN)
             ELSE
               G_DIN = GMAX(ng)*G_FAC
             ENDIF

! growth is either DIN or light limited

             RATE = MIN(G_LIGHT,G_DIN)

! apply loss terms if above mean critical depth (-20m)
            IF (z_r(i,j,k).GT.C_DEPTH) THEN
              RATE = RATE -                                             &
! test 1, MORT=0.075
!     &          MORT*2.3_r8**((t(i,j,k,nnew,itemp)*Hz_inv(i,j,k)        &
!     &          -4.5_r8)/10.0_r8)
! test 2, MORT=0.050
!     &          MORT*6.0_r8**((t(i,j,k,nnew,itemp)*Hz_inv(i,j,k)        &
!     &          -6.0_r8)/10.0_r8)
! test 3, MORT=0.050
!     &          MORT*6.0_r8**((t(i,j,k,nnew,itemp)*Hz_inv(i,j,k)        &
!     &          -4.5_r8)/10.0_r8)
! test 4, MORT=0.025
!     &          MORT*60.0_r8**((t(i,j,k,nnew,itemp)*Hz_inv(i,j,k)       &
!     &          -10.0_r8)/10.0_r8)
! test 5, /home/ruoying/Redtides/Cell/cell_data/funct2.m
!     &          0.047_r8*31.27_r8**((t(i,j,k,nnew,itemp)*Hz_inv(i,j,k)  &
!     &          -10.20_r8)/10.0_r8)-0.018
! test 6, /home/ruoying/Redtides/Cell/cell_data/funct3.m
     &          0.066_r8*21.76_r8**((t(i,j,k,nnew,itemp)*Hz_inv(i,j,k)  &
     &          -10.35_r8)/10.0_r8)-0.019


            END IF

! apply encystment term according to McGillicuddy et al. DSRII, 2005, 2698-2714
!            IF (G_DIN.LT.G_LIGHT) THEN
!              EN = 0.5*(G_LIGHT-G_DIN)
!            ELSE
!              EN = 0.0_r8
!            END IF
!            RATE = RATE -EN

! update concentration after growth, mortality and germination processes.
            GROWTHFAC = 1.0_r8 + dtdays * RATE 
            IF (K.gt.1) then
             CW(I,J,K)= CW(I,J,K)*GROWTHFAC                     !cell/m3
            ELSE              ! k=1, adding cellflux to the bottom layer
             CW(I,J,K)= CW(I,J,K)*GROWTHFAC+CELLFLX(I,J)*dtdays !cell/m3
            END IF
          END DO
        END DO
       END DO
!	print*,'MORT=',MORT
!
!-----------------------------------------------------------------------
!  Update global tracer variables (m Tunits).
!-----------------------------------------------------------------------

        DO itrc=1,NBT
          ibio=idbio(itrc)
          DO k=1,N(ng)
           DO j=Jstr,Jend
            DO i=Istr,Iend
              t(i,j,k,nnew,ibio)=MAX(CW(i,j,k)*Hz(i,j,k),0.0_r8) 
#ifdef TS_MPDATA
              t(i,j,k,3,ibio)=t(i,j,k,nnew,ibio)
#endif
            END DO
           END DO
          END DO
        END DO 

      RETURN
      END SUBROUTINE biology_tile
