#include "cppdefs.h"
      MODULE tl_obc_volcons_mod
#if defined TANGENT && defined OBC_VOLCONS
!
!=======================================================================
!  Copyright (c) 2005 ROMS/TOMS Adjoint Group                          !
!================================================== Hernan G. Arango ===
!                                                                      !
!  This routine computes tangent linear integral mass flux "obc_flux"  !
!  across the open boundaries, which is needed to enforce global mass  !
!  conservation constraint.                                            !
!                                                                      !
!=======================================================================
!
      implicit none

      PRIVATE
      PUBLIC  :: tl_obc_flux_tile, tl_set_DUV_bc_tile

      CONTAINS
!
!***********************************************************************
      SUBROUTINE tl_obc_flux (ng, tile)
!***********************************************************************
!
      USE mod_param
      USE mod_grid
      USE mod_ocean
      USE mod_stepping
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, tile
!
!  Local variable declarations.
!
# include "tile.h"
!
      CALL tl_obc_flux_tile (ng, Istr, Iend, Jstr, Jend,                &
     &                       LBi, UBi, LBj, UBj,                        &
     &                       kinp(ng),                                  &
# ifdef MASKING
     &                       GRID(ng) % umask,                          &
     &                       GRID(ng) % vmask,                          &
# endif
     &                       GRID(ng) % h,                              &
     &                       GRID(ng) % tl_h,                           &
     &                       GRID(ng) % om_v,                           &
     &                       GRID(ng) % on_u,                           &
     &                       OCEAN(ng) % ubar,                          &
     &                       OCEAN(ng) % vbar,                          &
     &                       OCEAN(ng) % zeta,                          &
     &                       OCEAN(ng) % tl_ubar,                       &
     &                       OCEAN(ng) % tl_vbar,                       &
     &                       OCEAN(ng) % tl_zeta)
      RETURN
      END SUBROUTINE tl_obc_flux
!
!***********************************************************************
      SUBROUTINE tl_obc_flux_tile (ng, Istr, Iend, Jstr, Jend,          &
     &                             LBi, UBi, LBj, UBj,                  &
     &                             kinp,                                &
# ifdef MASKING
     &                             umask, vmask,                        &
# endif
     &                             h, tl_h,                             &
     &                             om_v, on_u,                          &
     &                             ubar, vbar, zeta,                    &
     &                             tl_ubar, tl_vbar, tl_zeta)
!***********************************************************************
!
      USE mod_param
      USE mod_parallel
      USE mod_scalars
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, Iend, Istr, Jend, Jstr
      integer, intent(in) :: LBi, UBi, LBj, UBj
      integer, intent(in) :: kinp
!
# ifdef ASSUMED_SHAPE
#  ifdef MASKING
      real(r8), intent(in) :: umask(LBi:,LBj:)
      real(r8), intent(in) :: vmask(LBi:,LBj:)
#  endif
      real(r8), intent(in) :: h(LBi:,LBj:)
      real(r8), intent(in) :: tl_h(LBi:,LBj:)
      real(r8), intent(in) :: om_v(LBi:,LBj:)
      real(r8), intent(in) :: on_u(LBi:,LBj:)

      real(r8), intent(in) :: ubar(LBi:,LBj:,:)
      real(r8), intent(in) :: vbar(LBi:,LBj:,:)
      real(r8), intent(in) :: zeta(LBi:,LBj:,:)
      real(r8), intent(in) :: tl_ubar(LBi:,LBj:,:)
      real(r8), intent(in) :: tl_vbar(LBi:,LBj:,:)
      real(r8), intent(in) :: tl_zeta(LBi:,LBj:,:)
# else
#  ifdef MASKING
      real(r8), intent(in) :: umask(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: vmask(LBi:UBi,LBj:UBj)
#  endif
      real(r8), intent(in) :: h(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: tl_h(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: om_v(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: on_u(LBi:UBi,LBj:UBj)

      real(r8), intent(in) :: ubar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: vbar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: zeta(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: tl_ubar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: tl_vbar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: tl_zeta(LBi:UBi,LBj:UBj,3)
# endif
!
!  Local variable declarations.
!
      integer :: IstrR, IendR, JstrR, JendR, IstrU, JstrV
      integer :: NSUB, i, j

      real(r8) :: cff, my_area, my_flux
      real(r8) :: tl_cff, tl_my_area, tl_my_flux

# ifdef DISTRIBUTE
      real(r8), dimension(4) :: buffer
      character (len=3), dimension(4) :: op_handle
# endif

# include "set_bounds.h"
!
!-----------------------------------------------------------------------
!  Compute open segments cross-section area and mass flux.
!-----------------------------------------------------------------------
!
      my_area=0.0_r8
      my_flux=0.0_r8

      tl_my_area=0.0_r8
      tl_my_flux=0.0_r8
!
# ifdef WEST_VOLCONS
      IF (WESTERN_EDGE) THEN
        DO j=Jstr,Jend
          cff=0.5_r8*on_u(Istr,j)*                                      &
     &        (zeta(Istr-1,j,kinp)+h(Istr-1,j)+                         &
     &         zeta(Istr  ,j,kinp)+h(Istr  ,j))
          tl_cff=0.5_r8*on_u(Istr,j)*                                   &
     &           (tl_zeta(Istr-1,j,kinp)+tl_h(Istr-1,j)+                &
     &            tl_zeta(Istr  ,j,kinp)+tl_h(Istr-1,j))
#  ifdef MASKING
          cff=cff*umask(Istr,j)
          tl_cff=tl_cff*umask(Istr,j)
#  endif
          my_area=my_area+cff
          tl_my_area=tl_my_area+tl_cff
          my_flux=my_flux+cff*ubar(Istr,j,kinp)
          tl_my_flux=tl_my_flux+                                        &
     &               tl_cff*ubar(Istr,j,kinp)+                          &
     &               cff*tl_ubar(Istr,j,kinp)
        END DO
      END IF
# endif
# ifdef EAST_VOLCONS
      IF (EASTERN_EDGE) THEN
        DO j=Jstr,Jend
          cff=0.5_r8*on_u(Iend+1,j)*                                    &
     &        (zeta(Iend  ,j,kinp)+h(Iend  ,j)+                         &
     &         zeta(Iend+1,j,kinp)+h(Iend+1,j))
          tl_cff=0.5_r8*on_u(Iend+1,j)*                                 &
     &           (tl_zeta(Iend  ,j,kinp)+tl_h(Iend  ,j)+                &
     &            tl_zeta(Iend+1,j,kinp)+tl_h(Iend+1,j))
#  ifdef MASKING
          cff=cff*umask(Iend+1,j)
          tl_cff=tl_cff*umask(Iend+1,j)
#  endif
          my_area=my_area+cff
          tl_my_area=tl_my_area+tl_cff
          my_flux=my_flux-cff*ubar(Iend+1,j,kinp)
          tl_my_flux=tl_my_flux-                                        &
     &               tl_cff*ubar(Iend+1,j,kinp)-                        &
     &               cff*tl_ubar(Iend+1,j,kinp)
        END DO
      END IF
# endif
# ifdef SOUTH_VOLCONS
      IF (SOUTHERN_EDGE) THEN
        DO i=Istr,Iend
          cff=0.5_r8*om_v(i,Jstr)*                                      &
     &        (zeta(i,Jstr-1,kinp)+h(i,Jstr-1)+                         &
     &         zeta(i,Jstr  ,kinp)+h(i,Jstr  ))
          tl_cff=0.5_r8*om_v(i,Jstr)*                                   &
     &           (tl_zeta(i,Jstr-1,kinp)+tl_h(i,Jstr-1)+                &
     &            tl_zeta(i,Jstr  ,kinp)+tl_h(i,Jstr  ))
#  ifdef MASKING
          cff=cff*vmask(i,Jstr)
          tl_cff=tl_cff*vmask(i,Jstr)
#  endif
          my_area=my_area+cff
          tl_my_area=tl_my_area+tl_cff
          my_flux=my_flux+cff*vbar(i,JstrV-1,kinp)
          tl_my_flux=tl_my_flux+                                        &
     &               tl_cff*vbar(i,JstrV-1,kinp)+                       &
     &               cff*tl_vbar(i,JstrV-1,kinp)
        END DO
      END IF
# endif
# ifdef NORTH_VOLCONS
      IF (NORTHERN_EDGE) THEN
        DO i=Istr,Iend
          cff=0.5_r8*om_v(i,Jend+1)*                                    &
     &        (zeta(i,Jend  ,kinp)+h(i,Jend  )+                         &
     &         zeta(i,Jend+1,kinp)+h(i,Jend+1))
          tl_cff=0.5_r8*om_v(i,Jend+1)*                                 &
     &           (tl_zeta(i,Jend  ,kinp)+tl_h(i,Jend  )+                &
     &            tl_zeta(i,Jend+1,kinp)+tl_h(i,Jend+1))
#  ifdef MASKING
          cff=cff*vmask(i,Jend+1)
          tl_cff=tl_cff*vmask(i,Jend+1)
#  endif
          my_area=my_area+cff
          tl_my_area=tl_my_area+tl_cff
          my_flux=my_flux-cff*vbar(i,Jend+1,kinp)
          tl_my_flux=tl_my_flux-                                        &
     &               tl_cff*vbar(i,Jend+1,kinp)-                        &
     &               cff*tl_vbar(i,Jend+1,kinp)
        END DO
      END IF
# endif
# if defined WEST_VOLCONS  || defined EAST_VOLCONS  || \
     defined SOUTH_VOLCONS || defined NORTH_VOLCONS
!
!-----------------------------------------------------------------------
!  Perform global summation and compute correction velocity.
!-----------------------------------------------------------------------
!
      IF (SOUTH_WEST_CORNER.and.                                        &
     &    NORTH_EAST_CORNER) THEN
        NSUB=1                           ! non-tiled application
      ELSE
        NSUB=NtileX(ng)*NtileE(ng)       ! tiled application
      END IF
!$OMP CRITICAL (TL_OBC_VOLUME)
      IF (tile_count.eq.0) THEN
        bc_flux=0.0_r8
        bc_area=0.0_r8
        tl_bc_flux=0.0_r8
        tl_bc_area=0.0_r8
      END IF
      bc_area=bc_area+my_area
      bc_flux=bc_flux+my_flux
      tl_bc_area=tl_bc_area+tl_my_area
      tl_bc_flux=tl_bc_flux+tl_my_flux
      tile_count=tile_count+1
      IF (tile_count.eq.NSUB) THEN
        tile_count=0
#  ifdef DISTRIBUTE
        buffer(1)=bc_area
        buffer(2)=bc_flux
        buffer(3)=tl_bc_area
        buffer(4)=tl_bc_flux
        op_handle(1)='SUM'
        op_handle(2)='SUM'
        op_handle(3)='SUM'
        op_handle(4)='SUM'
        CALL mp_reduce (ng, iTLM, 4, buffer, op_handle)
        bc_area=buffer(1)
        bc_flux=buffer(2)
        tl_bc_area=buffer(3)
        tl_bc_flux=buffer(4)
#  endif
        ubar_xs=bc_flux/bc_area
        tl_ubar_xs=tl_bc_flux/bc_area-                                  &
     &             tl_bc_area*ubar_xs/bc_area
      END IF
!$OMP END CRITICAL (TL_OBC_VOLUME)
# endif
      RETURN
      END SUBROUTINE tl_obc_flux_tile
!
!***********************************************************************
      SUBROUTINE tl_set_DUV_bc_tile (ng, Istr, Iend, Jstr, Jend,        &
     &                               LBi, UBi, LBj, UBj,                &
     &                               ILB, IUB, JLB, JUB,                &
     &                               kinp,                              &
# ifdef MASKING
     &                               umask, vmask,                      &
# endif
     &                               om_v, on_u,                        &
     &                               ubar, vbar,                        &
     &                               tl_ubar, tl_vbar,                  &
     &                               Drhs, Duon, Dvom,                  &
     &                               tl_Drhs, tl_Duon, tl_Dvom)
!***********************************************************************
!
      USE mod_param
      USE mod_scalars
# ifdef DISTRIBUTE
!
      USE exchange_2d_mod
# endif
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, Iend, Istr, Jend, Jstr
      integer, intent(in) :: LBi, UBi, LBj, UBj
      integer, intent(in) :: kinp
!
# ifdef ASSUMED_SHAPE
#  ifdef MASKING
      real(r8), intent(in) :: umask(LBi:,LBj:)
      real(r8), intent(in) :: vmask(LBi:,LBj:)
#  endif
      real(r8), intent(in) :: om_v(LBi:,LBj:)
      real(r8), intent(in) :: on_u(LBi:,LBj:)

      real(r8), intent(in) :: ubar(LBi:,LBj:,:)
      real(r8), intent(in) :: vbar(LBi:,LBj:,:)
      real(r8), intent(in) :: Drhs(ILB:,JLB:)
      real(r8), intent(in) :: Duon(ILB:,JLB:)
      real(r8), intent(in) :: Dvom(ILB:,JLB:)
      real(r8), intent(in) :: tl_ubar(LBi:,LBj:,:)
      real(r8), intent(in) :: tl_vbar(LBi:,LBj:,:)
      real(r8), intent(in) :: tl_Drhs(ILB:,JLB:)

      real(r8), intent(inout) :: tl_Duon(ILB:,JLB:)
      real(r8), intent(inout) :: tl_Dvom(ILB:,JLB:)
# else
#  ifdef MASKING
      real(r8), intent(in) :: umask(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: vmask(LBi:UBi,LBj:UBj)
#  endif
      real(r8), intent(in) :: om_v(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: on_u(LBi:UBi,LBj:UBj)

      real(r8), intent(in) :: ubar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: vbar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: Drhs(ILB:IUB,JLB:JUB)
      real(r8), intent(in) :: Duon(ILB:IUB,JLB:JUB)
      real(r8), intent(in) :: Dvom(ILB:IUB,JLB:JUB)
      real(r8), intent(in) :: tl_ubar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: tl_vbar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(in) :: tl_Drhs(ILB:,JLB:)

      real(r8), intent(inout) :: tl_Duon(ILB:IUB,JLB:JUB)
      real(r8), intent(inout) :: tl_Dvom(ILB:IUB,JLB:JUB)
# endif
!
!  Local variable declarations.
!
      integer :: IstrR, IendR, JstrR, JendR, IstrU, JstrV
      integer :: i, j

# include "set_bounds.h"
!
!-----------------------------------------------------------------------
!  Set vertically integrated mass fluxes "DUon" and "DVom" along
!  the open boundaries in such a way that the integral volume is
!  conserved.  This is done by applying "ubar_xs" correction to
!  the velocities.
!-----------------------------------------------------------------------
!
# ifdef DISTRIBUTE
#  define I_RANGE IstrU,MIN(Iend+1,Lm(ng))
#  define J_RANGE JstrV,MIN(Jend+1,Mm(ng))
# else
#  define I_RANGE MAX(2,IstrU-1),MIN(Iend+1,Lm(ng))
#  define J_RANGE MAX(2,JstrV-1),MIN(Jend+1,Mm(ng))
# endif

# ifdef WEST_VOLCONS
      IF (WESTERN_EDGE) THEN
        DO j=-2+J_RANGE+1
!>        Duon(Istr,j)=0.5_r8*(Drhs(Istr,j)+Drhs(Istr-1,j))*            &
!>   &                 (ubar(Istr,j,kinp)-ubar_xs)*                     &
!>   &                 on_u(Istr,j)
!>
          tl_DUon(Istr,j)=0.5_r8*                                       &
     &                    ((tl_Drhs(Istr,j)+tl_Drhs(Istr-1,j))*         &
     &                     (ubar(Istr,j,kinp)-ubar_xs)+                 &
     &                     (Drhs(Istr,j)+Drhs(Istr-1,j))*               &
     &                     (tl_ubar(Istr,j,kinp)-tl_ubar_xs))*          &
     &                    on_u(Istr,j)
#  ifdef MASKING
!>        Duon(Istr,j)=Duon(Istr,j)*umask(Istr,j)
!>
          tl_Duon(Istr,j)=tl_Duon(Istr,j)*umask(Istr,j)
#  endif
        END DO
      END IF
# endif
# ifdef EAST_VOLCONS
      IF (EASTERN_EDGE) THEN
        DO j=-2+J_RANGE+1
!>        Duon(Iend+1,j)=0.5_r8*(Drhs(Iend+1,j)+Drhs(Iend,j))*          &
!>   &                   (ubar(Iend+1,j,kinp)+ubar_xs)*                 &
!>   &                   on_u(Iend+1,j)
!>
          tl_DUon(Iend+1,j)=0.5_r8*                                     &
     &                      ((tl_Drhs(Iend+1,j)+tl_Drhs(Iend,j))*       &
     &                       (ubar(Iend+1,j,kinp)+ubar_xs)+             &
     &                       (Drhs(Iend+1,j)+Drhs(Iend,j))*             &
     &                       (tl_ubar(Iend+1,j,kinp)+tl_ubar_xs))*      &
     &                      on_u(Iend+1,j)
#  ifdef MASKING
!>        Duon(Iend+1,j)=Duon(Iend+1,j)*umask(Iend+1,j)
!>
          tl_Duon(Iend+1,j)=tl_Duon(Iend+1,j)*umask(Iend+1,j)
#  endif
        END DO
      END IF
# endif
# ifdef SOUTH_VOLCONS
      IF (SOUTHERN_EDGE) THEN
        DO i=-2+I_RANGE+1
!>        Dvom(i,Jstr)=0.5_r8*(Drhs(i,Jstr)+Drhs(i,Jstr-1))*            &
!>   &                 (vbar(i,Jstr,kinp)-ubar_xs)*                     &
!>   &                 om_v(i,Jstr)
!>
          tl_DVom(i,Jstr)=0.5_r8*                                       &
     &                    ((tl_Drhs(i,Jstr)+tl_Drhs(i,Jstr-1))*         &
     &                     (vbar(i,Jstr,kinp)-ubar_xs)+                 &
     &                     (Drhs(i,Jstr)+Drhs(i,Jstr-1))*               &
     &                     (tl_vbar(i,Jstr,kinp)-tl_ubar_xs))*          &
     &                    om_v(i,Jstr)
#  ifdef MASKING
!>        Dvom(i,Jstr)=Dvom(i,Jstr)*vmask(i,Jstr)
!>
          tl_Dvom(i,Jstr)=tl_Dvom(i,Jstr)*vmask(i,Jstr)
#  endif
        END DO
      END IF
# endif
# ifdef NORTH_VOLCONS
      IF (NORTHERN_EDGE) THEN
        DO i=-2+I_RANGE+1
!>        Dvom(i,Jend+1)=0.5_r8*(Drhs(i,Jend+1)+Drhs(i,Jend))*          &
!>   &                   (vbar(i,Jend+1,kinp)+ubar_xs)*                 &
!>   &                   om_v(i,Jend+1)
!>
          tl_DVom(i,Jend+1)=0.5_r8*                                     &
     &                      ((tl_Drhs(i,Jend+1)+tl_Drhs(i,Jend))*       &
     &                       (vbar(i,Jend+1,kinp)+ubar_xs)+             &
     &                       (Drhs(i,Jend+1)+Drhs(i,Jend))*             &
     &                       (tl_vbar(i,Jend+1,kinp)+tl_ubar_xs))*      &
     &                      om_v(i,Jend+1)
#  ifdef MASKING
!>        Dvom(i,Jend+1)=Dvom(i,Jend+1)*vmask(i,Jend+1)
!>
          tl_Dvom(i,Jend+1)=tl_Dvom(i,Jend+1)*vmask(i,Jend+1)
#  endif
        END DO
      END IF
# endif
# ifdef DISTRIBUTE
!
! Do a special exchange to avoid having three ghost points for high
! order numerical stencil.
!
#  if defined WEST_VOLCONS || defined EAST_VOLCONS
!>    CALL exchange_u2d_tile (ng, iNLM, Istr, Iend, Jstr, Jend,         &
!>   &                        ILB, IUB, JLB, JUB,                       &
!>   &                        NghostPoints,                             &
!>   &                        Duon)
!>
      CALL exchange_u2d_tile (ng, iTLM, Istr, Iend, Jstr, Jend,         &
     &                        ILB, IUB, JLB, JUB,                       &
     &                        NghostPoints,                             &
     &                        tl_Duon)
#  endif
#  if defined SOUTH_VOLCONS || defined NORTH_VOLCONS
!>    CALL exchange_v2d_tile (ng, iNLM, Istr, Iend, Jstr, Jend,         &
!>   &                        ILB, IUB, JLB, JUB,                       &
!>   &                        NghostPoints,                             &
!>   &                        Dvom)
!>
      CALL exchange_v2d_tile (ng, iTLM, Istr, Iend, Jstr, Jend,         &
     &                        ILB, IUB, JLB, JUB,                       &
     &                        NghostPoints,                             &
     &                        tl_Dvom)
#  endif
# endif
# undef I_RANGE
# undef J_RANGE

      RETURN
      END SUBROUTINE tl_set_DUV_bc_tile
!
!***********************************************************************
      SUBROUTINE tl_conserve_mass_tile (ng, Istr, Iend, Jstr, Jend,     &
     &                                  LBi, UBi, LBj, UBj,             &
     &                                  kinp,                           &
# ifdef MASKING
     &                                  umask, vmask,                   &
# endif
     &                                  tl_ubar, tl_vbar)
!***********************************************************************
!
      USE mod_param
      USE mod_scalars
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, Iend, Istr, Jend, Jstr
      integer, intent(in) :: LBi, UBi, LBj, UBj
      integer, intent(in) :: kinp
!
# ifdef ASSUMED_SHAPE
#  ifdef MASKING
      real(r8), intent(in) :: umask(LBi:,LBj:)
      real(r8), intent(in) :: vmask(LBi:,LBj:)
#  endif
      real(r8), intent(inout) :: tl_ubar(LBi:,LBj:,:)
      real(r8), intent(inout) :: tl_vbar(LBi:,LBj:,:)
# else
#  ifdef MASKING
      real(r8), intent(in) :: umask(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: vmask(LBi:UBi,LBj:UBj)
#  endif
      real(r8), intent(inout) :: tl_ubar(LBi:UBi,LBj:UBj,3)
      real(r8), intent(inout) :: tl_vbar(LBi:UBi,LBj:UBj,3)
# endif
!
!  Local variable declarations.
!
      integer :: i, j
!
!-----------------------------------------------------------------------
!  Corrects velocities across the open boundaries to enforce global
!  mass conservation constraint.
!-----------------------------------------------------------------------
!
# ifdef WEST_VOLCONS
      IF (WESTERN_EDGE) THEN
        DO j=Jstr,Jend
!>        ubar(Istr,j,kinp)=ubar(Istr,j,kinp)-ubar_xs
!>
          tl_ubar(Istr,j,kinp)=tl_ubar(Istr,j,kinp)-tl_ubar_xs
#  ifdef MASKING
!>        ubar(Istr,j,kinp)=ubar(Istr,j,kinp)*umask(Istr,j)
!>
          tl_ubar(Istr,j,kinp)=tl_ubar(Istr,j,kinp)*umask(Istr,j)
#  endif
        END DO
      END IF
# endif
# ifdef EAST_VOLCONS
      IF (EASTERN_EDGE) THEN
        DO j=Jstr,Jend
!>        ubar(Iend+1,j,kinp)=ubar(Iend+1,j,kinp)+ubar_xs
!>
          tl_ubar(Iend+1,j,kinp)=tl_ubar(Iend+1,j,kinp)+tl_ubar_xs
#  ifdef MASKING
!>        ubar(Iend+1,j,kinp)=ubar(Iend+1,j,kinp)*umask(Iend+1,j)
!>
          tl_ubar(Iend+1,j,kinp)=tl_ubar(Iend+1,j,kinp)*umask(Iend+1,j)
#  endif
        END DO
      END IF
# endif
# ifdef SOUTH_VOLCONS
      IF (SOUTHERN_EDGE) THEN
        DO i=Istr,Iend
!>        vbar(i,Jstr,kinp)=(vbar(i,Jstr,kinp)-ubar_xs)
!>
          tl_vbar(i,Jstr,kinp)=(tl_vbar(i,Jstr,kinp)-tl_ubar_xs)
#  ifdef MASKING
!>        vbar(i,Jstr,kinp)=vbar(i,Jstr,kinp)*vmask(i,Jstr)
!>
          tl_vbar(i,Jstr,kinp)=tl_vbar(i,Jstr,kinp)*vmask(i,Jstr)
#  endif
        END DO
      END IF
# endif
# ifdef NORTH_VOLCONS
      IF (NORTHERN_EDGE) THEN
        DO i=Istr,Iend
!>        vbar(i,Jend+1,kinp)=(vbar(i,Jend+1,kinp)+ubar_xs)
!>
          tl_vbar(i,Jend+1,kinp)=(tl_vbar(i,Jend+1,kinp)+tl_ubar_xs)
#  ifdef MASKING
!>        vbar(i,Jend+1,kinp)=vbar(i,Jend+1,kinp)*vmask(i,Jend+1)
!>
          tl_vbar(i,Jend+1,kinp)=tl_vbar(i,Jend+1,kinp)*vmask(i,Jend+1)
#  endif
        END DO
      END IF
# endif
      RETURN
      END SUBROUTINE tl_conserve_mass_tile
#endif
      END MODULE tl_obc_volcons_mod
